 /*********************************************************************/
 /*                                                                   */
 /* cpl.cpy - Control panel extension DLL definitions                 */
 /*                                                                   */
 /*           Version 3.10                                            */
 /*                                                                   */
 /*           Copyright 1992 - 1998 Microsoft Corp.                   */
 /*                                 All rights reserved               */
 /*                                                                   */
 /*********************************************************************/

 /*                                                                   */
 /*  General rules for being installed in the Control Panel:          */
 /*                                                                   */
 /*      1) The DLL must export a function named CPlApplet which will */
 /*         handle the messages discussed below.                      */
 /*      2) If the applet needs to save information in CONTROL.INI    */
 /*         minimize clutter by using the application name            */
 /*         [MMCPL.appletname].                                       */
 /*      2) If the applet is referenced in CONTROL.INI under [MMCPL]  */
 /*         use the following form:                                   */
 /*              ...                                                  */
 /*              [MMCPL]                                              */
 /*              uniqueName=c:\mydir\myapplet.dll                     */
 /*              ...                                                  */
 /*                                                                   */
 /*                                                                   */
 /*  The order applet DLL's are loaded by CONTROL.EXE is not          */
 /*  guaranteed. Control panels may be sorted for display, etc.       */
 /*                                                                   */

 %dcl #INC_CPL char ext;
 %if #INC_CPL = '' %then %do;
 %#INC_CPL = 'Y';

 %dcl #WINUSER_ char ext;
 %if #WINUSER_ = '' %then %do;
 %include winuser;
 %end;

 %dcl UNICODE char ext;
 %dcl WINVER fixed ext;
 %if WINVER = 0 %then %do;
 %WINVER = 1024; /* version 4.0 */
 %end; /* !WINVER */


 /*                                                                   */
 /* CONTROL.EXE will answer this message and launch an applet         */
 /*                                                                   */
 /* WM_CPL_LAUNCH                                                     */
 /*                                                                   */
 /*      wParam      - window handle of calling app                   */
 /*      lParam      - LPTSTR of name of applet to launch             */
 /*                                                                   */
 /* WM_CPL_LAUNCHED                                                   */
 /*                                                                   */
 /*      wParam      - TRUE/FALSE if applet was launched              */
 /*      lParam      - NULL                                           */
 /*                                                                   */
 /* CONTROL.EXE will post this message to the caller when the applet  */
 /* returns (ie., when wParam is a valid window handle)               */
 /*                                                                   */
 dcl WM_CPL_LAUNCH   fixed bin(31) value(WM_USER+1000);
 dcl WM_CPL_LAUNCHED fixed bin(31) value(WM_USER+1001);

 /* A function prototype for CPlApplet() */

 define alias APPLET_PROC
        entry (type HWND,
               type UINT,
               type LONG,
               type LONG)
        returns (type LONG byvalue optional)
        limited APIENTRY;

 /* The data structure CPlApplet() must fill in. */

 define structure
   1 CPLINFO,
     2 idIcon fixed bin(31),
              /* icon resource id, provided by CPlApplet() */
     2 idName fixed bin(31),
              /* name string res. id, provided by CPlApplet() */
     2 idInfo fixed bin(31),
              /* info string res. id, provided by CPlApplet() */
     2 lData  type LONG; /* user defined data */
 define alias LPCPLINFO handle CPLINFO;

 define structure
   1 NEWCPLINFOA,
     2 dwSize        type DWORD,/* similar to the commdlg */
     2 dwFlags       type DWORD,
     2 dwHelpContext type DWORD,/* help context to use */
     2 lData         type LONG, /* user defined data */
     2 hIcon         type HICON,
       /* icon to use, this is owned by CONTROL.EXE (may be deleted) */
     2 szName        char (32-1)  varz,/* short name */
     2 szInfo        char (64-1)  varz,/* long name (status line) */
     2 szHelpFile    char (128-1) varz;/* path to help file to use */
 define alias LPNEWCPLINFOA handle NEWCPLINFOA;

 define structure
   1 NEWCPLINFOW,
     2 dwSize        type DWORD,/* similar to the commdlg */
     2 dwFlags       type DWORD,
     2 dwHelpContext type DWORD,/* help context to use */
     2 lData         type LONG, /* user defined data */
     2 hIcon         type HICON,
       /* icon to use, this is owned by CONTROL.EXE (may be deleted) */
     2 szName        WCHAR (32-1)  varz,/* short name */
     2 szInfo        WCHAR (64-1)  varz,/* long name (status line) */
     2 szHelpFile    WCHAR (128-1) varz;/* path to help file to use */
 define alias LPNEWCPLINFOW handle NEWCPLINFOW;
 %if UNICODE ^= '' %then %do;
 define alias   NEWCPLINFO type   NEWCPLINFOW;
 define alias LPNEWCPLINFO type LPNEWCPLINFOW;
 %end; %else %do;
 define alias   NEWCPLINFO type   NEWCPLINFOA;
 define alias LPNEWCPLINFO type LPNEWCPLINFOA;
 %end; /* UNICODE */

 %if WINVER >= 1024 %then %do;
 dcl CPL_DYNAMIC_RES fixed bin(31) value(0);
 /* This constant may be used in place of real resource IDs for the   */
 /* idIcon,idName or idInfo members of the CPLINFO structure.         */
 /* Normally, the system uses these values to extract copies of the   */
 /* resources and store them in a cache.                              */
 /* Once the resource Information is in the cache, the system does    */
 /* not need to load a  CPL unless the user actually tries to use it. */
 /* CPL_DYNAMIC_RES tells the system not to cache the resource, but   */
 /* instead to load the CPL every time it needs to display            */
 /* information about an item. This allows a CPL to dynamically       */
 /* decide what information will be displayed, but is SIGNIFICANTLY   */
 /* SLOWER than displaying information from a cache.                  */
 /* Typically, CPL_DYNAMIC_RES is used when a control panel must      */
 /* inspect the runtime status of some device in order to provide     */
 /* text or icons to display.                                         */

 %end; /* WINVER >= 0x0400 */

 /* The messages CPlApplet() must handle: */

 dcl CPL_INIT      fixed bin(31) value(1);
 /* This message is sent to indicate CPlApplet() was found.           */
 /* lParam1 and lParam2 are not defined.                              */
 /* Return TRUE or FALSE indicating whether the control panel should  */
 /* proceed.                                                          */


 dcl CPL_GETCOUNT  fixed bin(31) value(2);
 /* This message is sent to determine the number of applets to be     */
 /* displayed.                                                        */
 /* lParam1 and lParam2 are not defined.                              */
 /* Return the number of applets you wish to display in the control   */
 /* panel window.                                                     */


 dcl CPL_INQUIRE   fixed bin(31) value(3);
 /* This message is sent for information about each applet. */

 /* A CPL SHOULD HANDLE BOTH THE CPL_INQUIRE AND CPL_NEWINQUIRE       */
 /* MESSAGES. The developer must not make any assumptions about the   */
 /* order or dependance of CPL inquiries.                             */

 /* lParam1 is the applet number to register, a value from 0 to       */
 /* (CPL_GETCOUNT - 1).  lParam2 is a far ptr to a CPLINFO structure. */
 /* Fill in CPLINFO's idIcon, idName, idInfo and lData fields with    */
 /* the resource id for an icon to display, name and description      */
 /* string ids, and a long data item associated with applet #lParam1. */
 /* This information may be cached by the caller at runtime and/or    */
 /* across sessions. To prevent caching, see CPL_DYNAMIC_RES, above.  */


 dcl CPL_SELECT     fixed bin(31) value(4);
 /* The CPL_SELECT message has been deleted.                          */


 dcl CPL_DBLCLK     fixed bin(31) value(5);
 /* This message is sent when the applet's icon has been              */
 /* double-clicked upon.                                              */
 /* lParam1 is the applet number which was selected.                  */
 /* lParam2 is the applet's lData value.                              */
 /*  This message should initiate the applet's dialog box.            */


 dcl CPL_STOP       fixed bin(31) value(6);
 /* This message is sent for each applet when the control panel is    */
 /* exiting.                                                          */
 /* lParam1 is the applet number.                                     */
 /* lParam2 is the applet's lData value.                              */
 /* Do applet specific cleaning up here.                              */


 dcl CPL_EXIT       fixed bin(31) value(7);
 /* This message is sent just before the control panel calls          */
 /* FreeLibrary.                                                      */
 /* lParam1 and lParam2 are not defined.                              */
 /* Do non-applet specific cleaning up here.                          */


 dcl CPL_NEWINQUIRE fixed bin(31) value(8);
 /* Same as CPL_INQUIRE execpt lParam2 is a pointer to a NEWCPLINFO   */
 /* struct.                                                           */

 /* A CPL SHOULD HANDLE BOTH THE CPL_INQUIRE AND CPL_NEWINQUIRE       */
 /* MESSAGES. The developer must not make any assumptions about the   */
 /* order or dependance of CPL inquiries.                             */

 %if WINVER >= 1024 %then %do;
 dcl CPL_STARTWPARMSA fixed bin(31) value(9);
 dcl CPL_STARTWPARMSW fixed bin(31) value(10);
 %if UNICODE ^= '' %then %do;
 dcl CPL_STARTWPARMS  fixed bin(31) value(CPL_STARTWPARMSW);
 %end; %else %do;
 dcl CPL_STARTWPARMS  fixed bin(31) value(CPL_STARTWPARMSA);
 %end;

 /* this message parallels CPL_DBLCLK in that the applet should       */
 /* initiate its dialog box.  where it differs is that this           */
 /* invocation is coming out of RUNDLL, and there may be some         */
 /* extra directions for execution.                                   */
 /* lParam1: the applet number.                                       */
 /* lParam2: an LPSTR to any extra directions that might exist.       */
 /* returns: TRUE if the message was handled; FALSE if not.           */
 %end; /* WINVER >= 0x0400 */


 /* This message is internal to the Control Panel and MAIN applets.   */
 /* It is only sent when an applet is invoked from the Command line   */
 /* during system installation.                                       */

 dcl CPL_SETUP fixed bin(31) value(200);

 %end; /* _INC_CPL */

