 /* ++                                                                */
 /*                                                                   */
 /* Copyright (c) 1996-1999 Microsoft Corporation                     */
 /*                                                                   */
 /* Module Name:                                                      */
 /*                                                                   */
 /*     traffic.cpy                                                   */
 /*                                                                   */
 /* Abstract:                                                         */
 /*                                                                   */
 /*     This module contains API definitions for the traffic control  */
 /*     interface.                                                    */
 /*                                                                   */
 /* --                                                                */

 %dcl #_TRAFFIC_H char ext;
 %if #_TRAFFIC_H = '' %then %do;
 %#_TRAFFIC_H = 'Y';

 %dcl #WINSOCK2API_ char ext;
 %if #WINSOCK2API_ = '' %then %do;
   %include winsock2;
 %end;


 /* following are from NtDDNdis, which haven't been translated yet */
 define structure
   1 NETWORK_ADDRESS,
     2 AddressLength type USHORT,
       /* length in bytes of Address[] in this */
     2 AddressType   type USHORT,
       /* type of this address (NDIS_PROTOCOL_ID_XXX) */
     2 Address(1)    type UCHAR;
       /* actually AddressLength bytes long */
 define alias PNETWORK_ADDRESS handle NETWORK_ADDRESS;

 define structure
   1 NETWORK_ADDRESS_LIST,
     2 AddressCount type LONG,
       /* number of addresses following */
     2 AddressType  type USHORT,
       /* type of this address (NDIS_PROTOCOL_ID_XXX) */
     2 Address(1)   type NETWORK_ADDRESS;
       /* actually AddressCount elements long */
 define alias PNETWORK_ADDRESS_LIST handle NETWORK_ADDRESS_LIST;
 /* end of declarations from NtDDNdis */


 /*------------------------------------------------------------------ */

 /* Define's */

 dcl CURRENT_TCI_VERSION       fixed bin(31) value('0002'xu);


 /* Definitions of notification events. These may be passed */
 /* to the client's notification handler, to identify the */
 /* notification type                                                 */

 /* A TC interface has come up                                        */
 dcl TC_NOTIFY_IFC_UP          fixed bin(31) value(1);

 /* A TC interface has come down                                      */
 dcl TC_NOTIFY_IFC_CLOSE       fixed bin(31) value(2);

 /* A change on a TC interface, typically a change in the             */
 /* list of supported network addresses                               */
 dcl TC_NOTIFY_IFC_CHANGE      fixed bin(31) value(3);

 /* A TC parameter has changed                                        */
 dcl TC_NOTIFY_PARAM_CHANGED   fixed bin(31) value(4);

 /* A flow has been closed by the TC interface                        */
 /* for example: after a remote call close, or the whole interface    */
 /* is going down                                                     */
 dcl TC_NOTIFY_FLOW_CLOSE      fixed bin(31) value(5);

 dcl TC_INVALID_HANDLE         type HANDLE value(sysnull());

 dcl MAX_STRING_LENGTH         fixed bin(31) value(256);


 /*------------------------------------------------------------------ */

 /* Typedef's and structures */

 /* Handlers registered by the TCI client */

 define alias TCI_NOTIFY_HANDLER
        entry (type HANDLE,
               type HANDLE,
               type ULONG,   /* See list below */
               type ULONG,
               type ULONG,
               type PVOID)
        limited CALLBACK;

 define alias TCI_ADD_FLOW_COMPLETE_HANDLER
        entry (type HANDLE,
               type ULONG)
        limited CALLBACK;

 define alias TCI_MOD_FLOW_COMPLETE_HANDLER
        entry (type HANDLE,
               type ULONG)
        limited CALLBACK;

 define alias TCI_DEL_FLOW_COMPLETE_HANDLER
        entry (type HANDLE,
               type ULONG)
        limited CALLBACK;


 define structure
   1 TCI_CLIENT_FUNC_LIST,
     2 ClNotifyHandler             type TCI_NOTIFY_HANDLER,
     2 ClAddFlowCompleteHandler    type TCI_ADD_FLOW_COMPLETE_HANDLER,
     2 ClModifyFlowCompleteHandler type TCI_MOD_FLOW_COMPLETE_HANDLER,
     2 ClDeleteFlowCompleteHandler type TCI_DEL_FLOW_COMPLETE_HANDLER;
 define alias PTCI_CLIENT_FUNC_LIST handle TCI_CLIENT_FUNC_LIST;


 /* Network address descriptor */

 define structure
   1 ADDRESS_LIST_DESCRIPTOR,
     2 MediaType   type ULONG,
     2 AddressList type NETWORK_ADDRESS_LIST;
 define alias PADDRESS_LIST_DESCRIPTOR handle ADDRESS_LIST_DESCRIPTOR;


 /* An interface ID that is returned by the enumerator */

 define structure
   1 TC_IFC_DESCRIPTOR,
     2 Length          type ULONG,
     2 pInterfaceName  type LPWSTR,
     2 pInterfaceID    type LPWSTR,
     2 AddressListDesc type ADDRESS_LIST_DESCRIPTOR;
 define alias PTC_IFC_DESCRIPTOR handle TC_IFC_DESCRIPTOR;


 /* This structure is returned by a QoS data provider in reply to */
 /* GUID_QOS_SUPPORTED query or with an interface UP notification */

 define structure
   1 TC_SUPPORTED_INFO_BUFFER,
     2 InstanceIDLength type USHORT,     /* device or interface ID */
     2 InstanceID       WCHAR (MAX_STRING_LENGTH), /* address list */
     2 AddrListDesc     type ADDRESS_LIST_DESCRIPTOR;
 define alias PTC_SUPPORTED_INFO_BUFFER handle TC_SUPPORTED_INFO_BUFFER;


 /* Filters are used to match packets. The Pattern field              */
 /* indicates the values to which bits in corresponding               */
 /* positions in candidate packets should be compared. The            */
 /* Mask field indicates which bits are to be compared and            */
 /* which bits are don't cares.                                       */
 /*                                                                   */
 /* Different filters can be submitted on the TCI interface.          */
 /* The generic filter structure is defined to include an             */
 /* AddressType, which indicates the specific type of filter to       */
 /* follow.                                                           */

 define structure
   1 TC_GEN_FILTER,
     2 AddressType type USHORT,/* IP, IPX, etc. */
     2 PatternSize type ULONG, /* byte count of the pattern */
     2 Pattern     type PVOID, /* specific format, e.g. IP_PATTERN */
     2 Mask        type PVOID; /* same type as Pattern */
 define alias PTC_GEN_FILTER handle TC_GEN_FILTER;


 /* A generic flow includes two flowspecs and a freeform */
 /* buffer which contains flow specific TC objects. */

 define structure
   1 TC_GEN_FLOW,
     2 SendingFlowspec   type FLOWSPEC,
     2 ReceivingFlowspec type FLOWSPEC,
     2 TcObjectsLength   type ULONG,/* number of optional bytes */
     2 TcObjects (1)     type QOS_OBJECT_HDR;
 define alias PTC_GEN_FLOW handle TC_GEN_FLOW;



 /* Format of specific pattern or mask used by GPC */
 /* for the IP protocol                            */

 define structure
   1 IP_PATTERN,
     2 Reserved1     type ULONG,
     2 Reserved2     type ULONG,
     2 SrcAddr       type ULONG,
     2 DstAddr       type ULONG,
     2 S_un          union,
       3 S_un_ports,
         4 s_srcport type USHORT,
         4 s_dstport type USHORT,
       3 S_un_icmp,
         4 s_type    type UCHAR,
         4 s_code    type UCHAR,
         4 filler    type USHORT,
       3 S_Spi       type ULONG,
     2 ProtocolId    type UCHAR,
     2 Reserved3 (3) type UCHAR;
 define alias PIP_PATTERN handle IP_PATTERN;

 %dcl tcSrcPort  char;
 %tcSrcPort  = 'S_un.S_un_ports.s_srcport';
 %dcl tcDstPort  char;
 %tcDstPort  = 'S_un.S_un_ports.s_dstport';
 %dcl tcIcmpType char;
 %tcIcmpType = 'S_un.S_un_icmp.s_type';
 %dcl tcIcmpCode char;
 %tcIcmpCode = 'S_un.S_un_icmp.s_code';
 %dcl tcSpi      char;
 %tcSpi      = 'S_un.S_Spi';


 /* Format of specific pattern or mask used by GPC */
 /* for the IPX protocol                           */

 define structure
   1 IPX_PATTERN,
     2 Src,
       3 NetworkAddress  type ULONG,
       3 NodeAddress (6) type UCHAR,
       3 Socket          type USHORT,
     2 Dest,
       3 NetworkAddress  type ULONG,
       3 NodeAddress (6) type UCHAR,
       3 Socket          type USHORT;
 define alias PIPX_PATTERN handle IPX_PATTERN;


 /* The enumeration buffer is the flow parameters + a list of filters */

 define structure
   1 ENUMERATION_BUFFER,
     2 Length            type ULONG,
     2 OwnerProcessId    type ULONG,
     2 FlowNameLength    type USHORT,
     2 FlowName          WCHAR (MAX_STRING_LENGTH) ,
     2 pFlow             type PTC_GEN_FLOW,
     2 NumberOfFilters   type ULONG,
     2 GenericFilter (1) type TC_GEN_FILTER; /* one for each filter */
 define alias PENUMERATION_BUFFER handle ENUMERATION_BUFFER;


 /* QoS objects supported by traffic */

 dcl QOS_TRAFFIC_GENERAL_ID_BASE type ULONG value(4000);

 dcl QOS_OBJECT_DS_CLASS         type ULONG
     value('00000001'xu + QOS_TRAFFIC_GENERAL_ID_BASE);
     /* QOS_DS_CLASS structure passed */
 dcl QOS_OBJECT_TRAFFIC_CLASS    type ULONG
     value('00000002'xu + QOS_TRAFFIC_GENERAL_ID_BASE);
     /* QOS_Traffic class structure passed */
 dcl QOS_OBJECT_DIFFSERV         type ULONG
     value('00000003'xu + QOS_TRAFFIC_GENERAL_ID_BASE);
     /* QOS_DIFFSERV Structure */
 dcl QOS_OBJECT_TCP_TRAFFIC      type ULONG
     value('00000004'xu + QOS_TRAFFIC_GENERAL_ID_BASE);
     /* QOS_TCP_TRAFFIC structure */


 /* This structure may carry an 802.1 TrafficClass parameter which   */
 /* has been provided to the host by a layer 2 network, for example, */
 /* in an 802.1 extended RSVP RESV message. If this object is        */
 /* obtained from the network, hosts will stamp the MAC headers of   */
 /* corresponding transmitted packets, with the value in the object. */
 /* Otherwise, hosts may select a value based on the standard        */
 /* Intserv mapping of ServiceType to 802.1 TrafficClass.            */

 define structure
   1 QOS_TRAFFIC_CLASS,
     2 ObjectHdr    type QOS_OBJECT_HDR,
     2 TrafficClass type ULONG;
 define alias LPQOS_TRAFFIC_CLASS handle QOS_TRAFFIC_CLASS;

 /* This structure may carry an DSField parameter which  has been     */
 /* provided to the host by a layer 3 network, for example, in an     */
 /* extended RSVP RESV message. If this object is obtained from the   */
 /* network, hosts will stamp the DS Field on the IP header of        */
 /* transmitted packets, with the value in the object. Otherwise,     */
 /* hosts may select a value based on the standard Intserv mapping    */
 /* of ServiceType to DS Field                                        */

 define structure
   1 QOS_DS_CLASS,
     2 ObjectHdr type QOS_OBJECT_HDR,
     2 DSField   type ULONG;
 define alias LPQOS_DS_CLASS handle QOS_DS_CLASS;


 /* This structure is used to create DiffServ Flows. This creates     */
 /* flows in the packet scheduler and allows it to classify to        */
 /* packets based on a particular DS field. This structure takes      */
 /* a variable length array of QOS_DIFFSERV_RULE, where each DS field */
 /* is specified by a QOS_DIFFSERV_RULE                               */

 define structure
   1 QOS_DIFFSERV,
     2 ObjectHdr    type QOS_OBJECT_HDR,
     2 DSFieldCount type ULONG,
     2 DiffservRule (1) type UCHAR;
 define alias LPQOS_DIFFSERV handle QOS_DIFFSERV;


 /* The rule for a Diffserv DS codepoint. */

 define structure
   1 QOS_DIFFSERV_RULE,
     2 InboundDSField               type UCHAR,
     2 ConformingOutboundDSField    type UCHAR,
     2 NonConformingOutboundDSField type UCHAR,
     2 ConformingUserPriority       type UCHAR,
     2 NonConformingUserPriority    type UCHAR;
 define alias LPQOS_DIFFSERV_RULE handle QOS_DIFFSERV_RULE;


 /* This structure is passed to indicate that the IP Precedence and   */
 /* User Priority mappings for the flow have to be set to the system  */
 /* defaults for TCP traffic. If this object is passed, the           */
 /* ServiceType ==> DSField mapping,                                  */
 /* ServiceType ==> UserPriorityMapping, QOS_OBJECT_DS_CLASS          */
 /* and QOS_OBJECT_TRAFFIC_CLASS will be ignored.                     */

 define structure
   1 QOS_TCP_TRAFFIC,
     2 ObjectHdr type QOS_OBJECT_HDR;
 define alias LPQOS_TCP_TRAFFIC handle QOS_TCP_TRAFFIC;


 /*------------------------------------------------------------------ */

 /* Interface Function Definitions */

 dcl TcRegisterClient
     entry (type ULONG,
            type HANDLE,
            type TCI_CLIENT_FUNC_LIST byaddr,
            type HANDLE byaddr)
     returns (type ULONG byvalue optional)
     APIENTRY external('TcRegisterClient');

 dcl TcEnumerateInterfaces
     entry (type HANDLE,
            type ULONG byaddr,
            type TC_IFC_DESCRIPTOR byaddr)
     returns (type ULONG byvalue optional)
     APIENTRY external('TcEnumerateInterfaces');

 dcl TcOpenInterfaceA
     entry (char(*) varz byaddr,
            type HANDLE,
            type HANDLE,
            type HANDLE byaddr)
     returns (type ULONG byvalue optional)
     APIENTRY external('TcOpenInterfaceA');

 dcl TcOpenInterfaceW
     entry (WCHAR(*) varz byaddr,
            type HANDLE,
            type HANDLE,
            type HANDLE byaddr)
     returns (type ULONG byvalue optional)
     APIENTRY external('TcOpenInterfaceW');

 dcl TcCloseInterface
     entry (type HANDLE)
     returns (type ULONG byvalue optional)
     APIENTRY external('TcCloseInterface');

 dcl TcQueryInterface
     entry (type HANDLE,
            type GUID byaddr,
            type BOOLEAN,
            type ULONG byaddr,
            type PVOID)
     returns (type ULONG byvalue optional)
     APIENTRY external('TcQueryInterface');

 dcl TcSetInterface
     entry (type HANDLE,
            type GUID byaddr,
            type ULONG,
            type PVOID)
     returns (type ULONG byvalue optional)
     APIENTRY external('TcSetInterface');

 dcl TcQueryFlowA
     entry (char(*) varz byaddr,
            type GUID byaddr,
            type ULONG byaddr,
            type PVOID)
     returns (type ULONG byvalue optional)
     APIENTRY external('TcQueryFlowA');

 dcl TcQueryFlowW
     entry (WCHAR(*) varz byaddr,
            type GUID byaddr,
            type ULONG byaddr,
            type PVOID)
     returns (type ULONG byvalue optional)
     APIENTRY external('TcQueryFlowW');

 dcl TcSetFlowA
     entry (char(*) varz byaddr,
            type GUID byaddr,
            type ULONG,
            type PVOID)
     returns (type ULONG byvalue optional)
     APIENTRY external('TcSetFlowA');

 dcl TcSetFlowW
     entry (WCHAR(*) varz byaddr,
            type GUID byaddr,
            type ULONG,
            type PVOID)
     returns (type ULONG byvalue optional)
     APIENTRY external('TcSetFlowW');

 dcl TcAddFlow
     entry (type HANDLE,
            type HANDLE,
            type ULONG,
            type TC_GEN_FLOW byaddr,
            type HANDLE byaddr)
     returns (type ULONG byvalue optional)
     APIENTRY external('TcAddFlow');

 dcl TcGetFlowNameA
     entry (type HANDLE,
            type ULONG,
            char(*) varz byaddr)
     returns (type ULONG byvalue optional)
     APIENTRY external('TcGetFlowNameA');

 dcl TcGetFlowNameW
     entry (type HANDLE,
            type ULONG,
            WCHAR(*) varz byaddr)
     returns (type ULONG byvalue optional)
     APIENTRY external('TcGetFlowNameW');

 dcl TcModifyFlow
     entry (type HANDLE,
            type TC_GEN_FLOW byaddr)
     returns (type ULONG byvalue optional)
     APIENTRY external('TcModifyFlow');

 dcl TcAddFilter
     entry (type HANDLE,
            type TC_GEN_FILTER byaddr,
            type HANDLE byaddr)
     returns (type ULONG byvalue optional)
     APIENTRY external('TcAddFilter');


 dcl TcDeregisterClient
     entry (type HANDLE)
     returns (type ULONG byvalue optional)
     APIENTRY external('TcDeregisterClient');


 dcl TcDeleteFlow
     entry (type HANDLE)
     returns (type ULONG byvalue optional)
     APIENTRY external('TcDeleteFlow');

 dcl TcDeleteFilter
     entry (type HANDLE)
     returns (type ULONG byvalue optional)
     APIENTRY external('TcDeleteFilter');

 dcl TcEnumerateFlows
     entry (type HANDLE,
            type HANDLE byaddr,
            type ULONG byaddr,
            type ULONG byaddr,
            type ENUMERATION_BUFFER byaddr)
     returns (type ULONG byvalue optional)
     APIENTRY external('TcEnumerateFlows');

 %dcl TcOpenInterface char ext;
 %dcl TcQueryFlow char ext;
 %dcl TcSetFlow char ext;
 %dcl TcGetFlowName char ext;

 %dcl UNICODE char ext;
 %if UNICODE ^= '' %then %do;
   %TcOpenInterface='TcOpenInterfaceW';
   %TcQueryFlow='TcQueryFlowW';
   %TcSetFlow='TcSetFlowW';
   %TcGetFlowName='TcGetFlowNameW';
 %end; %else %do;
   %TcOpenInterface='TcOpenInterfaceA';
   %TcQueryFlow='TcQueryFlowA';
   %TcSetFlow='TcSetFlowA';
   %TcGetFlowName='TcGetFlowNameA';
 %end; /* UNICODE */

 %end;