 /*++                                                                 */
 /*                                                                   */
 /* Copyright (C) 1993-1999 Microsoft Corporation                     */
 /*                                                                   */
 /* Module Name:                                                      */
 /*                                                                   */
 /*   winperf.cpy                                                     */
 /*                                                                   */
 /* Abstract:                                                         */
 /*                                                                   */
 /*   Header file for the Performance Monitor data.                   */
 /*                                                                   */
 /*   This file contains the definitions of the data structures       */
 /*   returned by the Configuration Registry in response to a request */
 /*   for performance data.  This file is used by both the            */
 /*   Configuration Registry and the Performance Monitor to define    */
 /*   their interface. The complete interface is described here,      */
 /*   except for the name of the node to query in the registry. It is */
 /*                                                                   */
 /*                  HKEY_PERFORMANCE_DATA.                           */
 /*                                                                   */
 /*   By querying that node with a subkey of "Global" the caller will */
 /*   retrieve the structures described here.                         */
 /*                                                                   */
 /*   There is no need to RegOpenKey() the reserved handle            */
 /*   HKEY_PERFORMANCE_DATA, but the caller should RegCloseKey() the  */
 /*   handle so that network transports and drivers can be removed or */
 /*   installed (which cannot happen while they are open for          */
 /*   monitoring.)  Remote requests must first RegConnectRegistry().  */
 /*                                                                   */
 /*--                                                                 */

 %dcl #WINPERF_ char ext;
 %if #WINPERF_ = '' %then %do;
 %#WINPERF_ = 'Y';

 /*  Data structure definitions. */

 /*  In order for data to be returned through the Configuration       */
 /*  Registry in a system-independent fashion, it must be             */
 /*  self-describing.                                                 */
 /*                                                                   */
 /*  In the following, all offsets are in bytes.                      */
 /*                                                                   */
 /*  Data is returned through the Configuration Registry in a         */
 /*  a data block which begins with a PERF_DATA_BLOCK structure.      */
 /*                                                                   */

 dcl PERF_DATA_VERSION  fixed bin(31) value(1);
 dcl PERF_DATA_REVISION fixed bin(31) value(1);


 define structure
   1 PERF_DATA_BLOCK,
     2 Signature        WCHAR(4),  /* Signature: Unicode "PERF"       */
     2 LittleEndian     type DWORD,/* 0= Big Endian, 1= Little Endian */
     2 Version          type DWORD,/* Version of these data structures*/
                                   /* starting at 1                   */
     2 Revision         type DWORD,/*Revision of these data structures*/
                                   /* starting at 0 for each Version  */
     2 TotalByteLength  type DWORD,/* Total length of data block      */
     2 HeaderLength     type DWORD,/* Length of this structure        */
     2 NumObjectTypes   type DWORD,/* Number of types of objects      */
                                   /* being reported                  */
     2 DefaultObject    type LONG, /* Object Title Index of default   */
                                   /* object to display when data from*/
                                   /* this system is retrieved (-1 =  */
                                   /* none, but this is not expected  */
                                   /* to be used)                     */
     2 SystemTime  type SYSTEMTIME,/* Time at the system under        */
                                   /* measurement                     */
     2 PerfTime    type LARGE_INTEGER,/* Performance counter value    */
                                   /* at the system under measurement */
     2 PerfFreq    type LARGE_INTEGER,/* Performance counter frequency*/
                                   /* at the system under measurement */
     2 PerfTime100nSec  type LARGE_INTEGER,/* Performance counter time*/
                                   /* in 100 nsec units               */
                                   /* at the system under measurement */
     2 SystemNameLength type DWORD,/* Length of the system name       */
     2 SystemNameOffset type DWORD;/* Offset, from beginning of this  */
                                   /* structure, to name of system    */
                                   /* being measured                  */

 define alias PPERF_DATA_BLOCK handle PERF_DATA_BLOCK;


 /* The PERF_DATA_BLOCK structure is followed by NumObjectTypes of    */
 /* data sections, one for each type of object measured. Each object  */
 /* type section begins with a _PERF_OBJECT_TYPE structure.           */


 define structure
   1 PERF_OBJECT_TYPE,
     2 TotalByteLength  type DWORD,/* Length of this object definition*/
                                   /* including this structure, the   */
                                   /* counter definitions, and the    */
                                   /* instance definitions and the    */
                                   /* counter blocks for each instance*/
                                   /* This is the offset from this    */
                                   /* structure to the next object,   */
                                   /* if any                          */
     2 DefinitionLength type DWORD,/* Length of object definition,    */
                                   /* which includes this structure   */
                                   /* and the counter definition      */
                                   /* structures for this object: this*/
                                   /* is the offset of the first      */
                                   /* instance or of the counters     */
                                   /* for this object if there is     */
                                   /* no instance                     */
     2 HeaderLength     type DWORD,/* Length of this structure: this  */
                                   /* is the offset to the first      */
                                   /* counter definition for this     */
                                   /* object                          */
     2 ObjectNameTitleIndex type DWORD,
                                   /* Index to name in Title Database */
     2 ObjectNameTitle  type LPWSTR,/* Initially NULL, for use by     */
                                   /* analysis program to point to    */
                                   /* retrieved title string          */
     2 ObjectHelpTitleIndex type DWORD,
                                   /* Index to Help in Title Database */
     2 ObjectHelpTitle  type LPWSTR,/* Initially NULL, for use by     */
                                   /* analysis program to point to    */
                                   /* retrieved title string          */
     2 DetailLevel      type DWORD,/* Object level of detail (for     */
                                   /* controlling display complexity),*/
                                   /* will be min of detail levels    */
                                   /* for all this object's counters  */
     2 NumCounters      type DWORD,/* Number of counters in each      */
                                   /* counter block (one counter      */
                                   /* block per instance)             */
     2 DefaultCounter   type LONG, /* Default counter to display when */
                                   /* this object is selected, index  */
                                   /* starting at 0 (-1 = none, but   */
                                   /* this is not expected to be used)*/
     2 NumInstances     type LONG, /* Number of object instances      */
                             /* for which counters are being          */
                             /* returned from the system under        */
                             /* measurement. If the object defined    */
                             /* will never have any instance data     */
                             /* structures (PERF_INSTANCE_DEFINITION) */
                             /* then this value should be -1, if the  */
                             /* object can have 0 or more instances,  */
                             /* but has none present, then this       */
                             /* should be 0, otherwise this field     */
                             /* contains the number of instances of   */
                             /* this counter.                         */
     2 CodePage         type DWORD,/* 0 if instance strings are in    */
                                   /* UNICODE, else the Code Page of  */
                                   /* the instance names              */
     2 PerfTime type LARGE_INTEGER,/* Sample Time in "Object" units   */
     2 PerfFreq type LARGE_INTEGER;/* Frequency of "Object" units in  */
                                   /* counts per second.              */

 define alias PPERF_OBJECT_TYPE handle PERF_OBJECT_TYPE;

 dcl PERF_NO_INSTANCES fixed bin (31) value (-1);
     /* no instances (see NumInstances above) */

 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 /*                                                                   */
 /*  PERF_COUNTER_DEFINITION.CounterType field values                 */
 /*                                                                   */
 /*                                                                   */
 /*        Counter ID Field Definition:                               */
 /*                                                                   */
 /* 3      2        2    2    2        1        1    1                */
 /* 1      8        4    2    0        6        2    0    8 7        0*/
 /*+--------+-------+----+----+--------+--------+----+----+----------+*/
 /*|Display |Calculation |Time|Counter |        |Ctr |Size|          |*/
 /*|Flags   |Modifiers   |Base|SubType |Reserved|Type|Fld | Reserved |*/
 /*+--------+-------+----+----+--------+--------+----+----+----------+*/
 /*                                                                   */
 /*                                                                   */
 /* The counter type is the "or" of the following values as described */
 /* below                                                             */

 /* select one of the following to indicate the counter's data size   */

 dcl PERF_SIZE_DWORD                     type DWORD value('00000000'xu);
 dcl PERF_SIZE_LARGE                     type DWORD value('00000100'xu);
 dcl PERF_SIZE_ZERO                      type DWORD value('00000200'xu);
     /* for Zero Length fields */
 dcl PERF_SIZE_VARIABLE_LEN              type DWORD value('00000300'xu);
     /* length is in CounterLength field of Counter Definition struct */

 /* select one of the following values to indicate the counter field  */
 /* usage                                                             */

 dcl PERF_TYPE_NUMBER                    type DWORD value('00000000'xu);
     /* a number (not a counter) */
 dcl PERF_TYPE_COUNTER                   type DWORD value('00000400'xu);
     /* an increasing numeric value */
 dcl PERF_TYPE_TEXT                      type DWORD value('00000800'xu);
     /* a text field */
 dcl PERF_TYPE_ZERO                      type DWORD value('00000C00'xu);
     /* displays a zero */

 /* If the PERF_TYPE_NUMBER field was selected, then select one of    */
 /* the following to describe the Number                              */

 dcl PERF_NUMBER_HEX                     type DWORD value('00000000'xu);
     /* display as HEX value */
 dcl PERF_NUMBER_DECIMAL                 type DWORD value('00010000'xu);
     /* display as a decimal integer */
 dcl PERF_NUMBER_DEC_1000                type DWORD value('00020000'xu);
     /* display as a decimal/1000 */

 /* If the PERF_TYPE_COUNTER value was selected then select one of    */
 /* the following to indicate the type of counter                     */

 dcl PERF_COUNTER_VALUE                  type DWORD value('00000000'xu);
     /* display counter value */
 dcl PERF_COUNTER_RATE                   type DWORD value('00010000'xu);
     /* divide ctr / delta time */
 dcl PERF_COUNTER_FRACTION               type DWORD value('00020000'xu);
     /* divide ctr / base */
 dcl PERF_COUNTER_BASE                   type DWORD value('00030000'xu);
     /* base value used in fractions */
 dcl PERF_COUNTER_ELAPSED                type DWORD value('00040000'xu);
     /* subtract counter from current time */
 dcl PERF_COUNTER_QUEUELEN               type DWORD value('00050000'xu);
     /* Use Queuelen processing func. */
 dcl PERF_COUNTER_HISTOGRAM              type DWORD value('00060000'xu);
     /* Counter begins or ends a histogram */
 dcl PERF_COUNTER_PRECISION              type DWORD value('00070000'xu);
     /* divide ctr / private clock */

 /* If the PERF_TYPE_TEXT value was selected, then select one of      */
 /* the following to indicate the type of TEXT data.                  */

 dcl PERF_TEXT_UNICODE                   type DWORD value('00000000'xu);
     /* type of text in text field */
 dcl PERF_TEXT_ASCII                     type DWORD value('00010000'xu);
     /* ASCII using the CodePage field */

 /* Timer SubTypes */

 dcl PERF_TIMER_TICK                     type DWORD value('00000000'xu);
     /* use system perf. freq for base */
 dcl PERF_TIMER_100NS                    type DWORD value('00100000'xu);
     /* use 100 NS timer time base units */
 dcl PERF_OBJECT_TIMER                   type DWORD value('00200000'xu);
     /* use the object timer freq */

 /* Any types that have calculations performed can use one or more */
 /* of the following calculation modification flags listed here    */
 dcl PERF_DELTA_COUNTER                  type DWORD value('00400000'xu);
     /* compute difference first */
 dcl PERF_DELTA_BASE                     type DWORD value('00800000'xu);
     /* compute base diff as well */
 dcl PERF_INVERSE_COUNTER                type DWORD value('01000000'xu);
     /* show as 1.00-value (assumes: */
 dcl PERF_MULTI_COUNTER                  type DWORD value('02000000'xu);
     /* sum of multiple instances */

 /* Select one of the following values to indicate the display suffix */
 /* (if any) */

 dcl PERF_DISPLAY_NO_SUFFIX              type DWORD value('00000000'xu);
     /* no suffix */
 dcl PERF_DISPLAY_PER_SEC                type DWORD value('10000000'xu);
     /* "/sec" */
 dcl PERF_DISPLAY_PERCENT                type DWORD value('20000000'xu);
     /* "%" */
 dcl PERF_DISPLAY_SECONDS                type DWORD value('30000000'xu);
     /* "secs" */
 dcl PERF_DISPLAY_NOSHOW                 type DWORD value('40000000'xu);
     /* value is not displayed */

 /* Predefined counter types */

 /* 32-bit Counter. Divide delta by delta time. Display suffix: "/sec"*/

 dcl PERF_COUNTER_COUNTER
     type DWORD value (ior (PERF_SIZE_DWORD,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_RATE,
                            PERF_TIMER_TICK,
                            PERF_DELTA_COUNTER,
                            PERF_DISPLAY_PER_SEC));

 /* 64-bit Timer. Divide delta by delta time. Display suffix: "%"     */
 dcl PERF_COUNTER_TIMER
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_RATE,
                            PERF_TIMER_TICK,
                            PERF_DELTA_COUNTER,
                            PERF_DISPLAY_PERCENT));

 /* Queue Length Space-Time Product. Divide delta by delta time.      */
 /* No Display Suffix.                                                */
 dcl PERF_COUNTER_QUEUELEN_TYPE
     type DWORD value (ior (PERF_SIZE_DWORD,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_QUEUELEN,
                            PERF_TIMER_TICK,
                            PERF_DELTA_COUNTER,
                            PERF_DISPLAY_NO_SUFFIX));

 /* Queue Length Space-Time Product. Divide delta by delta time.      */
 /* No Display Suffix.                                                */
 dcl PERF_COUNTER_LARGE_QUEUELEN_TYPE
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_QUEUELEN,
                            PERF_TIMER_TICK,
                            PERF_DELTA_COUNTER,
                            PERF_DISPLAY_NO_SUFFIX));

 /* Queue Length Space-Time Product using 100 Ns timebase.            */
 /* Divide delta by delta time. No Display Suffix.                    */
 dcl PERF_COUNTER_100NS_QUEUELEN_TYPE
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_QUEUELEN,
                            PERF_TIMER_100NS,
                            PERF_DELTA_COUNTER,
                            PERF_DISPLAY_NO_SUFFIX));

 /* Queue Length Space-Time Product using Object specific timebase.   */
 /* Divide delta by delta time. No Display Suffix.                    */
 dcl PERF_COUNTER_OBJ_TIME_QUEUELEN_TYPE
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_QUEUELEN,
                            PERF_OBJECT_TIMER,
                            PERF_DELTA_COUNTER,
                            PERF_DISPLAY_NO_SUFFIX));

 /* 64-bit Counter. Divide delta by delta time. Display Suffix: "/sec"*/
 dcl PERF_COUNTER_BULK_COUNT
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_RATE,
                            PERF_TIMER_TICK,
                            PERF_DELTA_COUNTER,
                            PERF_DISPLAY_PER_SEC));

 /* Indicates the counter is not a  counter but rather Unicode text.  */
 /* Display as text.                                                  */
 dcl PERF_COUNTER_TEXT
     type DWORD value (ior (PERF_SIZE_VARIABLE_LEN,
                            PERF_TYPE_TEXT,
                            PERF_TEXT_UNICODE,
                            PERF_DISPLAY_NO_SUFFIX));

 /* Indicates the data is a counter  which should not be time         */
 /* averaged on display (such as an error counter on a serial line)   */
 /* Display as is. No Display Suffix.                                 */
 dcl PERF_COUNTER_RAWCOUNT
     type DWORD value (ior (PERF_SIZE_DWORD,
                            PERF_TYPE_NUMBER,
                            PERF_NUMBER_DECIMAL,
                            PERF_DISPLAY_NO_SUFFIX));

 /* Same as PERF_COUNTER_RAWCOUNT except its size is a large integer  */
 dcl PERF_COUNTER_LARGE_RAWCOUNT
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_NUMBER,
                            PERF_NUMBER_DECIMAL,
                            PERF_DISPLAY_NO_SUFFIX));

 /* Special case for RAWCOUNT that want to be displayed in hex        */
 /* Indicates the data is a counter  which should not be time         */
 /* averaged on display (such as an error counter on a serial line)   */
 /* Display as is.  No Display Suffix. */
 dcl PERF_COUNTER_RAWCOUNT_HEX
     type DWORD value (ior (PERF_SIZE_DWORD,
                            PERF_TYPE_NUMBER,
                            PERF_NUMBER_HEX,
                            PERF_DISPLAY_NO_SUFFIX));

 /* Same as PERF_COUNTER_RAWCOUNT_HEX except its size is a large      */
 /* integer                                                           */
 dcl PERF_COUNTER_LARGE_RAWCOUNT_HEX
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_NUMBER,
                            PERF_NUMBER_HEX,
                            PERF_DISPLAY_NO_SUFFIX));


 /* A count which is either 1 or 0 on each sampling interrupt (% busy)*/
 /* Divide delta by delta base. Display Suffix: "%"                   */
 dcl PERF_SAMPLE_FRACTION
     type DWORD value (ior (PERF_SIZE_DWORD,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_FRACTION,
                            PERF_DELTA_COUNTER,
                            PERF_DELTA_BASE,
                            PERF_DISPLAY_PERCENT));

 /* A count which is sampled on each sampling interrupt (queue length)*/
 /* Divide delta by delta time. No Display Suffix.                    */
 dcl PERF_SAMPLE_COUNTER
     type DWORD value (ior (PERF_SIZE_DWORD,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_RATE,
                            PERF_TIMER_TICK,
                            PERF_DELTA_COUNTER,
                            PERF_DISPLAY_NO_SUFFIX));

 /* A label: no data is associated with this counter (it has 0 length)*/
 /* Do not display.                                                   */
 dcl PERF_COUNTER_NODATA
     type DWORD value (ior (PERF_SIZE_ZERO,
                            PERF_DISPLAY_NOSHOW));

 /* 64-bit Timer inverse (e.g., idle is measured, but display busy %) */
 /* Display 100 - delta divided by delta time.  Display suffix: "%"   */
 dcl PERF_COUNTER_TIMER_INV
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_RATE,
                            PERF_TIMER_TICK,
                            PERF_DELTA_COUNTER,
                            PERF_INVERSE_COUNTER,
                            PERF_DISPLAY_PERCENT));

 /* The divisor for a sample, used with the previous counter to form  */
 /* a sampled %. You must check for >0 before dividing by this! This  */
 /* counter will directly follow the  numerator counter. It should    */
 /* not be displayed to the user.                                     */
 dcl PERF_SAMPLE_BASE
     type DWORD value (ior (PERF_SIZE_DWORD,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_BASE,
                            PERF_DISPLAY_NOSHOW,
                                '00000001'xu));
                          /* for compatibility with pre-beta versions */


 /* A timer which, when divided by an average base, produces a time   */
 /* in seconds which is the average time of some operation.           */
 /* This timer times total operations, and the base is the number of  */
 /* operations. Display Suffix: "sec"                                 */
 dcl PERF_AVERAGE_TIMER
     type DWORD value (ior (PERF_SIZE_DWORD,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_FRACTION,
                            PERF_DISPLAY_SECONDS));

 /* Used as the denominator in the computation of time or count       */
 /* averages.  Must directly follow the numerator counter.            */
 /* Not displayed to the user.                                        */
 dcl PERF_AVERAGE_BASE
     type DWORD value (ior (PERF_SIZE_DWORD,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_BASE,
                            PERF_DISPLAY_NOSHOW,
                                '00000002'xu));
                          /* for compatibility with pre-beta versions */

 /* A bulk count which, when divided (typically) by the number of     */
 /* operations, gives (typically) the number of bytes per operation.  */
 /* No Display Suffix.                                                */
 dcl PERF_AVERAGE_BULK
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_FRACTION,
                            PERF_DISPLAY_NOSHOW));

 /* 64-bit Timer in object specific units. Display delta divided by   */
 /* delta time as returned in the object type header structure.       */
 /* Display suffix: "%"                                               */
 dcl PERF_OBJ_TIME_TIMER	
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_RATE,
                            PERF_OBJECT_TIMER,
                            PERF_DELTA_COUNTER,
                            PERF_DISPLAY_PERCENT));

 /* 64-bit Timer in 100 nsec units. Display delta divided by          */
 /* delta time. Display suffix: "%"                                   */
 dcl PERF_100NSEC_TIMER
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_RATE,
                            PERF_TIMER_100NS,
                            PERF_DELTA_COUNTER,
                            PERF_DISPLAY_PERCENT));

 /* 64-bit Timer inverse (e.g., idle is measured, but display busy %) */
 /* Display 100 - delta divided by delta time. Display suffix: "%"    */
 dcl PERF_100NSEC_TIMER_INV
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_RATE,
                            PERF_TIMER_100NS,
                            PERF_DELTA_COUNTER,
                            PERF_INVERSE_COUNTER,
                            PERF_DISPLAY_PERCENT));

 /* 64-bit Timer.  Divide delta by delta time. Display suffix: "%"    */
 /* Timer for multiple instances, so result can exceed 100%.          */
 dcl PERF_COUNTER_MULTI_TIMER
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_RATE,
                            PERF_DELTA_COUNTER,
                            PERF_TIMER_TICK,
                            PERF_MULTI_COUNTER,
                            PERF_DISPLAY_PERCENT));

 /* 64-bit Timer inverse (e.g., idle is measured, but display busy %) */
 /* Display 100 * _MULTI_BASE - delta divided by delta time.          */
 /* Display suffix: "%" Timer for multiple instances, so result       */
 /* can exceed 100%. Followed by a counter of type _MULTI_BASE.       */
 dcl PERF_COUNTER_MULTI_TIMER_INV
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_RATE,
                            PERF_DELTA_COUNTER,
                            PERF_MULTI_COUNTER,
                            PERF_TIMER_TICK,
                            PERF_INVERSE_COUNTER,
                            PERF_DISPLAY_PERCENT));

 /* Number of instances to which the preceding _MULTI_..._INV counter */
 /* applies. Used as a factor to get the percentage.                  */
 dcl PERF_COUNTER_MULTI_BASE
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_BASE,
                            PERF_MULTI_COUNTER,
                            PERF_DISPLAY_NOSHOW));

 /* 64-bit Timer in 100 nsec units. Display delta divided by delta    */
 /* time. Display suffix: "%" Timer for multiple instances, so result */
 /* can exceed 100%.                                                  */
 dcl PERF_100NSEC_MULTI_TIMER
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_COUNTER,
                            PERF_DELTA_COUNTER ,
                            PERF_COUNTER_RATE,
                            PERF_TIMER_100NS,
                            PERF_MULTI_COUNTER,
                            PERF_DISPLAY_PERCENT));

 /* 64-bit Timer inverse (e.g., idle is measured, but display busy %) */
 /* Display 100 * _MULTI_BASE - delta divided by delta time.          */
 /* Display suffix: "%" Timer for multiple instances, so result       */
 /* can exceed 100%. Followed by a counter of type _MULTI_BASE.       */
 dcl PERF_100NSEC_MULTI_TIMER_INV
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_COUNTER,
                            PERF_DELTA_COUNTER ,
                            PERF_COUNTER_RATE,
                            PERF_TIMER_100NS,
                            PERF_MULTI_COUNTER,
                            PERF_INVERSE_COUNTER,
                            PERF_DISPLAY_PERCENT));

 /* Indicates the data is a fraction of the following counter  which  */
 /* should not be time averaged on display (such as free space over   */
 /* total space.) Display as is. Display the quotient as "%".         */
 dcl PERF_RAW_FRACTION
     type DWORD value (ior (PERF_SIZE_DWORD,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_FRACTION,
                            PERF_DISPLAY_PERCENT));

 dcl PERF_LARGE_RAW_FRACTION
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_FRACTION,
                            PERF_DISPLAY_PERCENT));

 /* Indicates the data is a base for the preceding counter which      */
 /* should not be time averaged on display (such as free space over   */
 /* total space.)                                                     */
 dcl PERF_RAW_BASE
     type DWORD value (ior (PERF_SIZE_DWORD,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_BASE,
                            PERF_DISPLAY_NOSHOW,
                                '00000003'xu));
                          /* for compatibility with pre-beta versions */

 dcl PERF_LARGE_RAW_BASE
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_BASE,
                            PERF_DISPLAY_NOSHOW));

 /* The data collected in this counter is actually the start time of  */
 /* the item being measured. For display, this data is subtracted     */
 /* from the sample time to yield the elapsed time as the difference  */
 /* between the two. In the definition below, the PerfTime field of   */
 /* the Object contains the sample time as indicated by the           */
 /* PERF_OBJECT_TIMER bit and the  difference is scaled by the        */
 /* PerfFreq of the Object to convert the time units into seconds.    */
 dcl PERF_ELAPSED_TIME
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_ELAPSED,
                            PERF_OBJECT_TIMER,
                            PERF_DISPLAY_SECONDS));

 /* The following counter type can be used with the preceding types   */
 /* to define a range of values to be displayed in a histogram.       */

 dcl PERF_COUNTER_HISTOGRAM_TYPE         type DWORD value('80000000'xu);
                               /* Counter begins or ends a histogram  */

 /*  This counter is used to display the difference from one sample   */
 /*  to the next. The counter value is a constantly increasing number */
 /*  and the value displayed is the difference between the current    */
 /*  value and the previous value. Negative numbers are not allowed   */
 /*  which shouldn't be a problem as long as the counter value is     */
 /*  increasing or unchanged.                                         */
 dcl PERF_COUNTER_DELTA
     type DWORD value (ior (PERF_SIZE_DWORD,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_VALUE,
                            PERF_DELTA_COUNTER,
                            PERF_DISPLAY_NO_SUFFIX));

 dcl PERF_COUNTER_LARGE_DELTA
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_VALUE,
                            PERF_DELTA_COUNTER,
                            PERF_DISPLAY_NO_SUFFIX));

 /* The precision counters are timers that consist of two counter     */
 /* values:                                                           */
 /*     1) the count of elapsed time of the event being monitored     */
 /*     2) the "clock" time in the same units                         */
 /*                                                                   */
 /* The precision timers are used where the standard system timers    */
 /* are not precise enough for accurate readings. It's assumed that   */
 /* the service providing the data is also providing a timestamp at   */
 /* the same time which will eliminate any error that may occur since */
 /* some small and variable time elapses between the time the system  */
 /* timestamp is captured and when the data is collected from the     */
 /* performance DLL. Only in extreme cases has this been observed to  */
 /* be problematic.                                                   */
 /*                                                                   */
 /* When using this type of timer, the definition of the              */
 /*     PERF_PRECISION_TIMESTAMP counter must immediately follow the  */
 /*     definition of the PERF_PRECISION_*_TIMER in the Object header */

 /* The timer used has the same frequency as the System Performance   */
 /* Timer                                                             */
 dcl PERF_PRECISION_SYSTEM_TIMER
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_PRECISION,
                            PERF_TIMER_TICK,
                            PERF_DELTA_COUNTER,
                            PERF_DISPLAY_PERCENT));

 /* The timer used has the same frequency as the 100 NanoSecond Timer */
 dcl PERF_PRECISION_100NS_TIMER
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_PRECISION,
                            PERF_TIMER_100NS,
                            PERF_DELTA_COUNTER,
                            PERF_DISPLAY_PERCENT));

 /* The timer used is of the frequency specified in the Object        */
 /* header's PerfFreq field (PerfTime is ignored)                     */
 dcl PERF_PRECISION_OBJECT_TIMER
     type DWORD value (ior (PERF_SIZE_LARGE,
                            PERF_TYPE_COUNTER,
                            PERF_COUNTER_PRECISION,
                            PERF_OBJECT_TIMER,
                            PERF_DELTA_COUNTER,
                            PERF_DISPLAY_PERCENT));

 /* This is the timestamp to use in the computation of the timer      */
 /* specified in the previous description block                       */
 dcl PERF_PRECISION_TIMESTAMP type DWORD value(PERF_LARGE_RAW_BASE);

 /* The following are used to determine the level of detail associated*/
 /* with the counter. The user will be setting the level of detail    */
 /* that should be displayed at any given time.                       */

 dcl PERF_DETAIL_NOVICE   fixed bin(31) value(100);
    /* The uninformed can understand it */
 dcl PERF_DETAIL_ADVANCED fixed bin(31) value(200);
    /* For the advanced user */
 dcl PERF_DETAIL_EXPERT   fixed bin(31) value(300);
    /* For the expert user */
 dcl PERF_DETAIL_WIZARD   fixed bin(31) value(400);
    /* For the system designer */


 /* There is one of the following for each of the                     */
 /* PERF_OBJECT_TYPE.NumCounters. The Unicode names in this structure */
 /* MUST come from a message file.                                    */

 define structure
   1 PERF_COUNTER_DEFINITION,    /* For the system designer           */
     2 ByteLength     type DWORD,/* Length in bytes of this structure */
     2 CounterNameTitleIndex type DWORD,/* Index of Counter name into */
                                        /* Title Database             */
     2 CounterNameTitle type LPWSTR,/* Initially NULL, for use by     */
                                    /* analysis program to point to   */
                                    /* retrieved title string         */
     2 CounterHelpTitleIndex type DWORD,/* Index of Counter Help into */
                                        /* Title Database             */
     2 CounterHelpTitle type LPWSTR,/* Initially NULL, for use by     */
                                    /* analysis program to point to   */
                                    /* retrieved title string         */
     2 DefaultScale     type LONG, /* Power of 10 by which to scale   */
                               /* chart line if vertical axis is 100  */
                               /* 0 ==> 1, 1 ==> 10, -1 ==>1/10, etc. */
     2 DetailLevel      type DWORD,/* Counter level of detail (for    */
                                   /* controlling display complexity) */
     2 CounterType      type DWORD,/* Type of counter                 */
     2 CounterSize      type DWORD,/* Size of counter in bytes        */
     2 CounterOffset    type DWORD;/* Offset from the start of the    */
                                   /* PERF_COUNTER_BLOCK to the first */
                                   /* byte of this counter */

 define alias PPERF_COUNTER_DEFINITION handle PERF_COUNTER_DEFINITION;


 /* If (PERF_DATA_BLOCK.NumInstances >= 0) then there will be         */
 /* PERF_DATA_BLOCK.NumInstances of a (PERF_INSTANCE_DEFINITION       */
 /* followed by a PERF_COUNTER_BLOCK followed by the counter data     */
 /* fields) for each instance.                                        */
 /*                                                                   */
 /* If (PERF_DATA_BLOCK.NumInstances < 0) then the counter definition */
 /* structure above will be followed by only a PERF_COUNTER_BLOCK and */
 /* the counter data for that COUNTER.                                */

 dcl PERF_NO_UNIQUE_ID type DWORD value (iunsigned(bin(-1,31)));

 define structure
   1 PERF_INSTANCE_DEFINITION,
     2 ByteLength   type DWORD, /* Length in bytes of this structure, */
                                /* including the subsequent name      */
     2 ParentObjectTitleIndex type DWORD,
                                /* Title Index to name of "parent"    */
                                /* object (e.g., if thread, then      */
                                /* process is parent object type);    */
                                /* if logical drive, the physical     */
                                /* drive is parent object type        */
     2 ParentObjectInstance   type DWORD,
                                /* Index to instance of parent object */
                                /* type which is the parent of this   */
                                /* instance.                          */
     2 UniqueID     type LONG,  /* A unique ID used instead of        */
                                /* matching the name to identify      */
                                /* this instance, -1 = none           */
     2 NameOffset   type DWORD, /* Offset from beginning of           */
                                /* this struct to the Unicode name    */
                                /* of this instance                   */
     2 NameLength   type DWORD; /* Length in bytes of name; 0 = none  */
                                /* this length includes the characters*/
                                /* in the string plus the size of the */
                                /* terminating NULL char. It does not */
                                /* include any additional pad bytes to*/
                                /* correct structure alignment        */

 define alias PPERF_INSTANCE_DEFINITION handle PERF_INSTANCE_DEFINITION;

 /* If .ParentObjectName is 0, there                                  */
 /* is no parent-child hierarchy for this object type. Otherwise,     */
 /* the .ParentObjectInstance is an index, starting at 0, into the    */
 /* instances reported for the parent object type.  It is only        */
 /* meaningful if .ParentObjectName is not 0. The purpose of all this */
 /* is to permit reporting/summation of object instances like threads */
 /* within processes, and logical drives within physical drives.      */
 /*                                                                   */
 /* The PERF_INSTANCE_DEFINITION will be followed by a                */
 /* PERF_COUNTER_BLOCK.                                               */


 define structure
   1 PERF_COUNTER_BLOCK,
     2 ByteLength type DWORD; /* Length in bytes of this structure, */
                              /* including the following counters   */
 define alias PPERF_COUNTER_BLOCK handle PERF_COUNTER_BLOCK;

 /*                                                   */
 /*  The PERF_COUNTER_BLOCK is followed by            */
 /*  PERF_OBJECT_TYPE.NumCounters number of counters. */
 /*                                                   */

 /* Support for New Extensible API starting with NT 5.0 */

 dcl PERF_QUERY_OBJECTS      type LONG value ('80000000'xn);
 dcl PERF_QUERY_GLOBAL       type LONG value ('80000001'xn);
 dcl PERF_QUERY_COSTLY       type LONG value ('80000002'xn);

 /*  function typedefs for extensible counter function prototypes */

 define alias PM_OPEN_PROC
              entry (WCHAR(*) varz byaddr)
              returns (type DWORD byvalue optional)
              limited APIENTRY;

 define alias PM_COLLECT_PROC
              entry (WCHAR(*) varz byaddr,
                     pointer byaddr,
                     type DWORD byaddr,
                     type DWORD byaddr)
              returns (type DWORD byvalue optional)
              limited APIENTRY;

 define alias PM_CLOSE_PROC
              entry ()
              returns (type DWORD byvalue optional)
              limited APIENTRY;

 define alias PM_QUERY_PROC
              entry (type DWORD byaddr,
                     pointer byaddr,
                     type DWORD byaddr,
                     type DWORD byaddr)
              returns (type DWORD byvalue optional)
              limited APIENTRY;

 dcl MAX_PERF_OBJECTS_IN_QUERY_FUNCTION fixed bin (31) value (8);

 %end; /* #WINPERF_ */


