 /* ================================================================= */
 /*                                                                   */
 /*  mmsystem.cpy -- Include file for Multimedia API's                */
 /*                                                                   */
 /*  Version 4.00                                                     */
 /*                                                                   */
 /*  Copyright 1992 - 1998 Microsoft Corporation. All Rights Reserved.*/
 /* ----------------------------------------------------------------- */
 /*                                                                   */
 /*  Define:         Prevent inclusion of:                            */
 /*  --------------  ------------------------------------------------ */
 /*  MMNODRV         Installable driver support                       */
 /*  MMNOSOUND       Sound support                                    */
 /*  MMNOWAVE        Waveform support                                 */
 /*  MMNOMIDI        MIDI support                                     */
 /*  MMNOAUX         Auxiliary audio support                          */
 /*  MMNOMIXER       Mixer support                                    */
 /*  MMNOTIMER       Timer support                                    */
 /*  MMNOJOY         Joystick support                                 */
 /*  MMNOMCI         MCI support                                      */
 /*  MMNOMMIO        Multimedia file I/O support                      */
 /*  MMNOMMSYSTEM    General MMSYSTEM functions                       */
 /*                                                                   */
 /* ================================================================= */

 %dcl #INC_MMSYSTEM char ext;
 %if #INC_MMSYSTEM = '' %then %do;
 %#INC_MMSYSTEM = 'Y'; /* %declared if mmsystem.cpy has been included */

 %dcl #WINUSER_ char ext;
 %if #WINUSER_ = '' %then %do;
 %include winuser;
 %end;

 %dcl UNICODE char ext;
 %dcl WINVER  fixed ext;

 %dcl WINMMAPI char ext;
 %WINMMAPI = '';


 /*********************************************************************/
 /*                                                                   */
 /*           General constants and data types                        */
 /*                                                                   */
 /*********************************************************************/

 /* general constants */
 dcl MAXPNAMELEN            fixed bin(31) value(32);
     /* max product name length (including NULL) */
 dcl MAXERRORLENGTH         fixed bin(31) value(256);
     /* max error text length   (including NULL) */
 dcl MAX_JOYSTICKOEMVXDNAME fixed bin(31) value(260);
     /* max oem vxd name length (including NULL) */

 /* Microsoft Manufacturer and Product ID's                  */
 /* have been moved to MMREG.CPY for Windows 4.00 and above. */
 /* Accordingly they are omitted here in the PL/I version).  */

 /* general data types */

 define alias MMVERSION type UINT;
 /* major (high byte), minor (low byte) */
 define alias MMRESULT type UINT;
 /* error return code, 0 means no error */
 /* call as if(err=xxxx(...)) Error(err); else */
 %dcl #MMRESULT_ char ext;
 %#MMRESULT_ = 'Y';

 /* define alias LPUINT pointer -- not used here */

 /* MMTIME data structure */
 define structure
   1 MMTIME,
     2 wType    type UINT, /* indicates the contents of the union */
     2 u union,
       3 ms     type DWORD,/* milliseconds */
       3 sample type DWORD,/* samples */
       3 cb     type DWORD,/* byte count */
       3 ticks  type DWORD,/* ticks in MIDI stream */
       3 smpte,            /* SMPTE */
         4 hour       type BYTE,/* hours */
         4 min        type BYTE,/* minutes */
         4 sec        type BYTE,/* seconds */
         4 frame      type BYTE,/* frames */
         4 fps        type BYTE,/* frames per second */
         4 dummy      type BYTE,/* pad */
         4 pad (2)    type BYTE,
       3 midi,             /* MIDI */
         4 songptrpos type DWORD;/* song pointer position */

 define alias  PMMTIME handle MMTIME;
 define alias NPMMTIME handle MMTIME;
 define alias LPMMTIME handle MMTIME;

 /* types for wType field in MMTIME struct */
 dcl TIME_MS      fixed bin(15) value('0001'xn);
     /* time in milliseconds */
 dcl TIME_SAMPLES fixed bin(15) value('0002'xn);
     /* number of wave samples */
 dcl TIME_BYTES   fixed bin(15) value('0004'xn);
     /* current byte offset */
 dcl TIME_SMPTE   fixed bin(15) value('0008'xn);
     /* SMPTE time */
 dcl TIME_MIDI    fixed bin(15) value('0010'xn);
     /* MIDI time */
 dcl TIME_TICKS   fixed bin(15) value('0020'xn);
     /* Ticks within MIDI stream */


 %MAKEFOURCC: proc (ch0, ch1, ch2, ch3) returns (char);
   dcl (ch0, ch1, ch2, ch3) char;
   return ('ior (rank (' || ch0 || '), ' ||
                'isll ( rank (' || ch1 || '),  8), ' ||
                'isll ( rank (' || ch2 || '), 16), ' ||
                'isll ( rank (' || ch3 || '), 24))');
 %end MAKEFOURCC;
 %activate MAKEFOURCC;


 /*********************************************************************/
 /*                                                                   */
 /*           Multimedia Extensions Window Messages                   */
 /*                                                                   */
 /*********************************************************************/

 /* joystick */
 dcl MM_JOY1MOVE            fixed bin(15) value('3A0'xn);
 dcl MM_JOY2MOVE            fixed bin(15) value('3A1'xn);
 dcl MM_JOY1ZMOVE           fixed bin(15) value('3A2'xn);
 dcl MM_JOY2ZMOVE           fixed bin(15) value('3A3'xn);
 dcl MM_JOY1BUTTONDOWN      fixed bin(15) value('3B5'xn);
 dcl MM_JOY2BUTTONDOWN      fixed bin(15) value('3B6'xn);
 dcl MM_JOY1BUTTONUP        fixed bin(15) value('3B7'xn);
 dcl MM_JOY2BUTTONUP        fixed bin(15) value('3B8'xn);

 /* MCI */
 dcl MM_MCINOTIFY           fixed bin(15) value('3B9'xn);

 /* waveform output */
 dcl MM_WOM_OPEN            fixed bin(15) value('3BB'xn);
 dcl MM_WOM_CLOSE           fixed bin(15) value('3BC'xn);
 dcl MM_WOM_DONE            fixed bin(15) value('3BD'xn);

 /* waveform input */
 dcl MM_WIM_OPEN            fixed bin(15) value('3BE'xn);
 dcl MM_WIM_CLOSE           fixed bin(15) value('3BF'xn);
 dcl MM_WIM_DATA            fixed bin(15) value('3C0'xn);

 /* MIDI input */
 dcl MM_MIM_OPEN            fixed bin(15) value('3C1'xn);
 dcl MM_MIM_CLOSE           fixed bin(15) value('3C2'xn);
 dcl MM_MIM_DATA            fixed bin(15) value('3C3'xn);
 dcl MM_MIM_LONGDATA        fixed bin(15) value('3C4'xn);
 dcl MM_MIM_ERROR           fixed bin(15) value('3C5'xn);
 dcl MM_MIM_LONGERROR       fixed bin(15) value('3C6'xn);

 /* MIDI output */
 dcl MM_MOM_OPEN            fixed bin(15) value('3C7'xn);
 dcl MM_MOM_CLOSE           fixed bin(15) value('3C8'xn);
 dcl MM_MOM_DONE            fixed bin(15) value('3C9'xn);

 /* these are also in msvideo.h */
 %dcl #MM_DRVM_OPEN char ext;
 %if #MM_DRVM_OPEN = '' %then %do;
 %#MM_DRVM_OPEN = 'Y';
 /* installable drivers */
 dcl MM_DRVM_OPEN           fixed bin(15) value('3D0'xn);
 dcl MM_DRVM_CLOSE          fixed bin(15) value('3D1'xn);
 dcl MM_DRVM_DATA           fixed bin(15) value('3D2'xn);
 dcl MM_DRVM_ERROR          fixed bin(15) value('3D3'xn);
 %end;

 /* these are used by msacm.h */
 dcl MM_STREAM_OPEN         fixed bin(15) value('3D4'xn);
 dcl MM_STREAM_CLOSE        fixed bin(15) value('3D5'xn);
 dcl MM_STREAM_DONE         fixed bin(15) value('3D6'xn);
 dcl MM_STREAM_ERROR        fixed bin(15) value('3D7'xn);


 %if WINVER >= 1024 %then %do;
 dcl MM_MOM_POSITIONCB      fixed bin(15) value('3CA'xn);
 /* Callback for MEVT_POSITIONCB */

 %dcl #MM_MCISIGNAL char ext;
 %if #MM_MCISIGNAL = '' %then %do;
 %#MM_MCISIGNAL = 'Y';
 dcl MM_MCISIGNAL           fixed bin(15) value('3CB'xn);
 %end;

 dcl MM_MIM_MOREDATA        fixed bin(15) value('3CC'xn);
     /* MIM_DONE w/ pending events */

 %end; /* WINVER >= 0x0400 */
 dcl MM_MIXM_LINE_CHANGE    fixed bin(15) value('3D0'xn);
     /* mixer line change notify */
 dcl MM_MIXM_CONTROL_CHANGE fixed bin(15) value('3D1'xn);
     /* mixer control change notify */

 /*********************************************************************/
 /*                                                                   */
 /*       String resource number bases (internal use)                 */
 /*                                                                   */
 /*********************************************************************/

 dcl MMSYSERR_BASE     fixed bin(31) value(0);
 dcl WAVERR_BASE       fixed bin(31) value(32);
 dcl MIDIERR_BASE      fixed bin(31) value(64);
 dcl TIMERR_BASE       fixed bin(31) value(96);
 dcl JOYERR_BASE       fixed bin(31) value(160);
 dcl MCIERR_BASE       fixed bin(31) value(256);
 dcl MIXERR_BASE       fixed bin(31) value(1024);

 dcl MCI_STRING_OFFSET fixed bin(31) value(512);
 dcl MCI_VD_OFFSET     fixed bin(31) value(1024);
 dcl MCI_CD_OFFSET     fixed bin(31) value(1088);
 dcl MCI_WAVE_OFFSET   fixed bin(31) value(1152);
 dcl MCI_SEQ_OFFSET    fixed bin(31) value(1216);

 /*********************************************************************/
 /*                                                                   */
 /*          General error return values                              */
 /*                                                                   */
 /*********************************************************************/

     /* general error return values */
 dcl MMSYSERR_NOERROR      fixed bin(31) value(0);
     /* no error */
 dcl MMSYSERR_ERROR        fixed bin(31) value(MMSYSERR_BASE + 1);
     /* unspecified error */
 dcl MMSYSERR_BADDEVICEID  fixed bin(31) value(MMSYSERR_BASE + 2);
     /* device ID out of range */
 dcl MMSYSERR_NOTENABLED   fixed bin(31) value(MMSYSERR_BASE + 3);
     /* driver failed enable */
 dcl MMSYSERR_ALLOCATED    fixed bin(31) value(MMSYSERR_BASE + 4);
     /* device already allocated */
 dcl MMSYSERR_INVALHANDLE  fixed bin(31) value(MMSYSERR_BASE + 5);
     /* device handle is invalid */
 dcl MMSYSERR_NODRIVER     fixed bin(31) value(MMSYSERR_BASE + 6);
     /* no device driver present */
 dcl MMSYSERR_NOMEM        fixed bin(31) value(MMSYSERR_BASE + 7);
     /* memory allocation error */
 dcl MMSYSERR_NOTSUPPORTED fixed bin(31) value(MMSYSERR_BASE + 8);
     /* function isn't supported */
 dcl MMSYSERR_BADERRNUM    fixed bin(31) value(MMSYSERR_BASE + 9);
     /* error value out of range */
 dcl MMSYSERR_INVALFLAG    fixed bin(31) value(MMSYSERR_BASE + 10);
     /* invalid flag passed */
 dcl MMSYSERR_INVALPARAM   fixed bin(31) value(MMSYSERR_BASE + 11);
     /* invalid parameter passed */
 dcl MMSYSERR_HANDLEBUSY   fixed bin(31) value(MMSYSERR_BASE + 12);
     /* handle being used simultaneously on another */
     /* thread (eg callback) */
 dcl MMSYSERR_INVALIDALIAS fixed bin(31) value(MMSYSERR_BASE + 13);
     /* specified alias not found */
 dcl MMSYSERR_BADDB        fixed bin(31) value(MMSYSERR_BASE + 14);
     /* bad registry database */
 dcl MMSYSERR_KEYNOTFOUND  fixed bin(31) value(MMSYSERR_BASE + 15);
     /* registry key not found */
 dcl MMSYSERR_READERROR    fixed bin(31) value(MMSYSERR_BASE + 16);
     /* registry read error */
 dcl MMSYSERR_WRITEERROR   fixed bin(31) value(MMSYSERR_BASE + 17);
     /* registry write error */
 dcl MMSYSERR_DELETEERROR  fixed bin(31) value(MMSYSERR_BASE + 18);
     /* registry delete error */
 dcl MMSYSERR_VALNOTFOUND  fixed bin(31) value(MMSYSERR_BASE + 19);
     /* registry value not found */
 dcl MMSYSERR_NODRIVERCB   fixed bin(31) value(MMSYSERR_BASE + 20);
     /* driver does not call DriverCallback */
 dcl MMSYSERR_LASTERROR    fixed bin(31) value(MMSYSERR_BASE + 20);
     /* last error in range */

 DECLARE_HANDLE (HDRVR);

 %dcl MMNODRV char ext;
 %if MMNODRV = '' %then %do;

 /*********************************************************************/
 /*                                                                   */
 /*          Installable driver support                               */
 /*                                                                   */
 /*********************************************************************/

 define structure
   1 DRVCONFIGINFOEX,
     2 dwDCISize          type DWORD,
     2 lpszDCISectionName type LPCWSTR,
     2 lpszDCIAliasName   type LPCWSTR,
     2 dnDevNode          type DWORD;
 define alias  PDRVCONFIGINFOEX handle DRVCONFIGINFOEX;
 define alias NPDRVCONFIGINFOEX handle DRVCONFIGINFOEX;
 define alias LPDRVCONFIGINFOEX handle DRVCONFIGINFOEX;


 %dcl #DRV_LOAD char ext;
 %if #DRV_LOAD = '' %then %do;
 %#DRV_LOAD = 'Y';

 /* Driver messages */
 dcl DRV_LOAD           fixed bin(15) value('0001'xn);
 dcl DRV_ENABLE         fixed bin(15) value('0002'xn);
 dcl DRV_OPEN           fixed bin(15) value('0003'xn);
 dcl DRV_CLOSE          fixed bin(15) value('0004'xn);
 dcl DRV_DISABLE        fixed bin(15) value('0005'xn);
 dcl DRV_FREE           fixed bin(15) value('0006'xn);
 dcl DRV_CONFIGURE      fixed bin(15) value('0007'xn);
 dcl DRV_QUERYCONFIGURE fixed bin(15) value('0008'xn);
 dcl DRV_INSTALL        fixed bin(15) value('0009'xn);
 dcl DRV_REMOVE         fixed bin(15) value('000A'xn);
 dcl DRV_EXITSESSION    fixed bin(15) value('000B'xn);
 dcl DRV_POWER          fixed bin(15) value('000F'xn);
 dcl DRV_RESERVED       fixed bin(15) value('0800'xn);
 dcl DRV_USER           fixed bin(15) value('4000'xn);

 /* LPARAM of DRV_CONFIGURE message */
 define structure
   1 DRVCONFIGINFO,
     2 dwDCISize          type DWORD,
     2 lpszDCISectionName type LPCWSTR,
     2 lpszDCIAliasName   type LPCWSTR;

 define alias  PDRVCONFIGINFO handle DRVCONFIGINFO;
 define alias NPDRVCONFIGINFO handle DRVCONFIGINFO;
 define alias LPDRVCONFIGINFO handle DRVCONFIGINFO;

 /* Supported return values for DRV_CONFIGURE message */
 dcl DRVCNF_CANCEL  fixed bin(15) value('0000'xn);
 dcl DRVCNF_OK      fixed bin(15) value('0001'xn);
 dcl DRVCNF_RESTART fixed bin(15) value('0002'xn);

 /* installable driver function prototypes */

 define alias DRIVERPROC
        entry (type DWORD,
               type HDRVR,
               type UINT,
               type LPARAM,
               type LPARAM)
        returns (type LRESULT byvalue optional)
        limited CALLBACK;

 dcl CloseDriver
     entry (type HDRVR,
            type LONG,
            type LONG)
     returns (type LRESULT byvalue optional)
     WINAPI WINMMAPI external('CloseDriver');

 dcl OpenDriver
     entry (WCHAR(*) varz byaddr nonasgn,
            WCHAR(*) varz byaddr nonasgn optional,
            type LONG)
     returns (type HDRVR byvalue optional)
     WINAPI WINMMAPI external('OpenDriver');

 dcl SendDriverMessage
     entry (type HDRVR,
            type UINT,
            type LONG,
            type LONG)
     returns (type LRESULT byvalue optional)
     WINAPI WINMMAPI external('SendDriverMessage');

 dcl DrvGetModuleHandle
     entry (type HDRVR)
     returns (type HMODULE byvalue optional)
     WINAPI WINMMAPI external('DrvGetModuleHandle');

 dcl GetDriverModuleHandle
     entry (type HDRVR )
     returns (type HMODULE byvalue optional)
     WINAPI WINMMAPI external('GetDriverModuleHandle');

 dcl DefDriverProc
     entry (type DWORD,
            type HDRVR,
            type UINT,
            type LPARAM,
            type LPARAM)
     returns (type LRESULT byvalue optional)
     WINAPI WINMMAPI external('DefDriverProc');
 %end; /* #DRV_LOAD */


 %if WINVER >= 778 %then %do;
 /* return values from DriverProc() function */
 dcl DRV_CANCEL  fixed bin(15) value(DRVCNF_CANCEL);
 dcl DRV_OK      fixed bin(15) value(DRVCNF_OK);
 dcl DRV_RESTART fixed bin(15) value(DRVCNF_RESTART);
 %end; /* ifdef WINVER >= 0x030a */

 dcl DRV_MCI_FIRST fixed bin(15) value(DRV_RESERVED);
 dcl DRV_MCI_LAST  fixed bin(15) value(DRV_RESERVED + '0FFF'xn);

 %end; /* ifndef MMNODRV */

 /*********************************************************************/
 /*                                                                   */
 /*            Driver callback support                                */
 /*                                                                   */
 /*********************************************************************/


 /* flags used with waveOutOpen(), waveInOpen(), midiInOpen(), and */
 /* midiOutOpen() to specify the type of the dwCallback parameter. */

 dcl CALLBACK_TYPEMASK fixed bin(31) value('00070000'xn);
     /* callback type mask */
 dcl CALLBACK_NULL     fixed bin(31) value('00000000'xn);
     /* no callback */
 dcl CALLBACK_WINDOW   fixed bin(31) value('00010000'xn);
     /* dwCallback is a HWND */
 dcl CALLBACK_TASK     fixed bin(31) value('00020000'xn);
     /* dwCallback is a HTASK */
 dcl CALLBACK_FUNCTION fixed bin(31) value('00030000'xn);
     /* dwCallback is a FARPROC */
 dcl CALLBACK_THREAD   fixed bin(31) value (CALLBACK_TASK);
 dcl CALLBACK_EVENT    fixed bin(31) value('00050000'xn);
     /* dwCallback is an EVENT Handle */


 define alias LPDRVCALLBACK
        entry (type HDRVR,
               type UINT,
               type DWORD,
               type DWORD,
               type DWORD)
        limited CALLBACK;
 define alias PDRVCALLBACK type LPDRVCALLBACK;


 %dcl MMNOMMSYSTEM char ext;
 %if MMNOMMSYSTEM = '' %then %do;
 /*********************************************************************/
 /*                                                                   */
 /*           General MMSYSTEM support                                */
 /*                                                                   */
 /*********************************************************************/


 %if WINVER <= 778 %then %do;
 dcl mmsystemGetVersion
     entry ()
     returns (type UINT byvalue optional)
     WINAPI WINMMAPI external('mmsystemGetVersion');
 %end;
 %dcl OutputDebugStr char ext;
 %OutputDebugStr='OutputDebugString';

 %end; /* ifndef MMNOMMSYSTEM */

 %dcl MMNOSOUND char ext;
 %if MMNOSOUND = '' %then %do;
 /*********************************************************************/
 /*                                                                   */
 /*               Sound support                                       */
 /*                                                                   */
 /*********************************************************************/


 dcl sndPlaySoundA
     entry (char (*) varz byaddr nonasgn optional,
            type UINT)
     returns (type BOOL byvalue optional)
     WINAPI WINMMAPI external('sndPlaySoundA');
 dcl sndPlaySoundW
     entry (WCHAR (*) varz byaddr nonasgn optional,
            type UINT)
     returns (type BOOL byvalue optional)
     WINAPI WINMMAPI external('sndPlaySoundW');

 %dcl sndPlaySound char ext;
 %if UNICODE ^= '' %then %do;
   %sndPlaySound='sndPlaySoundW';
 %end; %else %do;
   %sndPlaySound='sndPlaySoundA';
 %end; /* !UNICODE */

 /* flag values for fuSound and fdwSound arguments on [snd]PlaySound */
 dcl SND_SYNC        fixed bin(31) value('0000'xn);
     /* play synchronously (default) */
 dcl SND_ASYNC       fixed bin(31) value('0001'xn);
     /* play asynchronously */
 dcl SND_NODEFAULT   fixed bin(31) value('0002'xn);
     /* silence (!default) if sound not found */
 dcl SND_MEMORY      fixed bin(31) value('0004'xn);
     /* pszSound points to a memory file */
 dcl SND_LOOP        fixed bin(31) value('0008'xn);
     /* loop the sound until next sndPlaySound */
 dcl SND_NOSTOP      fixed bin(31) value('0010'xn);
     /* don't stop any currently playing sound */

 dcl SND_NOWAIT      fixed bin(31) value('00002000'xn);
     /* don't wait if the driver is busy */
 dcl SND_ALIAS       fixed bin(31) value('00010000'xn);
     /* name is a registry alias */
 dcl SND_ALIAS_ID    fixed bin(31) value('00110000'xn);
     /* alias is a predefined ID */
 dcl SND_FILENAME    fixed bin(31) value('00020000'xn);
     /* name is file name */
 dcl SND_RESOURCE    fixed bin(31) value('00040004'xn);
     /* name is resource name or atom */

 %if WINVER >= 1024 %then %do;
 dcl SND_PURGE       fixed bin(31) value('0040'xn);
     /* purge non-static events for task */
 dcl SND_APPLICATION fixed bin(31) value('0080'xn);
     /* look for application specific association */
 %end; /* WINVER >= 0x0400 */

 dcl SND_ALIAS_START fixed bin(31) value(0);
     /* alias base */

 %sndAlias: proc (ch0, ch1) returns (char);
   dcl (ch0, ch1) char;
   return ('(SND_ALIAS_START + ior (rank (' || ch0 || '),  ' ||
                                   'isll (rank (' || ch1 || '), 8)))');
 %end sndAlias;
 %activate sndAlias;


 dcl SND_ALIAS_SYSTEMASTERISK fixed bin(31) value(sndAlias ('S', '*'));
 dcl SND_ALIAS_SYSTEMQUESTION fixed bin(31) value(sndAlias ('S', '?'));
 dcl SND_ALIAS_SYSTEMHAND     fixed bin(31) value(sndAlias ('S', 'H'));
 dcl SND_ALIAS_SYSTEMEXIT     fixed bin(31) value(sndAlias ('S', 'E'));
 dcl SND_ALIAS_SYSTEMSTART    fixed bin(31) value(sndAlias ('S', 'S'));
 dcl SND_ALIAS_SYSTEMWELCOME  fixed bin(31) value(sndAlias ('S', 'W'));
 dcl SND_ALIAS_SYSTEMEXCLAMATION
                              fixed bin(31) value(sndAlias ('S', '!'));
 dcl SND_ALIAS_SYSTEMDEFAULT  fixed bin(31) value(sndAlias ('S', 'D'));

 dcl PlaySoundA
     entry (char (*) varz byaddr nonasgn optional,
            type HMODULE,
            type DWORD)
     returns (type BOOL byvalue optional)
     WINAPI WINMMAPI external('PlaySoundA');
 dcl PlaySoundW
     entry (WCHAR (*) varz byaddr nonasgn optional,
            type HMODULE,
            type DWORD)
     returns (type BOOL byvalue optional)
     WINAPI WINMMAPI external('PlaySoundW');

 %dcl PlaySound char ext;
 %if UNICODE ^= '' %then %do;
   %PlaySound='PlaySoundW';
 %end; %else %do;
   %PlaySound='PlaySoundA';
 %end; /* !UNICODE */


 %end; /* ifndef MMNOSOUND */

 %dcl MMNOWAVE char ext;
 %if MMNOWAVE = '' %then %do;
 /*********************************************************************/
 /*                                                                   */
 /*          Waveform audio support                                   */
 /*                                                                   */
 /*********************************************************************/

 /* waveform audio error return values */
 dcl WAVERR_BADFORMAT    fixed bin(31) value(WAVERR_BASE + 0);
     /* unsupported wave format */
 dcl WAVERR_STILLPLAYING fixed bin(31) value(WAVERR_BASE + 1);
     /* still something playing */
 dcl WAVERR_UNPREPARED   fixed bin(31) value(WAVERR_BASE + 2);
     /* header not prepared */
 dcl WAVERR_SYNC         fixed bin(31) value(WAVERR_BASE + 3);
     /* device is synchronous */
 dcl WAVERR_LASTERROR    fixed bin(31) value(WAVERR_BASE + 3);
     /* last error in range */

 /* waveform audio data types */
 DECLARE_HANDLE (HWAVE);
 DECLARE_HANDLE (HWAVEIN);
 DECLARE_HANDLE (HWAVEOUT);
 define alias LPHWAVEIN      pointer;
 define alias LPHWAVEOUT     pointer;
 define alias LPWAVECALLBACK type LPDRVCALLBACK;

 /* wave callback messages */
 dcl WOM_OPEN  fixed bin (15) value (MM_WOM_OPEN);
 dcl WOM_CLOSE fixed bin (15) value (MM_WOM_CLOSE);
 dcl WOM_DONE  fixed bin (15) value (MM_WOM_DONE);
 dcl WIM_OPEN  fixed bin (15) value (MM_WIM_OPEN);
 dcl WIM_CLOSE fixed bin (15) value (MM_WIM_CLOSE);
 dcl WIM_DATA  fixed bin (15) value (MM_WIM_DATA);

 /* device ID for wave device mapper */
 dcl WAVE_MAPPER fixed bin(31) value(-1 );

 /* flags for dwFlags parameter in waveOutOpen() and waveInOpen() */
 dcl WAVE_FORMAT_QUERY fixed bin(15) value('0001'xn);
 dcl WAVE_ALLOWSYNC    fixed bin(15) value('0002'xn);

 %if WINVER >= 1024 %then %do;
 dcl WAVE_MAPPED        fixed bin(15) value('0004'xn);
 dcl WAVE_FORMAT_DIRECT fixed bin(15) value('0008'xn);
 dcl WAVE_FORMAT_DIRECT_QUERY fixed bin(15)
     value(ior (WAVE_FORMAT_QUERY, WAVE_FORMAT_DIRECT));
 %end; /* WINVER >= 0x0400 */

 /* wave data block header */
 define structure
   1 WAVEHDR,
     2 lpData          type LPSTR,/* pointer to locked data buffer */
     2 dwBufferLength  type DWORD,/* length of data buffer */
     2 dwBytesRecorded type DWORD,/* used for input only */
     2 dwUser          type DWORD,/* for client's use */
     2 dwFlags         type DWORD,/* assorted flags (see defines) */
     2 dwLoops         type DWORD,/* loop control counter */
     2 lpNext          handle WAVEHDR,/* reserved for driver */
     2 reserved        type DWORD;/* reserved for driver */

 define alias  PWAVEHDR handle WAVEHDR;
 define alias NPWAVEHDR handle WAVEHDR;
 define alias LPWAVEHDR handle WAVEHDR;

 /* flags for dwFlags field of WAVEHDR */
 dcl WHDR_DONE      fixed bin(31) value('00000001'xn);
     /* done bit */
 dcl WHDR_PREPARED  fixed bin(31) value('00000002'xn);
     /* set if this header has been prepared */
 dcl WHDR_BEGINLOOP fixed bin(31) value('00000004'xn);
     /* loop start block */
 dcl WHDR_ENDLOOP   fixed bin(31) value('00000008'xn);
     /* loop end block */
 dcl WHDR_INQUEUE   fixed bin(31) value('00000010'xn);
     /* reserved for driver */

 /* waveform output device capabilities structure */

 define structure
   1 WAVEOUTCAPSA,
     2 wMid           type WORD, /* manufacturer ID */
     2 wPid           type WORD, /* product ID */
     2 vDriverVersion type MMVERSION,/* version of the driver */
     2 szPname        char (MAXPNAMELEN-1) varz,
       /* product name (NULL terminated string) */
     2 dwFormats      type DWORD,/* formats supported */
     2 wChannels      type WORD, /* number of sources supported */
     2 wReserved1     type WORD, /* packing */
     2 dwSupport      type DWORD;/* functionality supported by driver */
 define alias  PWAVEOUTCAPSA handle WAVEOUTCAPSA;
 define alias NPWAVEOUTCAPSA handle WAVEOUTCAPSA;
 define alias LPWAVEOUTCAPSA handle WAVEOUTCAPSA;

 define structure
   1 WAVEOUTCAPSW,
     2 wMid           type WORD, /* manufacturer ID */
     2 wPid           type WORD, /* product ID */
     2 vDriverVersion type MMVERSION,/* version of the driver */
     2 szPname        WCHAR (MAXPNAMELEN-1) varz,
       /* product name (NULL terminated string) */
     2 dwFormats      type DWORD,/* formats supported */
     2 wChannels      type WORD, /* number of sources supported */
     2 wReserved1     type WORD, /* packing */
     2 dwSupport      type DWORD;/* functionality supported by driver */
 define alias  PWAVEOUTCAPSW handle WAVEOUTCAPSW;
 define alias NPWAVEOUTCAPSW handle WAVEOUTCAPSW;
 define alias LPWAVEOUTCAPSW handle WAVEOUTCAPSW;

 %if UNICODE ^= '' %then %do;
 define alias   WAVEOUTCAPS type   WAVEOUTCAPSW;
 define alias  PWAVEOUTCAPS type  PWAVEOUTCAPSW;
 define alias NPWAVEOUTCAPS type NPWAVEOUTCAPSW;
 define alias LPWAVEOUTCAPS type LPWAVEOUTCAPSW;
 %end; %else %do;
 define alias   WAVEOUTCAPS type   WAVEOUTCAPSA;
 define alias  PWAVEOUTCAPS type  PWAVEOUTCAPSA;
 define alias NPWAVEOUTCAPS type NPWAVEOUTCAPSA;
 define alias LPWAVEOUTCAPS type LPWAVEOUTCAPSA;
 %end; /* UNICODE */

 /* flags for dwSupport field of WAVEOUTCAPS */
 dcl WAVECAPS_PITCH          fixed bin(15) value('0001'xn);
     /* supports pitch control */
 dcl WAVECAPS_PLAYBACKRATE   fixed bin(15) value('0002'xn);
     /* supports playback rate control */
 dcl WAVECAPS_VOLUME         fixed bin(15) value('0004'xn);
     /* supports volume control */
 dcl WAVECAPS_LRVOLUME       fixed bin(15) value('0008'xn);
     /* separate left-right volume control */
 dcl WAVECAPS_SYNC           fixed bin(15) value('0010'xn);
 dcl WAVECAPS_SAMPLEACCURATE fixed bin(15) value('0020'xn);
 dcl WAVECAPS_DIRECTSOUND    fixed bin(15) value('0040'xn);

 /* waveform input device capabilities structure */

 define structure
   1 WAVEINCAPSA,
     2 wMid           type WORD, /* manufacturer ID */
     2 wPid           type WORD, /* product ID */
     2 vDriverVersion type MMVERSION,/* version of the driver */
     2 szPname        char (MAXPNAMELEN-1) varz,
       /* product name (NULL terminated string) */
     2 dwFormats      type DWORD,/* formats supported */
     2 wChannels      type WORD, /* number of channels supported */
     2 wReserved1     type WORD; /* structure packing */
 define alias  PWAVEINCAPSA handle WAVEINCAPSA;
 define alias NPWAVEINCAPSA handle WAVEINCAPSA;
 define alias LPWAVEINCAPSA handle WAVEINCAPSA;

 define structure
   1 WAVEINCAPSW,
     2 wMid           type WORD, /* manufacturer ID */
     2 wPid           type WORD, /* product ID */
     2 vDriverVersion type MMVERSION,/* version of the driver */
     2 szPname        WCHAR (MAXPNAMELEN-1) varz,
       /* product name (NULL terminated string) */
     2 dwFormats      type DWORD,/* formats supported */
     2 wChannels      type WORD, /* number of channels supported */
     2 wReserved1     type WORD; /* structure packing */
 define alias  PWAVEINCAPSW handle WAVEINCAPSW;
 define alias NPWAVEINCAPSW handle WAVEINCAPSW;
 define alias LPWAVEINCAPSW handle WAVEINCAPSW;

 %if UNICODE ^= '' %then %do;
 define alias   WAVEINCAPS type   WAVEINCAPSW;
 define alias  PWAVEINCAPS type  PWAVEINCAPSW;
 define alias NPWAVEINCAPS type NPWAVEINCAPSW;
 define alias LPWAVEINCAPS type LPWAVEINCAPSW;
 %end; %else %do;
 define alias   WAVEINCAPS type   WAVEINCAPSA;
 define alias  PWAVEINCAPS type  PWAVEINCAPSA;
 define alias NPWAVEINCAPS type NPWAVEINCAPSA;
 define alias LPWAVEINCAPS type LPWAVEINCAPSA;
 %end; /* UNICODE */


 /* defines for dwFormat field of WAVEINCAPS and WAVEOUTCAPS */
 dcl WAVE_INVALIDFORMAT fixed bin(31) value('00000000'xn);
    /* invalid format */
 dcl WAVE_FORMAT_1M08   fixed bin(31) value('00000001'xn);
    /* 11.025 kHz, Mono,   8-bit  */
 dcl WAVE_FORMAT_1S08   fixed bin(31) value('00000002'xn);
    /* 11.025 kHz, Stereo, 8-bit  */
 dcl WAVE_FORMAT_1M16  fixed bin(31) value('00000004'xn);
    /* 11.025 kHz, Mono,   16-bit */
 dcl WAVE_FORMAT_1S16   fixed bin(31) value('00000008'xn);
    /* 11.025 kHz, Stereo, 16-bit */
 dcl WAVE_FORMAT_2M08   fixed bin(31) value('00000010'xn);
     /* 22.05  kHz, Mono,   8-bit  */
 dcl WAVE_FORMAT_2S08   fixed bin(31) value('00000020'xn);
    /* 22.05  kHz, Stereo, 8-bit  */
 dcl WAVE_FORMAT_2M16   fixed bin(31) value('00000040'xn);
     /* 22.05  kHz, Mono,   16-bit */
 dcl WAVE_FORMAT_2S16   fixed bin(31) value('00000080'xn);
     /* 22.05  kHz, Stereo, 16-bit */
 dcl WAVE_FORMAT_4M08   fixed bin(31) value('00000100'xn);
     /* 44.1   kHz, Mono,   8-bit  */
 dcl WAVE_FORMAT_4S08   fixed bin(31) value('00000200'xn);
     /* 44.1   kHz, Stereo, 8-bit  */
 dcl WAVE_FORMAT_4M16   fixed bin(31) value('00000400'xn);
     /* 44.1   kHz, Mono,   16-bit */
 dcl WAVE_FORMAT_4S16   fixed bin(31) value('00000800'xn);
     /* 44.1   kHz, Stereo, 16-bit */

 /* OLD general waveform format structure */
 /* (information common to all formats)   */
 define structure
   1 WAVEFORMAT,
     2 wFormatTag      type WORD, /* format type */
     2 nChannels       type WORD,
       /* number of channels (i.e. mono, stereo, etc.) */
     2 nSamplesPerSec  type DWORD,/* sample rate */
     2 nAvgBytesPerSec type DWORD,/* for buffer estimation */
     2 nBlockAlign     type WORD; /* block size of data */
 define alias  PWAVEFORMAT handle WAVEFORMAT;
 define alias NPWAVEFORMAT handle WAVEFORMAT;
 define alias LPWAVEFORMAT handle WAVEFORMAT;

 /* flags for wFormatTag field of WAVEFORMAT */
 dcl WAVE_FORMAT_PCM fixed bin(31) value(1);
 %dcl #WAVE_FORMAT_PCM char ext;
 %#WAVE_FORMAT_PCM = 'Y';

 /* specific waveform format structure for PCM data */
 define structure
   1 PCMWAVEFORMAT,
     2 wf             type WAVEFORMAT,
     2 wBitsPerSample type WORD;
 define alias  PPCMWAVEFORMAT handle PCMWAVEFORMAT;
 define alias NPPCMWAVEFORMAT handle PCMWAVEFORMAT;
 define alias LPPCMWAVEFORMAT handle PCMWAVEFORMAT;

 %dcl #WAVEFORMATEX_ char ext;
 %if #WAVEFORMATEX_ = '' %then %do;
 %#WAVEFORMATEX_ = 'Y';

 /* Extended waveform format structure used for all non-PCM formats. */
 /* This structure is common to all non-PCM formats.                 */
 define structure
   1 WAVEFORMATEX,
     2 wFormatTag      type WORD,/* format type */
     2 nChannels       type WORD,
       /* number of channels (i.e. mono, stereo...) */
     2 nSamplesPerSec  type DWORD,/* sample rate */
     2 nAvgBytesPerSec type DWORD,/* for buffer estimation */
     2 nBlockAlign     type WORD, /* block size of data */
     2 wBitsPerSample  type WORD,
       /* number of bits per sample of mono data */
     2 cbSize          type WORD;/* the count in bytes of the size of */
                                 /* extra information (after cbSize)  */
 define alias  PWAVEFORMATEX handle WAVEFORMATEX;
 define alias NPWAVEFORMATEX handle WAVEFORMATEX;
 define alias LPWAVEFORMATEX handle WAVEFORMATEX;
 define alias LPCWAVEFORMATEX handle WAVEFORMATEX /* nonasgn */;

 %end; /* _WAVEFORMATEX_ */

 /* waveform audio function prototypes */
 dcl waveOutGetNumDevs
     entry ()
     returns (type UINT byvalue optional)
     WINAPI WINMMAPI external('waveOutGetNumDevs');


 dcl waveOutGetDevCapsA
     entry (type UINT,
            type WAVEOUTCAPSA byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutGetDevCapsA');
 dcl waveOutGetDevCapsW
     entry (type UINT,
            type WAVEOUTCAPSW byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutGetDevCapsW');

 %dcl waveOutGetDevCaps char ext;
 %if UNICODE ^= '' %then %do;
   %waveOutGetDevCaps='waveOutGetDevCapsW';
 %end; %else %do;
   %waveOutGetDevCaps='waveOutGetDevCapsA';
 %end; /* !UNICODE */


 %if WINVER >= 1024 %then %do;
 dcl waveOutGetVolume
     entry (type HWAVEOUT,
            type DWORD byaddr)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutGetVolume');

 dcl waveOutSetVolume
     entry (type HWAVEOUT,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutSetVolume');
 %end; %else %do;
 dcl waveOutGetVolume
     entry (type UINT,
            type DWORD byaddr)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutGetVolume');

 dcl waveOutSetVolume
     entry (type UINT,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutSetVolume');
 %end;


 dcl waveOutGetErrorTextA
     entry (type MMRESULT,
            char (*) varz byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutGetErrorTextA');
 dcl waveOutGetErrorTextW
     entry (type MMRESULT,
            WCHAR (*) varz byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutGetErrorTextW');

 %dcl waveOutGetErrorText char ext;
 %if UNICODE ^= '' %then %do;
   %waveOutGetErrorText='waveOutGetErrorTextW';
 %end; %else %do;
   %waveOutGetErrorText='waveOutGetErrorTextA';
 %end; /* !UNICODE */

 dcl waveOutOpen
     entry (type HWAVEOUT byaddr,
            type UINT,
            type WAVEFORMATEX byaddr nonasgn, /* may have extra data */
            type DWORD,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutOpen');

 dcl waveOutClose
     entry (type HWAVEOUT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutClose');

 dcl waveOutPrepareHeader
     entry (type HWAVEOUT,
            type WAVEHDR byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutPrepareHeader');

 dcl waveOutUnprepareHeader
     entry (type HWAVEOUT,
            type WAVEHDR byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutUnprepareHeader');

 dcl waveOutWrite
     entry (type HWAVEOUT,
            type WAVEHDR byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutWrite');

 dcl waveOutPause
     entry (type HWAVEOUT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutPause');

 dcl waveOutRestart
     entry (type HWAVEOUT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutRestart');

 dcl waveOutReset
     entry (type HWAVEOUT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutReset');

 dcl waveOutBreakLoop
     entry (type HWAVEOUT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutBreakLoop');

 dcl waveOutGetPosition
     entry (type HWAVEOUT,
            type MMTIME byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutGetPosition');

 dcl waveOutGetPitch
     entry (type HWAVEOUT,
            fixed bin (31,16) byaddr) /* not DWORD */
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutGetPitch');

 dcl waveOutSetPitch
     entry (type HWAVEOUT,
            fixed bin (31,16)) /* not DWORD */
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutSetPitch');

 dcl waveOutGetPlaybackRate
     entry (type HWAVEOUT,
            type DWORD byaddr)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutGetPlaybackRate');

 dcl waveOutSetPlaybackRate
     entry (type HWAVEOUT,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutSetPlaybackRate');

 dcl waveOutGetID
     entry (type HWAVEOUT,
            type UINT byaddr)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutGetID');


 %if WINVER >= 778 %then %do;
 dcl waveOutMessage
     entry (type HWAVEOUT,
            type UINT,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveOutMessage');
 %end; /* ifdef WINVER >= 0x030a */

 dcl waveInGetNumDevs
     entry ()
     returns (type UINT byvalue optional)
     WINAPI WINMMAPI external('waveInGetNumDevs');


 dcl waveInGetDevCapsA
     entry (type UINT,
            type WAVEINCAPSA byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveInGetDevCapsA');
 dcl waveInGetDevCapsW
     entry (type UINT,
            type WAVEINCAPSW byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveInGetDevCapsW');

 %dcl waveInGetDevCaps char ext;
 %if UNICODE ^= '' %then %do;
   %waveInGetDevCaps='waveInGetDevCapsW';
 %end; %else %do;
   %waveInGetDevCaps='waveInGetDevCapsA';
 %end; /* !UNICODE */


 dcl waveInGetErrorTextA
     entry (type MMRESULT,
            char (*) varz byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveInGetErrorTextA');
 dcl waveInGetErrorTextW
     entry (type MMRESULT,
            WCHAR (*) varz byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveInGetErrorTextW');

 %dcl waveInGetErrorText char ext;
 %if UNICODE ^= '' %then %do;
   %waveInGetErrorText='waveInGetErrorTextW';
 %end; %else %do;
 %waveInGetErrorText='waveInGetErrorTextA';
 %end; /* !UNICODE */

 dcl waveInOpen
     entry (type HWAVEIN byaddr,
            type UINT,
            type WAVEFORMATEX byaddr nonasgn, /* may have extra data */
            type DWORD,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveInOpen');

 dcl waveInClose
     entry (type HWAVEIN)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveInClose');

 dcl waveInPrepareHeader
     entry (type HWAVEIN,
            type WAVEHDR byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveInPrepareHeader');

 dcl waveInUnprepareHeader
     entry (type HWAVEIN,
            type WAVEHDR byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveInUnprepareHeader');

 dcl waveInAddBuffer
     entry (type HWAVEIN,
            type WAVEHDR byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveInAddBuffer');

 dcl waveInStart
     entry (type HWAVEIN)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveInStart');

 dcl waveInStop
     entry (type HWAVEIN)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveInStop');

 dcl waveInReset
     entry (type HWAVEIN)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveInReset');

 dcl waveInGetPosition
     entry (type HWAVEIN,
            type MMTIME byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveInGetPosition');

 dcl waveInGetID
     entry (type HWAVEIN,
            type UINT byaddr)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveInGetID');


 %if WINVER >= 778 %then %do;
 dcl waveInMessage
     entry (type HWAVEIN,
            type UINT,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('waveInMessage');
 %end; /* ifdef WINVER >= 0x030a */

 %end; /* ifndef MMNOWAVE */

 %dcl MMNOMIDI char ext;
 %if MMNOMIDI = '' %then %do;
 /*********************************************************************/
 /*                                                                   */
 /*             MIDI audio support                                    */
 /*                                                                   */
 /*********************************************************************/

     /* MIDI error return values */
 dcl MIDIERR_UNPREPARED    fixed bin(31) value(MIDIERR_BASE + 0);
     /* header not prepared */
 dcl MIDIERR_STILLPLAYING  fixed bin(31) value(MIDIERR_BASE + 1);
     /* still something playing */
 dcl MIDIERR_NOMAP         fixed bin(31) value(MIDIERR_BASE + 2);
     /* no configured instruments */
 dcl MIDIERR_NOTREADY      fixed bin(31) value(MIDIERR_BASE + 3);
     /* hardware is still busy */
 dcl MIDIERR_NODEVICE      fixed bin(31) value(MIDIERR_BASE + 4);
     /* port no longer connected */
 dcl MIDIERR_INVALIDSETUP  fixed bin(31) value(MIDIERR_BASE + 5);
     /* invalid MIF */
 dcl MIDIERR_BADOPENMODE   fixed bin(31) value(MIDIERR_BASE + 6);
     /* operation unsupported w/ open mode */
 dcl MIDIERR_DONT_CONTINUE fixed bin(31) value(MIDIERR_BASE + 7);
     /* thru device 'eating' a message */
 dcl MIDIERR_LASTERROR     fixed bin(31) value(MIDIERR_BASE + 7);
     /* last error in range */

 /* MIDI audio data types */
 DECLARE_HANDLE (HMIDI);
 DECLARE_HANDLE (HMIDIIN);
 DECLARE_HANDLE (HMIDIOUT);
 DECLARE_HANDLE (HMIDISTRM);
 define alias LPHMIDI     pointer;
 define alias LPHMIDIIN   pointer;
 define alias LPHMIDIOUT  pointer;
 define alias LPHMIDISTRM pointer;
 define alias LPMIDICALLBACK type LPDRVCALLBACK;
 dcl MIDIPATCHSIZE fixed bin(31) value(128);

 /* following has no PL/I equivalent:      */
 /* typedef WORD PATCHARRAY[MIDIPATCHSIZE] */
 %dcl PATCHARRAY char;
 %PATCHARRAY = '(MIDIPATCHSIZE) type WORD';
 define alias LPPATCHARRAY pointer;
 /* following has no PL/I equivalent:      */
 /* typedef WORD KEYARRAY[MIDIPATCHSIZE]   */
 %dcl KEYARRAY char;
 %KEYARRAY   = '(MIDIPATCHSIZE) type WORD';
 define alias LPKEYARRAY   pointer;

 /* MIDI callback messages */
 dcl MIM_OPEN       fixed bin (31) value(MM_MIM_OPEN);
 dcl MIM_CLOSE      fixed bin (31) value(MM_MIM_CLOSE);
 dcl MIM_DATA       fixed bin (31) value(MM_MIM_DATA);
 dcl MIM_LONGDATA   fixed bin (31) value(MM_MIM_LONGDATA);
 dcl MIM_ERROR      fixed bin (31) value(MM_MIM_ERROR);
 dcl MIM_LONGERROR  fixed bin (31) value(MM_MIM_LONGERROR);
 dcl MOM_OPEN       fixed bin (31) value(MM_MOM_OPEN);
 dcl MOM_CLOSE      fixed bin (31) value(MM_MOM_CLOSE);
 dcl MOM_DONE       fixed bin (31) value(MM_MOM_DONE);

 %if WINVER >= 1024 %then %do;
 dcl MIM_MOREDATA   fixed bin (31) value(MM_MIM_MOREDATA);
 dcl MOM_POSITIONCB fixed bin (31) value(MM_MOM_POSITIONCB);
 %end;
 /* WINVER >= 0x0400 */

 /* device ID for MIDI mapper */
 dcl MIDIMAPPER     fixed bin(31) value(-1);
 dcl MIDI_MAPPER    fixed bin(31) value(-1);


 %if WINVER >= 1024 %then %do;
 /* flags for dwFlags parm of midiInOpen() */
 dcl MIDI_IO_STATUS fixed bin(31) value('00000020'xn);
 %end; /* WINVER >= 0x0400 */

 /* flags for wFlags parm of midiOutCachePatches(), */
 /* midiOutCacheDrumPatches()                       */
 dcl MIDI_CACHE_ALL     fixed bin(31) value(1);
 dcl MIDI_CACHE_BESTFIT fixed bin(31) value(2);
 dcl MIDI_CACHE_QUERY   fixed bin(31) value(3);
 dcl MIDI_UNCACHE       fixed bin(31) value(4);

 /* MIDI output device capabilities structure */

 define structure
   1 MIDIOUTCAPSA,
     2 wMid           type WORD,/* manufacturer ID */
     2 wPid           type WORD,/* product ID */
     2 vDriverVersion type MMVERSION,/* version of the driver */
     2 szPname        char (MAXPNAMELEN-1) varz,
                      /* product name (NULL terminated string) */
     2 wTechnology    type WORD,/* type of device */
     2 wVoices        type WORD,/* # of voices   (internal synth only)*/
     2 wNotes         type WORD,/* max # of notes(internal synth only)*/
     2 wChannelMask   type WORD,/* channels used (internal synth only)*/
     2 dwSupport      type DWORD;/* functionality supported by driver */
 define alias  PMIDIOUTCAPSA handle MIDIOUTCAPSA;
 define alias NPMIDIOUTCAPSA handle MIDIOUTCAPSA;
 define alias LPMIDIOUTCAPSA handle MIDIOUTCAPSA;

 define structure
   1 MIDIOUTCAPSW,
     2 wMid           type WORD,/* manufacturer ID */
     2 wPid           type WORD,/* product ID */
     2 vDriverVersion type MMVERSION,/* version of the driver */
     2 szPname        WCHAR (MAXPNAMELEN-1) varz,
                      /* product name (NULL terminated string) */
     2 wTechnology    type WORD,/* type of device */
     2 wVoices        type WORD,/* # of voices   (internal synth only)*/
     2 wNotes         type WORD,/* max # of notes(internal synth only)*/
     2 wChannelMask   type WORD,/* channels used (internal synth only)*/
     2 dwSupport      type DWORD;/* functionality supported by driver */
 define alias  PMIDIOUTCAPSW handle MIDIOUTCAPSW;
 define alias NPMIDIOUTCAPSW handle MIDIOUTCAPSW;
 define alias LPMIDIOUTCAPSW handle MIDIOUTCAPSW;

 %if UNICODE ^= '' %then %do;
 define alias   MIDIOUTCAPS type   MIDIOUTCAPSW;
 define alias  PMIDIOUTCAPS type  PMIDIOUTCAPSW;
 define alias NPMIDIOUTCAPS type NPMIDIOUTCAPSW;
 define alias LPMIDIOUTCAPS type LPMIDIOUTCAPSW;
 %end; %else %do;
 define alias   MIDIOUTCAPS type   MIDIOUTCAPSA;
 define alias  PMIDIOUTCAPS type  PMIDIOUTCAPSA;
 define alias NPMIDIOUTCAPS type NPMIDIOUTCAPSA;
 define alias LPMIDIOUTCAPS type LPMIDIOUTCAPSA;
 %end; /* UNICODE */

 /* flags for wTechnology field of MIDIOUTCAPS structure */
 dcl MOD_MIDIPORT fixed bin(31) value(1);/* output port */
 dcl MOD_SYNTH    fixed bin(31) value(2);/* generic internal synth */
 dcl MOD_SQSYNTH  fixed bin(31) value(3);/* square wave internal synth*/
 dcl MOD_FMSYNTH  fixed bin(31) value(4);/* FM internal synth */
 dcl MOD_MAPPER   fixed bin(31) value(5);/* MIDI mapper */

 /* flags for dwSupport field of MIDIOUTCAPS structure */
 dcl MIDICAPS_VOLUME   fixed bin(15) value('0001'xn);
     /* supports volume control */
 dcl MIDICAPS_LRVOLUME fixed bin(15) value('0002'xn);
     /* separate left-right volume control */
 dcl MIDICAPS_CACHE    fixed bin(15) value('0004'xn);

 %if WINVER >= 1024 %then %do;
 dcl MIDICAPS_STREAM   fixed bin(15) value('0008'xn);
     /* driver supports midiStreamOut directly */
 %end; /* WINVER >= 0x0400 */

 /* MIDI input device capabilities structure */

 define structure
   1 MIDIINCAPSA,
     2 wMid           type WORD,/* manufacturer ID */
     2 wPid           type WORD,/* product ID */
     2 vDriverVersion type MMVERSION,/* version of the driver */
     2 szPname        char (MAXPNAMELEN-1) varz
                      /* product name (NULL terminated string) */
 %if WINVER >= 1024 %then %do;
   , 2 dwSupport type DWORD /* functionality supported by driver */
 %end;
   ;
 define alias  PMIDIINCAPSA handle MIDIINCAPSA;
 define alias NPMIDIINCAPSA handle MIDIINCAPSA;
 define alias LPMIDIINCAPSA handle MIDIINCAPSA;

 define structure
   1 MIDIINCAPSW,
     2 wMid           type WORD,/* manufacturer ID */
     2 wPid           type WORD,/* product ID */
     2 vDriverVersion type MMVERSION,/* version of the driver */
     2 szPname        WCHAR (MAXPNAMELEN-1) varz
                      /* product name (NULL terminated string) */
 %if WINVER >= 1024 %then %do;
   , 2 dwSupport type DWORD /* functionality supported by driver */
 %end;
   ;
 define alias  PMIDIINCAPSW handle MIDIINCAPSW;
 define alias NPMIDIINCAPSW handle MIDIINCAPSW;
 define alias LPMIDIINCAPSW handle MIDIINCAPSW;

 %if UNICODE ^= '' %then %do;
 define alias   MIDIINCAPS type   MIDIINCAPSW;
 define alias  PMIDIINCAPS type  PMIDIINCAPSW;
 define alias NPMIDIINCAPS type NPMIDIINCAPSW;
 define alias LPMIDIINCAPS type LPMIDIINCAPSW;
 %end; %else %do;
 define alias   MIDIINCAPS type   MIDIINCAPSA;
 define alias  PMIDIINCAPS type  PMIDIINCAPSA;
 define alias NPMIDIINCAPS type NPMIDIINCAPSA;
 define alias LPMIDIINCAPS type LPMIDIINCAPSA;
 %end; /* UNICODE */

 /* MIDI data block header */
 define structure
   1 MIDIHDR,
     2 lpData          type LPSTR,/* pointer to locked data block */
     2 dwBufferLength  type DWORD,/* length of data in data block */
     2 dwBytesRecorded type DWORD,/* used for input only */
     2 dwUser          type DWORD,/* for client's use */
     2 dwFlags         type DWORD,/* assorted flags (see defines) */
     2 lpNext          handle MIDIHDR,/* reserved for driver */
     2 reserved        type DWORD /* reserved for driver */
 %if WINVER >= 1024 %then %do;
   , 2 dwOffset        type DWORD,/* Callback offset into buffer */
     2 dwReserved (8)  type DWORD /* Reserved for MMSYSTEM */
 %end;
   ;
 define alias  PMIDIHDR handle MIDIHDR;
 define alias NPMIDIHDR handle MIDIHDR;
 define alias LPMIDIHDR handle MIDIHDR;


 %if WINVER >= 1024 %then %do;
 define structure
   1 MIDIEVENT,
     2 dwDeltaTime type DWORD,/* Ticks since last event */
     2 dwStreamID  type DWORD,/* Reserved; must be zero */
     2 dwEvent     type DWORD,/* Event type and parameters */
     2 dwParms (1) type DWORD;/* Parameters if this is a long event */

 define structure
   1 MIDISTRMBUFFVER,
     2 dwVersion    type DWORD,/* Stream buffer format version */
     2 dwMid        type DWORD,
                    /* Manufacturer ID as defined in MMREG.CPY*/
     2 dwOEMVersion type DWORD;/* Manufacturer version for custom ext */
 %end; /* WINVER >= 0x0400 */

 /* flags for dwFlags field of MIDIHDR structure */
 dcl MHDR_DONE     fixed bin(31) value('00000001'xn);
     /* done bit */
 dcl MHDR_PREPARED fixed bin(31) value('00000002'xn);
     /* set if header prepared */
 dcl MHDR_INQUEUE  fixed bin(31) value('00000004'xn);
     /* reserved for driver */
 dcl MHDR_ISSTRM   fixed bin(31) value('00000008'xn);
     /* Buffer is stream buffer */


 %if WINVER >= 1024 %then %do;
 /* Type codes which go in the high byte of the event DWORD of        */
 /* a stream buffer                                                   */
 /*                                                                   */
 /* Type codes 00-7F contain parameters within the low 24 bits        */
 /* Type codes 80-FF contain a length of their parameter in the low   */
 /* 24 bits, followed by their parameter data in the buffer.          */
 /* The event DWORD contains the exact byte length; the parm data     */
 /* itself must be padded to be an even multiple of 4 bytes long.     */


 dcl MEVT_F_SHORT    fixed bin(31) value('00000000'xn);
 dcl MEVT_F_LONG     fixed bin(31) value('80000000'xn);
 dcl MEVT_F_CALLBACK fixed bin(31) value('40000000'xn);

 MEVT_EVENTTYPE: proc (x) returns (type BYTE)
                 options (inline reorder) internal;
   dcl x type DWORD byvalue;
   dcl (iand, isrl) builtin;
   return (iand (isrl (x, 24), 'ff'xn));
 end MEVT_EVENTTYPE;

 MEVT_EVENTPARM: proc (x) returns (type DWORD)
                 options (inline reorder) internal;
   dcl x type DWORD byvalue;
   dcl iand builtin;
   return (iand (x, '00ffffff'xn));
 end MEVT_EVENTPARM;


 dcl MEVT_SHORTMSG type BYTE  value('00'xn);
     /* parm = shortmsg for midiOutShortMsg */
 dcl MEVT_TEMPO    type BYTE  value('01'xn);
     /* parm = new tempo in microsec/qn     */
 dcl MEVT_NOP      type BYTE  value('02'xn);
     /* parm = unused; does nothing         */

 /* 0x04-0x7F reserved */

 dcl MEVT_LONGMSG  type BYTE  value('80'xn);
     /* parm = bytes to send verbatim       */
 dcl MEVT_COMMENT  type BYTE  value('82'xn);
     /* parm = comment data                 */
 dcl MEVT_VERSION  type BYTE  value('84'xn);
     /* parm = MIDISTRMBUFFVER struct       */

 /* 0x81-0xFF reserved */

 dcl MIDISTRM_ERROR   fixed bin(31) value(-2 );


 /* Structures and defines for midiStreamProperty */

 dcl MIDIPROP_SET     fixed bin(31) value('80000000'xn);
 dcl MIDIPROP_GET     fixed bin(31) value('40000000'xn);

 /* These are intentionally both non-zero so the app cannot   */
 /* accidentally leave the operation off and happen to appear */
 /* to work due to default action.                            */

 dcl MIDIPROP_TIMEDIV fixed bin(31) value('00000001'xn);
 dcl MIDIPROP_TEMPO   fixed bin(31) value('00000002'xn);

 define structure
   1 MIDIPROPTIMEDIV,   /* parm = MIDISTRMBUFFVER struct       */
     2 cbStruct  type DWORD,
     2 dwTimeDiv type DWORD;
 define alias LPMIDIPROPTIMEDIV handle MIDIPROPTIMEDIV;

 define structure
   1 MIDIPROPTEMPO,
     2 cbStruct type DWORD,
     2 dwTempo  type DWORD;
 define alias LPMIDIPROPTEMPO handle MIDIPROPTEMPO;

 %end; /* WINVER >= 0x0400 */

 /* MIDI function prototypes */
 dcl midiOutGetNumDevs
     entry ()
     returns (type UINT byvalue optional)
     WINAPI WINMMAPI external('midiOutGetNumDevs');

 %if WINVER >= 1024 %then %do;
 dcl midiStreamOpen
     entry (type HMIDISTRM byaddr,
            type UINT byaddr,
            type DWORD,
            type DWORD,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiStreamOpen');

 dcl midiStreamClose
     entry (type HMIDISTRM)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiStreamClose');

 dcl midiStreamProperty
     entry (type HMIDISTRM,
            type LPBYTE,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiStreamProperty');

 dcl midiStreamPosition
     entry (type HMIDISTRM,
            type MMTIME byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiStreamPosition');

 dcl midiStreamOut
     entry (type HMIDISTRM,
            type MIDIHDR byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiStreamOut');

 dcl midiStreamPause
     entry (type HMIDISTRM)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiStreamPause');

 dcl midiStreamRestart
     entry (type HMIDISTRM)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiStreamRestart');

 dcl midiStreamStop
     entry (type HMIDISTRM)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiStreamStop');

 dcl midiConnect
     entry (type HMIDI,
            type HMIDIOUT,
            type LPVOID)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiConnect');

 dcl midiDisconnect
     entry (type HMIDI,
            type HMIDIOUT,
            type LPVOID)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiDisconnect');
 %end; /* WINVER >= 0x0400 */


 dcl midiOutGetDevCapsA
     entry (type UINT,
            type MIDIOUTCAPSA byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiOutGetDevCapsA');
 dcl midiOutGetDevCapsW
     entry (type UINT,
            type MIDIOUTCAPSW byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiOutGetDevCapsW');

 %dcl midiOutGetDevCaps char ext;
 %if UNICODE ^= '' %then %do;
   %midiOutGetDevCaps='midiOutGetDevCapsW';
 %end; %else %do;
   %midiOutGetDevCaps='midiOutGetDevCapsA';
 %end; /* !UNICODE */


 %if WINVER >= 1024 %then %do;
 dcl midiOutGetVolume
     entry (type HMIDIOUT,
            type DWORD byaddr)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiOutGetVolume');

 dcl midiOutSetVolume
     entry (type HMIDIOUT,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiOutSetVolume');
 %end; %else %do;
 dcl midiOutGetVolume
     entry (type UINT,
            type DWORD byaddr)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiOutGetVolume');

 dcl midiOutSetVolume
     entry (type UINT,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiOutSetVolume');
 %end;


 dcl midiOutGetErrorTextA
     entry (type MMRESULT,
            char (*) varz byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiOutGetErrorTextA');
 dcl midiOutGetErrorTextW
     entry (type MMRESULT,
            WCHAR (*) varz byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiOutGetErrorTextW');

 %dcl midiOutGetErrorText char ext;
 %if UNICODE ^= '' %then %do;
   %midiOutGetErrorText='midiOutGetErrorTextW';
 %end; %else %do;
   %midiOutGetErrorText='midiOutGetErrorTextA';
 %end; /* !UNICODE */


 dcl midiOutOpen
     entry (type HMIDIOUT byaddr,
            type UINT,
            type DWORD,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiOutOpen');

 dcl midiOutClose
     entry (type HMIDIOUT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiOutClose');

 dcl midiOutPrepareHeader
     entry (type HMIDIOUT,
            type MIDIHDR byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiOutPrepareHeader');

 dcl midiOutUnprepareHeader
     entry (type HMIDIOUT,
            type MIDIHDR byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiOutUnprepareHeader');

 dcl midiOutShortMsg
     entry (type HMIDIOUT,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiOutShortMsg');

 dcl midiOutLongMsg
     entry (type HMIDIOUT,
            type MIDIHDR byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiOutLongMsg');

 dcl midiOutReset
     entry (type HMIDIOUT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiOutReset');

 dcl midiOutCachePatches
     entry (type HMIDIOUT,
            type UINT,
            PATCHARRAY byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiOutCachePatches');

 dcl midiOutCacheDrumPatches
     entry (type HMIDIOUT,
            type UINT,
            KEYARRAY byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiOutCacheDrumPatches');

 dcl midiOutGetID
     entry (type HMIDIOUT,
            type UINT byaddr)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiOutGetID');


 %if WINVER >= 778 %then %do;
 dcl midiOutMessage
     entry (type HMIDIOUT,
            type UINT,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiOutMessage');
 %end; /* ifdef WINVER >= 0x030a */

 dcl midiInGetNumDevs
     entry ()
     returns (type UINT byvalue optional)
     WINAPI WINMMAPI external('midiInGetNumDevs');


 dcl midiInGetDevCapsA
     entry (type UINT,
            type MIDIINCAPSA byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiInGetDevCapsA');
 dcl midiInGetDevCapsW
     entry (type UINT,
            type MIDIINCAPSW byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiInGetDevCapsW');

 %dcl midiInGetDevCaps char ext;
 %if UNICODE ^= '' %then %do;
   %midiInGetDevCaps='midiInGetDevCapsW';
 %end; %else %do;
   %midiInGetDevCaps='midiInGetDevCapsA';
 %end; /* !UNICODE */

 dcl midiInGetErrorTextA
     entry (type MMRESULT,
            char(*) varz byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiInGetErrorTextA');
 dcl midiInGetErrorTextW
     entry (type MMRESULT,
            WCHAR(*) varz byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiInGetErrorTextW');

 %dcl midiInGetErrorText char ext;
 %if UNICODE ^= '' %then %do;
   %midiInGetErrorText='midiInGetErrorTextW';
 %end; %else %do;
   %midiInGetErrorText='midiInGetErrorTextA';
 %end; /* !UNICODE */

 dcl midiInOpen
     entry (type HMIDIIN byaddr,
            type UINT,
            type DWORD,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiInOpen');

 dcl midiInClose
     entry (type HMIDIIN)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiInClose');

 dcl midiInPrepareHeader
     entry (type HMIDIIN,
            type MIDIHDR byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiInPrepareHeader');

 dcl midiInUnprepareHeader
     entry (type HMIDIIN,
            type MIDIHDR byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiInUnprepareHeader');

 dcl midiInAddBuffer
     entry (type HMIDIIN,
            type MIDIHDR byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiInAddBuffer');

 dcl midiInStart
     entry (type HMIDIIN)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiInStart');

 dcl midiInStop
     entry (type HMIDIIN)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiInStop');

 dcl midiInReset
     entry (type HMIDIIN)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiInReset');

 dcl midiInGetID
     entry (type HMIDIIN,
            type UINT byaddr)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiInGetID');


 %if WINVER >= 778 %then %do;
 dcl midiInMessage
     entry (type HMIDIIN,
            type UINT,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('midiInMessage');
 %end; /* ifdef WINVER >= 0x030a */

 %end;  /* ifndef MMNOMIDI */

 %dcl MMNOAUX char ext;
 %if MMNOAUX = '' %then %do;
 /*********************************************************************/
 /*                                                                   */
 /*          Auxiliary audio support                                  */
 /*                                                                   */
 /*********************************************************************/

 /* device ID for aux device mapper */
 dcl AUX_MAPPER fixed bin(31) value(-1 );

 /* Auxiliary audio device capabilities structure */

 define structure
   1 AUXCAPSA,
     2 wMid           type WORD, /* manufacturer ID */
     2 wPid           type WORD, /* product ID */
     2 vDriverVersion type MMVERSION,/* version of the driver */
     2 szPname        char (MAXPNAMELEN-1) varz,
                      /* product name (NULL terminated string) */
     2 wTechnology    type WORD, /* type of device */
     2 wReserved1     type WORD, /* padding */
     2 dwSupport      type DWORD;/* functionality supported by driver */
 define alias  PAUXCAPSA handle AUXCAPSA;
 define alias NPAUXCAPSA handle AUXCAPSA;
 define alias LPAUXCAPSA handle AUXCAPSA;

 define structure
   1 AUXCAPSW,
     2 wMid           type WORD, /* manufacturer ID */
     2 wPid           type WORD, /* product ID */
     2 vDriverVersion type MMVERSION,/* version of the driver */
     2 szPname        WCHAR (MAXPNAMELEN-1) varz,
                      /* product name (NULL terminated string) */
     2 wTechnology    type WORD, /* type of device */
     2 wReserved1     type WORD, /* padding */
     2 dwSupport      type DWORD;/* functionality supported by driver */
 define alias  PAUXCAPSW handle AUXCAPSW;
 define alias NPAUXCAPSW handle AUXCAPSW;
 define alias LPAUXCAPSW handle AUXCAPSW;

 %if UNICODE ^= '' %then %do;
 define alias   AUXCAPS type   AUXCAPSW;
 define alias  PAUXCAPS type  PAUXCAPSW;
 define alias NPAUXCAPS type NPAUXCAPSW;
 define alias LPAUXCAPS type LPAUXCAPSW;
 %end; %else %do;
 define alias   AUXCAPS type   AUXCAPSA;
 define alias  PAUXCAPS type  PAUXCAPSA;
 define alias NPAUXCAPS type NPAUXCAPSA;
 define alias LPAUXCAPS type LPAUXCAPSA;
 %end; /* UNICODE */

 /* flags for wTechnology field in AUXCAPS structure */
 dcl AUXCAPS_CDAUDIO  fixed bin(31) value(1);
     /* audio from internal CD-ROM drive */
 dcl AUXCAPS_AUXIN    fixed bin(31) value(2);
     /* audio from auxiliary input jacks */

 /* flags for dwSupport field in AUXCAPS structure */
 dcl AUXCAPS_VOLUME   fixed bin(15) value('0001'xn);
     /* supports volume control */
 dcl AUXCAPS_LRVOLUME fixed bin(15) value('0002'xn);
     /* separate left-right volume control */

 /* auxiliary audio function prototypes */
 dcl auxGetNumDevs
     entry ()
     returns (type UINT byvalue optional)
     WINAPI WINMMAPI external('auxGetNumDevs');

 dcl auxGetDevCapsA entry (
            type UINT,
            type AUXCAPSA byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('auxGetDevCapsA');
 dcl auxGetDevCapsW entry (
            type UINT,
            type AUXCAPSW byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('auxGetDevCapsW');

 %dcl auxGetDevCaps char ext;
 %if UNICODE ^= '' %then %do;
   %auxGetDevCaps='auxGetDevCapsW';
 %end; %else %do;
   %auxGetDevCaps='auxGetDevCapsA';
 %end; /* !UNICODE */

 dcl auxSetVolume
     entry (type UINT,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('auxSetVolume');

 dcl auxGetVolume
     entry (type UINT,
            type DWORD byaddr)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('auxGetVolume');


 %if WINVER >= 778 %then %do;
 dcl auxOutMessage
     entry (type UINT,
            type UINT,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('auxOutMessage');
 %end;
 /* ifdef WINVER >= 0x030a */

 %end; /* ifndef MMNOAUX */

 %dcl MMNOMIXER char ext;
 %if MMNOMIXER = '' %then %do;
 /*********************************************************************/
 /*                                                                   */
 /*              Mixer Support                                        */
 /*                                                                   */
 /*********************************************************************/

 DECLARE_HANDLE (HMIXEROBJ);
 define alias LPHMIXEROBJ pointer;
 DECLARE_HANDLE (HMIXER);
 define alias LPHMIXER    pointer;

 dcl MIXER_SHORT_NAME_CHARS fixed bin(31) value(16);
 dcl MIXER_LONG_NAME_CHARS  fixed bin(31) value(64);

 /*  MMRESULT error return values specific to the mixer API */

 dcl MIXERR_INVALLINE       fixed bin(31) value(MIXERR_BASE + 0);
 dcl MIXERR_INVALCONTROL    fixed bin(31) value(MIXERR_BASE + 1);
 dcl MIXERR_INVALVALUE      fixed bin(31) value(MIXERR_BASE + 2);
 dcl MIXERR_LASTERROR       fixed bin(31) value(MIXERR_BASE + 2);

 dcl MIXER_OBJECTF_HANDLE   fixed bin(31) value('80000000'xn);
 dcl MIXER_OBJECTF_MIXER    fixed bin(31) value('00000000'xn);
 dcl MIXER_OBJECTF_HMIXER   fixed bin(31)
     value (ior (MIXER_OBJECTF_HANDLE, MIXER_OBJECTF_MIXER));
 dcl MIXER_OBJECTF_WAVEOUT  fixed bin(31) value('10000000'xn);
 dcl MIXER_OBJECTF_HWAVEOUT fixed bin(31)
     value (ior (MIXER_OBJECTF_HANDLE, MIXER_OBJECTF_WAVEOUT));

 dcl MIXER_OBJECTF_WAVEIN   fixed bin(31) value('20000000'xn);
 dcl MIXER_OBJECTF_HWAVEIN  fixed bin(31)
     value (ior (MIXER_OBJECTF_HANDLE, MIXER_OBJECTF_WAVEIN));

 dcl MIXER_OBJECTF_MIDIOUT  fixed bin(31) value('30000000'xn);
 dcl MIXER_OBJECTF_HMIDIOUT fixed bin(31)
     value (ior (MIXER_OBJECTF_HANDLE, MIXER_OBJECTF_MIDIOUT));

 dcl MIXER_OBJECTF_MIDIIN   fixed bin(31) value('40000000'xn);
 dcl MIXER_OBJECTF_HMIDIIN  fixed bin(31)
     value (ior (MIXER_OBJECTF_HANDLE, MIXER_OBJECTF_MIDIIN));

 dcl MIXER_OBJECTF_AUX      fixed bin(31) value('50000000'xn);

 dcl mixerGetNumDevs
     entry ()
     returns (type UINT byvalue optional)
     WINAPI WINMMAPI external('mixerGetNumDevs');

 define structure
   1 MIXERCAPSA,
     2 wMid           type WORD, /* manufacturer id */
     2 wPid           type WORD, /* product id */
     2 vDriverVersion type MMVERSION,/* version of the driver */
     2 szPname        char (MAXPNAMELEN-1) varz,/* product name */
     2 fdwSupport     type DWORD,/* misc. support bits */
     2 cDestinations  type DWORD;/* count of destinations */
 define alias  PMIXERCAPSA handle MIXERCAPSA;
 define alias LPMIXERCAPSA handle MIXERCAPSA;

 define structure
   1 MIXERCAPSW,
     2 wMid           type WORD, /* manufacturer id */
     2 wPid           type WORD, /* product id */
     2 vDriverVersion type MMVERSION,/* version of the driver */
     2 szPname        WCHAR (MAXPNAMELEN-1) varz,/* product name */
     2 fdwSupport     type DWORD,/* misc. support bits */
     2 cDestinations  type DWORD;/* count of destinations */
 define alias  PMIXERCAPSW handle MIXERCAPSW;
 define alias LPMIXERCAPSW handle MIXERCAPSW;

 %if UNICODE ^= '' %then %do;
 define alias   MIXERCAPS type   MIXERCAPSW;
 define alias  PMIXERCAPS type  PMIXERCAPSW;
 define alias LPMIXERCAPS type LPMIXERCAPSW;
 %end; %else %do;
 define alias   MIXERCAPS type   MIXERCAPSA;
 define alias  PMIXERCAPS type  PMIXERCAPSA;
 define alias LPMIXERCAPS type LPMIXERCAPSA;
 %end; /* UNICODE */


 dcl mixerGetDevCapsA
     entry (type UINT,
            type MIXERCAPSA byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('mixerGetDevCapsA');
 dcl mixerGetDevCapsW
     entry (type UINT,
            type MIXERCAPSW byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('mixerGetDevCapsW');

 %dcl mixerGetDevCaps char ext;
 %if UNICODE ^= '' %then %do;
   %mixerGetDevCaps='mixerGetDevCapsW';
 %end; %else %do;
   %mixerGetDevCaps='mixerGetDevCapsA';
 %end; /* !UNICODE */


 dcl mixerOpen
     entry (type HMIXER byaddr,
            type UINT,
            type DWORD,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('mixerOpen');

 dcl mixerClose
     entry (type HMIXER)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('mixerClose');

 dcl mixerMessage
     entry (type HMIXER,
            type UINT,
            type DWORD,
            type DWORD)
     returns (type DWORD byvalue optional)
     WINAPI WINMMAPI external('mixerMessage');

 define structure
   1 MIXERLINEA,
     2 cbStruct      type DWORD,/* size of MIXERLINE structure */
     2 dwDestination type DWORD,/* zero based destination index */
     2 dwSource      type DWORD,/* zero based source index (if source)*/
     2 dwLineID      type DWORD,/* unique line id for mixer device */
     2 fdwLine       type DWORD,/* state/information about line */
     2 dwUser        type DWORD,/* driver specific information */
     2 dwComponentType type DWORD,/* component type line connects to */
     2 cChannels     type DWORD,/* number of channels line supports */
     2 cConnections  type DWORD,/* number of connections [possible] */
     2 cControls     type DWORD,/* number of controls at this line */
     2 szShortName   char (MIXER_SHORT_NAME_CHARS-1) varz,
     2 szName        char (MIXER_LONG_NAME_CHARS-1) varz,
     2 Target,
       3 dwType      type DWORD,/* MIXERLINE_TARGETTYPE_xxxx */
       3 dwDeviceID  type DWORD,/* target device ID of device type */
       3 wMid        type WORD, /* of target device */
       3 wPid        type WORD, /* " */
       3 vDriverVersion type MMVERSION,/* " */
       3 szPname     char (MAXPNAMELEN-1) varz;/* " */
 define alias  PMIXERLINEA handle MIXERLINEA;
 define alias LPMIXERLINEA handle MIXERLINEA;

 define structure
   1 MIXERLINEW,
     2 cbStruct      type DWORD,/* size of MIXERLINE structure */
     2 dwDestination type DWORD,/* zero based destination index */
     2 dwSource      type DWORD,/* zero based source index (if source)*/
     2 dwLineID      type DWORD,/* unique line id for mixer device */
     2 fdwLine       type DWORD,/* state/information about line */
     2 dwUser        type DWORD,/* driver specific information */
     2 dwComponentType type DWORD,/* component type line connects to */
     2 cChannels     type DWORD,/* number of channels line supports */
     2 cConnections  type DWORD,/* number of connections [possible] */
     2 cControls     type DWORD,/* number of controls at this line */
     2 szShortName   WCHAR (MIXER_SHORT_NAME_CHARS-1) varz,
     2 szName        WCHAR (MIXER_LONG_NAME_CHARS-1) varz,
     2 Target,
       3 dwType      type DWORD,/* MIXERLINE_TARGETTYPE_xxxx */
       3 dwDeviceID  type DWORD,/* target device ID of device type */
       3 wMid        type WORD,/* of target device */
       3 wPid        type WORD,/* " */
       3 vDriverVersion type MMVERSION,/* " */
       3 szPname     WCHAR (MAXPNAMELEN-1) varz;/* " */
 define alias  PMIXERLINEW handle MIXERLINEW;
 define alias LPMIXERLINEW handle MIXERLINEW;

 %if UNICODE ^= '' %then %do;
 define alias   MIXERLINE type   MIXERLINEW;
 define alias  PMIXERLINE type  PMIXERLINEW;
 define alias LPMIXERLINE type LPMIXERLINEW;
 %end; %else %do;
 define alias   MIXERLINE type   MIXERLINEA;
 define alias  PMIXERLINE type  PMIXERLINEA;
 define alias LPMIXERLINE type LPMIXERLINEA;
 %end; /* UNICODE */



 /*  MIXERLINE.fdwLine */

 dcl MIXERLINE_LINEF_ACTIVE       fixed bin(31) value('00000001'xn);
 dcl MIXERLINE_LINEF_DISCONNECTED fixed bin(31) value('00008000'xn);
 dcl MIXERLINE_LINEF_SOURCE       fixed bin(31) value('80000000'xn);


 /*  MIXERLINE.dwComponentType */

 /*  component types for destinations and sources */

 dcl MIXERLINE_COMPONENTTYPE_DST_FIRST       fixed bin(31)
     value('00000000'xn);
 dcl MIXERLINE_COMPONENTTYPE_DST_UNDEFINED   fixed bin(31)
     value(MIXERLINE_COMPONENTTYPE_DST_FIRST + 0);
 dcl MIXERLINE_COMPONENTTYPE_DST_DIGITAL     fixed bin(31)
     value(MIXERLINE_COMPONENTTYPE_DST_FIRST + 1);
 dcl MIXERLINE_COMPONENTTYPE_DST_LINE        fixed bin(31)
     value(MIXERLINE_COMPONENTTYPE_DST_FIRST + 2);
 dcl MIXERLINE_COMPONENTTYPE_DST_MONITOR     fixed bin(31)
     value(MIXERLINE_COMPONENTTYPE_DST_FIRST + 3);
 dcl MIXERLINE_COMPONENTTYPE_DST_SPEAKERS    fixed bin(31)
     value(MIXERLINE_COMPONENTTYPE_DST_FIRST + 4);
 dcl MIXERLINE_COMPONENTTYPE_DST_HEADPHONES  fixed bin(31)
     value(MIXERLINE_COMPONENTTYPE_DST_FIRST + 5);
 dcl MIXERLINE_COMPONENTTYPE_DST_TELEPHONE   fixed bin(31)
     value(MIXERLINE_COMPONENTTYPE_DST_FIRST + 6);
 dcl MIXERLINE_COMPONENTTYPE_DST_WAVEIN      fixed bin(31)
     value(MIXERLINE_COMPONENTTYPE_DST_FIRST + 7);
 dcl MIXERLINE_COMPONENTTYPE_DST_VOICEIN     fixed bin(31)
     value(MIXERLINE_COMPONENTTYPE_DST_FIRST + 8);
 dcl MIXERLINE_COMPONENTTYPE_DST_LAST        fixed bin(31)
     value(MIXERLINE_COMPONENTTYPE_DST_FIRST + 8);

 dcl MIXERLINE_COMPONENTTYPE_SRC_FIRST       fixed bin(31)
     value('00001000'xn);
 dcl MIXERLINE_COMPONENTTYPE_SRC_UNDEFINED   fixed bin(31)
     value(MIXERLINE_COMPONENTTYPE_SRC_FIRST + 0);
 dcl MIXERLINE_COMPONENTTYPE_SRC_DIGITAL     fixed bin(31)
     value(MIXERLINE_COMPONENTTYPE_SRC_FIRST + 1);
 dcl MIXERLINE_COMPONENTTYPE_SRC_LINE        fixed bin(31)
    value(MIXERLINE_COMPONENTTYPE_SRC_FIRST + 2);
 dcl MIXERLINE_COMPONENTTYPE_SRC_MICROPHONE  fixed bin(31)
    value(MIXERLINE_COMPONENTTYPE_SRC_FIRST + 3);
 dcl MIXERLINE_COMPONENTTYPE_SRC_SYNTHESIZER fixed bin(31)
    value(MIXERLINE_COMPONENTTYPE_SRC_FIRST + 4);
 dcl MIXERLINE_COMPONENTTYPE_SRC_COMPACTDISC fixed bin(31)
     value(MIXERLINE_COMPONENTTYPE_SRC_FIRST + 5);
 dcl MIXERLINE_COMPONENTTYPE_SRC_TELEPHONE   fixed bin(31)
     value(MIXERLINE_COMPONENTTYPE_SRC_FIRST + 6);
 dcl MIXERLINE_COMPONENTTYPE_SRC_PCSPEAKER   fixed bin(31)
     value(MIXERLINE_COMPONENTTYPE_SRC_FIRST + 7);
 dcl MIXERLINE_COMPONENTTYPE_SRC_WAVEOUT     fixed bin(31)
     value(MIXERLINE_COMPONENTTYPE_SRC_FIRST + 8);
 dcl MIXERLINE_COMPONENTTYPE_SRC_AUXILIARY   fixed bin(31)
     value(MIXERLINE_COMPONENTTYPE_SRC_FIRST + 9);
 dcl MIXERLINE_COMPONENTTYPE_SRC_ANALOG      fixed bin(31)
     value(MIXERLINE_COMPONENTTYPE_SRC_FIRST + 10);
 dcl MIXERLINE_COMPONENTTYPE_SRC_LAST        fixed bin(31)
     value(MIXERLINE_COMPONENTTYPE_SRC_FIRST + 10);


 /*  MIXERLINE.Target.dwType */

 dcl MIXERLINE_TARGETTYPE_UNDEFINED fixed bin(31) value(0);
 dcl MIXERLINE_TARGETTYPE_WAVEOUT   fixed bin(31) value(1);
 dcl MIXERLINE_TARGETTYPE_WAVEIN    fixed bin(31) value(2);
 dcl MIXERLINE_TARGETTYPE_MIDIOUT   fixed bin(31) value(3);
 dcl MIXERLINE_TARGETTYPE_MIDIIN    fixed bin(31) value(4);
 dcl MIXERLINE_TARGETTYPE_AUX       fixed bin(31) value(5);


 dcl mixerGetLineInfoA
     entry (type HMIXEROBJ,
            type MIXERLINEA byaddr,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('mixerGetLineInfoA');
 dcl mixerGetLineInfoW
     entry (type HMIXEROBJ,
            type MIXERLINEW byaddr,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('mixerGetLineInfoW');

 %dcl mixerGetLineInfo char ext;
 %if UNICODE ^= '' %then %do;
   %mixerGetLineInfo='mixerGetLineInfoW';
 %end; %else %do;
   %mixerGetLineInfo='mixerGetLineInfoA';
 %end; /* !UNICODE */

 dcl MIXER_GETLINEINFOF_DESTINATION   fixed bin(31) value('00000000'xn);
 dcl MIXER_GETLINEINFOF_SOURCE        fixed bin(31) value('00000001'xn);
 dcl MIXER_GETLINEINFOF_LINEID        fixed bin(31) value('00000002'xn);
 dcl MIXER_GETLINEINFOF_COMPONENTTYPE fixed bin(31) value('00000003'xn);
 dcl MIXER_GETLINEINFOF_TARGETTYPE    fixed bin(31) value('00000004'xn);

 dcl MIXER_GETLINEINFOF_QUERYMASK     fixed bin(31) value('0000000F'xn);

 dcl mixerGetID
     entry (type HMIXEROBJ,
            type UINT byaddr,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('mixerGetID');


 /*  MIXERCONTROL */

 define structure
   1 MIXERCONTROLA,
     2 cbStruct       type DWORD,/* size in bytes of MIXERCONTROL */
     2 dwControlID    type DWORD,/* unique control id for mixer device*/
     2 dwControlType  type DWORD,/* MIXERCONTROL_CONTROLTYPE_xxx */
     2 fdwControl     type DWORD,/* MIXERCONTROL_CONTROLF_xxx */
     2 cMultipleItems type DWORD,
                      /* if MIXERCONTROL_CONTROLF_MULTIPLE set */
     2 szShortName    char (MIXER_SHORT_NAME_CHARS-1) varz,
     2 szName         char (MIXER_LONG_NAME_CHARS-1) varz,
     2 Bounds union,
       3 *,
         4 lMinimum   type LONG, /* signed minimum for this control */
         4 lMaximum   type LONG, /* signed maximum for this control */
       3 *,
         4 dwMinimum  type DWORD,/* unsigned minimum for this control */
         4 dwMaximum  type DWORD,/* unsigned maximum for this control */
       3 dwReserved (6) type DWORD,
     2 Metrics union,
       3 cSteps       type DWORD,/* # of steps between min & max */
       3 cbCustomData type DWORD,/* size in bytes of custom data */
       3 dwReserved (6) type DWORD;/* !!! needed? we have cbStruct....*/

 define alias  PMIXERCONTROLA handle MIXERCONTROLA;
 define alias LPMIXERCONTROLA handle MIXERCONTROLA;

 define structure
   1 MIXERCONTROLW,
     2 cbStruct       type DWORD,/* size in bytes of MIXERCONTROL */
     2 dwControlID    type DWORD,/* unique control id for mixer device*/
     2 dwControlType  type DWORD,/* MIXERCONTROL_CONTROLTYPE_xxx */
     2 fdwControl     type DWORD,/* MIXERCONTROL_CONTROLF_xxx */
     2 cMultipleItems type DWORD,
                      /* if MIXERCONTROL_CONTROLF_MULTIPLE set */
     2 szShortName    WCHAR (MIXER_SHORT_NAME_CHARS-1) varz,
     2 szName         wCHAR (MIXER_LONG_NAME_CHARS-1) varz,
     2 Bounds union,
       3 *,
         4 lMinimum   type LONG, /* signed minimum for this control */
         4 lMaximum   type LONG, /* signed maximum for this control */
       3 *,
         4 dwMinimum  type DWORD,/* unsigned minimum for this control */
         4 dwMaximum  type DWORD,/* unsigned maximum for this control */
       3 dwReserved (6) type DWORD,
     2 Metrics union,
       3 cSteps       type DWORD,/* # of steps between min & max */
       3 cbCustomData type DWORD,/* size in bytes of custom data */
       3 dwReserved (6) type DWORD;/* !!! needed? we have cbStruct....*/
 define alias  PMIXERCONTROLW handle MIXERCONTROLW;
 define alias LPMIXERCONTROLW handle MIXERCONTROLW;

 %if UNICODE ^= '' %then %do;
 define alias   MIXERCONTROL type   MIXERCONTROLW;
 define alias  PMIXERCONTROL type  PMIXERCONTROLW;
 define alias LPMIXERCONTROL type LPMIXERCONTROLW;
 %end; %else %do;
 define alias   MIXERCONTROL type   MIXERCONTROLA;
 define alias  PMIXERCONTROL type  PMIXERCONTROLA;
 define alias LPMIXERCONTROL type LPMIXERCONTROLA;
 %end; /* UNICODE */


 /*  MIXERCONTROL.fdwControl */

 dcl MIXERCONTROL_CONTROLF_UNIFORM    fixed bin(31) value('00000001'xn);
 dcl MIXERCONTROL_CONTROLF_MULTIPLE   fixed bin(31) value('00000002'xn);
 dcl MIXERCONTROL_CONTROLF_DISABLED   fixed bin(31) value('80000000'xn);


 /*  MIXERCONTROL_CONTROLTYPE_xxx building block defines */

 dcl MIXERCONTROL_CT_CLASS_MASK       fixed bin(31) value('F0000000'xn);
 dcl MIXERCONTROL_CT_CLASS_CUSTOM     fixed bin(31) value('00000000'xn);
 dcl MIXERCONTROL_CT_CLASS_METER      fixed bin(31) value('10000000'xn);
 dcl MIXERCONTROL_CT_CLASS_SWITCH     fixed bin(31) value('20000000'xn);
 dcl MIXERCONTROL_CT_CLASS_NUMBER     fixed bin(31) value('30000000'xn);
 dcl MIXERCONTROL_CT_CLASS_SLIDER     fixed bin(31) value('40000000'xn);
 dcl MIXERCONTROL_CT_CLASS_FADER      fixed bin(31) value('50000000'xn);
 dcl MIXERCONTROL_CT_CLASS_TIME       fixed bin(31) value('60000000'xn);
 dcl MIXERCONTROL_CT_CLASS_LIST       fixed bin(31) value('70000000'xn);

 dcl MIXERCONTROL_CT_SUBCLASS_MASK    fixed bin(31) value('0F000000'xn);

 dcl MIXERCONTROL_CT_SC_SWITCH_BOOLEAN
                                      fixed bin(31) value('00000000'xn);
 dcl MIXERCONTROL_CT_SC_SWITCH_BUTTON fixed bin(31) value('01000000'xn);

 dcl MIXERCONTROL_CT_SC_METER_POLLED  fixed bin(31) value('00000000'xn);

 dcl MIXERCONTROL_CT_SC_TIME_MICROSECS
                                      fixed bin(31) value('00000000'xn);
 dcl MIXERCONTROL_CT_SC_TIME_MILLISECS
                                      fixed bin(31) value('01000000'xn);

 dcl MIXERCONTROL_CT_SC_LIST_SINGLE   fixed bin(31) value('00000000'xn);
 dcl MIXERCONTROL_CT_SC_LIST_MULTIPLE fixed bin(31) value('01000000'xn);

 dcl MIXERCONTROL_CT_UNITS_MASK       fixed bin(31) value('00FF0000'xn);
 dcl MIXERCONTROL_CT_UNITS_CUSTOM     fixed bin(31) value('00000000'xn);
 dcl MIXERCONTROL_CT_UNITS_BOOLEAN    fixed bin(31) value('00010000'xn);
 dcl MIXERCONTROL_CT_UNITS_SIGNED     fixed bin(31) value('00020000'xn);
 dcl MIXERCONTROL_CT_UNITS_UNSIGNED   fixed bin(31) value('00030000'xn);
 dcl MIXERCONTROL_CT_UNITS_DECIBELS   fixed bin(31) value('00040000'xn);
     /* in 10ths */
 dcl MIXERCONTROL_CT_UNITS_PERCENT    fixed bin(31) value('00050000'xn);
     /* in 10ths */


 /* Commonly used control types for specifying */
 /* MIXERCONTROL.dwControlType */

 dcl MIXERCONTROL_CONTROLTYPE_CUSTOM         fixed bin(31)
     value(ior (MIXERCONTROL_CT_CLASS_CUSTOM,
                MIXERCONTROL_CT_UNITS_CUSTOM));
 dcl MIXERCONTROL_CONTROLTYPE_BOOLEANMETER   fixed bin(31)
     value(ior (MIXERCONTROL_CT_CLASS_METER,
                MIXERCONTROL_CT_SC_METER_POLLED,
                MIXERCONTROL_CT_UNITS_BOOLEAN));
 dcl MIXERCONTROL_CONTROLTYPE_SIGNEDMETER    fixed bin(31)
     value(ior (MIXERCONTROL_CT_CLASS_METER,
                MIXERCONTROL_CT_SC_METER_POLLED,
                MIXERCONTROL_CT_UNITS_SIGNED));
 dcl MIXERCONTROL_CONTROLTYPE_PEAKMETER      fixed bin(31)
     value(MIXERCONTROL_CONTROLTYPE_SIGNEDMETER + 1);
 dcl MIXERCONTROL_CONTROLTYPE_UNSIGNEDMETER  fixed bin(31)
     value(ior (MIXERCONTROL_CT_CLASS_METER,
                MIXERCONTROL_CT_SC_METER_POLLED,
                MIXERCONTROL_CT_UNITS_UNSIGNED));
 dcl MIXERCONTROL_CONTROLTYPE_BOOLEAN        fixed bin(31)
     value(ior (MIXERCONTROL_CT_CLASS_SWITCH,
                MIXERCONTROL_CT_SC_SWITCH_BOOLEAN,
                MIXERCONTROL_CT_UNITS_BOOLEAN));
 dcl MIXERCONTROL_CONTROLTYPE_ONOFF          fixed bin(31)
     value(MIXERCONTROL_CONTROLTYPE_BOOLEAN + 1);
 dcl MIXERCONTROL_CONTROLTYPE_MUTE           fixed bin(31)
     value(MIXERCONTROL_CONTROLTYPE_BOOLEAN + 2);
 dcl MIXERCONTROL_CONTROLTYPE_MONO           fixed bin(31)
     value(MIXERCONTROL_CONTROLTYPE_BOOLEAN + 3);
 dcl MIXERCONTROL_CONTROLTYPE_LOUDNESS       fixed bin(31)
     value(MIXERCONTROL_CONTROLTYPE_BOOLEAN + 4);
 dcl MIXERCONTROL_CONTROLTYPE_STEREOENH      fixed bin(31)
     value(MIXERCONTROL_CONTROLTYPE_BOOLEAN + 5);
 dcl MIXERCONTROL_CONTROLTYPE_BUTTON         fixed bin(31)
     value(ior (MIXERCONTROL_CT_CLASS_SWITCH,
                MIXERCONTROL_CT_SC_SWITCH_BUTTON,
                MIXERCONTROL_CT_UNITS_BOOLEAN));
 dcl MIXERCONTROL_CONTROLTYPE_DECIBELS       fixed bin(31)
     value(ior (MIXERCONTROL_CT_CLASS_NUMBER,
                MIXERCONTROL_CT_UNITS_DECIBELS));
 dcl MIXERCONTROL_CONTROLTYPE_SIGNED         fixed bin(31)
     value(ior (MIXERCONTROL_CT_CLASS_NUMBER,
                MIXERCONTROL_CT_UNITS_SIGNED));
 dcl MIXERCONTROL_CONTROLTYPE_UNSIGNED       fixed bin(31)
     value(ior (MIXERCONTROL_CT_CLASS_NUMBER,
                MIXERCONTROL_CT_UNITS_UNSIGNED));
 dcl MIXERCONTROL_CONTROLTYPE_PERCENT        fixed bin(31)
     value(ior (MIXERCONTROL_CT_CLASS_NUMBER,
                MIXERCONTROL_CT_UNITS_PERCENT));
 dcl MIXERCONTROL_CONTROLTYPE_SLIDER         fixed bin(31)
     value(ior (MIXERCONTROL_CT_CLASS_SLIDER,
                MIXERCONTROL_CT_UNITS_SIGNED));
 dcl MIXERCONTROL_CONTROLTYPE_PAN            fixed bin(31)
     value(MIXERCONTROL_CONTROLTYPE_SLIDER + 1);
 dcl MIXERCONTROL_CONTROLTYPE_QSOUNDPAN      fixed bin(31)
     value(MIXERCONTROL_CONTROLTYPE_SLIDER + 2);
 dcl MIXERCONTROL_CONTROLTYPE_FADER          fixed bin(31)
     value(ior (MIXERCONTROL_CT_CLASS_FADER,
                MIXERCONTROL_CT_UNITS_UNSIGNED));
 dcl MIXERCONTROL_CONTROLTYPE_VOLUME         fixed bin(31)
     value(MIXERCONTROL_CONTROLTYPE_FADER + 1);
 dcl MIXERCONTROL_CONTROLTYPE_BASS           fixed bin(31)
     value(MIXERCONTROL_CONTROLTYPE_FADER + 2);
 dcl MIXERCONTROL_CONTROLTYPE_TREBLE         fixed bin(31)
     value(MIXERCONTROL_CONTROLTYPE_FADER + 3);
 dcl MIXERCONTROL_CONTROLTYPE_EQUALIZER      fixed bin(31)
     value(MIXERCONTROL_CONTROLTYPE_FADER + 4);
 dcl MIXERCONTROL_CONTROLTYPE_SINGLESELECT   fixed bin(31)
     value(ior (MIXERCONTROL_CT_CLASS_LIST,
                MIXERCONTROL_CT_SC_LIST_SINGLE,
                MIXERCONTROL_CT_UNITS_BOOLEAN));
 dcl MIXERCONTROL_CONTROLTYPE_MUX            fixed bin(31)
     value(MIXERCONTROL_CONTROLTYPE_SINGLESELECT + 1);
 dcl MIXERCONTROL_CONTROLTYPE_MULTIPLESELECT fixed bin(31)
     value(ior (MIXERCONTROL_CT_CLASS_LIST,
                MIXERCONTROL_CT_SC_LIST_MULTIPLE,
                MIXERCONTROL_CT_UNITS_BOOLEAN));
 dcl MIXERCONTROL_CONTROLTYPE_MIXER          fixed bin(31)
     value(MIXERCONTROL_CONTROLTYPE_MULTIPLESELECT + 1);
 dcl MIXERCONTROL_CONTROLTYPE_MICROTIME      fixed bin(31)
     value(ior (MIXERCONTROL_CT_CLASS_TIME,
                MIXERCONTROL_CT_SC_TIME_MICROSECS,
                MIXERCONTROL_CT_UNITS_UNSIGNED));
 dcl MIXERCONTROL_CONTROLTYPE_MILLITIME      fixed bin(31)
     value(ior (MIXERCONTROL_CT_CLASS_TIME,
                MIXERCONTROL_CT_SC_TIME_MILLISECS,
                MIXERCONTROL_CT_UNITS_UNSIGNED));


 /*  MIXERLINECONTROLS */

 define structure
   1 MIXERLINECONTROLSA,
     2 cbStruct    type DWORD,/* size in bytes of MIXERLINECONTROLS */
     2 dwLineID    type DWORD,/* line id (from MIXERLINE.dwLineID) */
     2 * union,
       3 dwControlID   type DWORD,/* MIXER_GETLINECONTROLSF_ONEBYID */
       3 dwControlType type DWORD,/* MIXER_GETLINECONTROLSF_ONEBYTYPE */
     2 cControls   type DWORD,/* count of controls pmxctrl points to */
     2 cbmxctrl    type DWORD,/* size in bytes of _one_ MIXERCONTROL */
     2 pamxctrl    type LPMIXERCONTROLA;
                              /* pointer to first MIXERCONTROL array */
 define alias  PMIXERLINECONTROLSA handle MIXERLINECONTROLSA;
 define alias LPMIXERLINECONTROLSA handle MIXERLINECONTROLSA;

 define structure
   1 MIXERLINECONTROLSW,
     2 cbStruct    type DWORD,/* size in bytes of MIXERLINECONTROLS */
     2 dwLineID    type DWORD,/* line id (from MIXERLINE.dwLineID) */
     2 * union,
       3 dwControlID   type DWORD,/* MIXER_GETLINECONTROLSF_ONEBYID */
       3 dwControlType type DWORD,/* MIXER_GETLINECONTROLSF_ONEBYTYPE */
     2 cControls   type DWORD,/* count of controls pmxctrl points to */
     2 cbmxctrl    type DWORD,/* size in bytes of _one_ MIXERCONTROL */
     2 pamxctrl    type LPMIXERCONTROLW;
                              /* pointer to first MIXERCONTROL array */
 define alias  PMIXERLINECONTROLSW handle MIXERLINECONTROLSW;
 define alias LPMIXERLINECONTROLSW handle MIXERLINECONTROLSW;

 %if UNICODE ^= '' %then %do;
 define alias   MIXERLINECONTROLS type   MIXERLINECONTROLSW;
 define alias  PMIXERLINECONTROLS type  PMIXERLINECONTROLSW;
 define alias LPMIXERLINECONTROLS type LPMIXERLINECONTROLSW;
 %end; %else %do;
 define alias   MIXERLINECONTROLS type   MIXERLINECONTROLSA;
 define alias  PMIXERLINECONTROLS type  PMIXERLINECONTROLSA;
 define alias LPMIXERLINECONTROLS type LPMIXERLINECONTROLSA;
 %end; /* UNICODE */


 dcl mixerGetLineControlsA
     entry (type HMIXEROBJ,
            type MIXERLINECONTROLSA byaddr,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('mixerGetLineControlsA');
 dcl mixerGetLineControlsW
     entry (type HMIXEROBJ,
            type MIXERLINECONTROLSW byaddr,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('mixerGetLineControlsW');

 %dcl mixerGetLineControls char ext;
 %if UNICODE ^= '' %then %do;
   %mixerGetLineControls='mixerGetLineControlsW';
 %end; %else %do;
   %mixerGetLineControls='mixerGetLineControlsA';
 %end; /* !UNICODE */


 dcl MIXER_GETLINECONTROLSF_ALL       fixed bin(31) value('00000000'xn);
 dcl MIXER_GETLINECONTROLSF_ONEBYID   fixed bin(31) value('00000001'xn);
 dcl MIXER_GETLINECONTROLSF_ONEBYTYPE fixed bin(31) value('00000002'xn);

 dcl MIXER_GETLINECONTROLSF_QUERYMASK fixed bin(31) value('0000000F'xn);

 define structure
   1 MIXERCONTROLDETAILS,
     2 cbStruct    type DWORD,/* size in bytes of MIXERCONTROLDETAILS */
     2 dwControlID type DWORD,/* control id to get/set details on */
     2 cChannels   type DWORD,/* number of channels in paDetails array*/
     2 * union,
       3 hwndOwner type HWND,/* for MIXER_SETCONTROLDETAILSF_CUSTOM */
       3 cMultipleItems type DWORD,
                   /* if _MULTIPLE, the number of items per channel */
     2 cbDetails   type DWORD,/* size of _one_ details_XX struct */
     2 paDetails   type LPVOID;
                   /* pointer to array of details_XX structs*/
 define alias  PMIXERCONTROLDETAILS handle MIXERCONTROLDETAILS;
 define alias LPMIXERCONTROLDETAILS handle MIXERCONTROLDETAILS;


 /*  MIXER_GETCONTROLDETAILSF_LISTTEXT */

 define structure
   1 MIXERCONTROLDETAILS_LISTTEXTA,
     2 dwParam1 type DWORD,
     2 dwParam2 type DWORD,
     2 szName   char (MIXER_LONG_NAME_CHARS-1) varz;
 define alias  PMIXERCONTROLDETAILS_LISTTEXTA
        handle MIXERCONTROLDETAILS_LISTTEXTA;
 define alias LPMIXERCONTROLDETAILS_LISTTEXTA
        handle MIXERCONTROLDETAILS_LISTTEXTA;

 define structure
   1 MIXERCONTROLDETAILS_LISTTEXTW,
     2 dwParam1 type DWORD,
     2 dwParam2 type DWORD,
     2 szName   WCHAR (MIXER_LONG_NAME_CHARS-1) varz;
 define alias PMIXERCONTROLDETAILS_LISTTEXTW
        handle MIXERCONTROLDETAILS_LISTTEXTW;
 define alias LPMIXERCONTROLDETAILS_LISTTEXTW
        handle MIXERCONTROLDETAILS_LISTTEXTW;

 %if UNICODE ^= '' %then %do;
 define alias   MIXERCONTROLDETAILS_LISTTEXT
         type   MIXERCONTROLDETAILS_LISTTEXTW;
 define alias  PMIXERCONTROLDETAILS_LISTTEXT
         type  PMIXERCONTROLDETAILS_LISTTEXTW;
 define alias LPMIXERCONTROLDETAILS_LISTTEXT
         type LPMIXERCONTROLDETAILS_LISTTEXTW;
 %end; %else %do;
 define alias   MIXERCONTROLDETAILS_LISTTEXT
         type   MIXERCONTROLDETAILS_LISTTEXTA;
 define alias  PMIXERCONTROLDETAILS_LISTTEXT
         type  PMIXERCONTROLDETAILS_LISTTEXTA;
 define alias LPMIXERCONTROLDETAILS_LISTTEXT
         type LPMIXERCONTROLDETAILS_LISTTEXTA;
 %end; /* UNICODE */


 /* MIXER_GETCONTROLDETAILSF_VALUE */

 define structure
   1 MIXERCONTROLDETAILS_BOOLEAN,
     2 fValue type LONG;
 define alias PMIXERCONTROLDETAILS_BOOLEAN
        handle MIXERCONTROLDETAILS_BOOLEAN;
 define alias LPMIXERCONTROLDETAILS_BOOLEAN
        handle MIXERCONTROLDETAILS_BOOLEAN;

 define structure
   1 MIXERCONTROLDETAILS_SIGNED,
     2 lValue type LONG;
 define alias PMIXERCONTROLDETAILS_SIGNED
        handle MIXERCONTROLDETAILS_SIGNED;
 define alias LPMIXERCONTROLDETAILS_SIGNED
        handle MIXERCONTROLDETAILS_SIGNED;

 define structure
   1 MIXERCONTROLDETAILS_UNSIGNED,
     2 dwValue type DWORD;
 define alias PMIXERCONTROLDETAILS_UNSIGNED
        handle MIXERCONTROLDETAILS_UNSIGNED;
 define alias LPMIXERCONTROLDETAILS_UNSIGNED
        handle MIXERCONTROLDETAILS_UNSIGNED;


 dcl mixerGetControlDetailsA
     entry (type HMIXEROBJ,
            type MIXERCONTROLDETAILS byaddr,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('mixerGetControlDetailsA');
 dcl mixerGetControlDetailsW
     entry (type HMIXEROBJ,
            type MIXERCONTROLDETAILS byaddr,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('mixerGetControlDetailsW');

 %dcl mixerGetControlDetails char ext;
 %if UNICODE ^= '' %then %do;
   %mixerGetControlDetails='mixerGetControlDetailsW';
 %end; %else %do;
   %mixerGetControlDetails='mixerGetControlDetailsA';
 %end; /* !UNICODE */


 dcl MIXER_GETCONTROLDETAILSF_VALUE     fixed bin(31)
     value('00000000'xn);
 dcl MIXER_GETCONTROLDETAILSF_LISTTEXT  fixed bin(31)
     value('00000001'xn);

 dcl MIXER_GETCONTROLDETAILSF_QUERYMASK fixed bin(31)
     value('0000000F'xn);

 dcl mixerSetControlDetails
     entry (type HMIXEROBJ,
            type MIXERCONTROLDETAILS byaddr,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('mixerSetControlDetails');

 dcl MIXER_SETCONTROLDETAILSF_VALUE  fixed bin(31) value('00000000'xn);
 dcl MIXER_SETCONTROLDETAILSF_CUSTOM fixed bin(31) value('00000001'xn);

 dcl MIXER_SETCONTROLDETAILSF_QUERYMASK
                                     fixed bin(31) value('0000000F'xn);

 %end; /* ifndef MMNOMIXER */

 %dcl MMNOTIMER char ext;
 %if MMNOTIMER = '' %then %do;
 /*********************************************************************/
 /*                                                                   */
 /*              Timer support                                        */
 /*                                                                   */
 /*********************************************************************/

 /* timer error return values */
 dcl TIMERR_NOERROR fixed bin(31) value(0);
     /* no error */
 dcl TIMERR_NOCANDO fixed bin(31) value(TIMERR_BASE+1);
     /* request not completed */
 dcl TIMERR_STRUCT  fixed bin(31) value(TIMERR_BASE+33);
     /* time struct size */

 /* timer data types */
 define alias LPTIMECALLBACK
        entry (type UINT,
               type UINT,
               type DWORD,
               type DWORD,
               type DWORD)
        limited CALLBACK;

 /* flags for fuEvent parameter of timeSetEvent() function */
 dcl TIME_ONESHOT              fixed bin(15) value('0000'xn);
     /* program timer for single event */
 dcl TIME_PERIODIC             fixed bin(15) value('0001'xn);
     /* program for continuous periodic event */

 dcl TIME_CALLBACK_FUNCTION    fixed bin(15) value('0000'xn);
     /* callback is function */
 dcl TIME_CALLBACK_EVENT_SET   fixed bin(15) value('0010'xn);
     /* callback is event - use SetEvent */
 dcl TIME_CALLBACK_EVENT_PULSE fixed bin(15) value('0020'xn);
     /* callback is event - use PulseEvent */

 /* timer device capabilities data structure */
 define structure
   1 TIMECAPS,   /* callback is event - use PulseEvent */
     2 wPeriodMin type UINT,/* minimum period supported */
     2 wPeriodMax type UINT;/* maximum period supported */
 define alias  PTIMECAPS handle TIMECAPS;
 define alias NPTIMECAPS handle TIMECAPS;
 define alias LPTIMECAPS handle TIMECAPS;

 /* timer function prototypes */
 dcl timeGetSystemTime
     entry (type MMTIME byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('timeGetSystemTime');

 dcl timeGetTime
     entry ()
     returns (type DWORD byvalue optional)
     WINAPI WINMMAPI external('timeGetTime');

 dcl timeSetEvent
     entry (type UINT,
            type UINT,
            type LPTIMECALLBACK, /* may be entryaddr(ptrvalue(EVENT)) */
            type DWORD,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('timeSetEvent');

 dcl timeKillEvent
     entry (type UINT )
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('timeKillEvent');

 dcl timeGetDevCaps
     entry (type TIMECAPS byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('timeGetDevCaps');

 dcl timeBeginPeriod
     entry (type UINT )
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('timeBeginPeriod');

 dcl timeEndPeriod
     entry (type UINT )
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('timeEndPeriod');

 %end;  /* ifndef MMNOTIMER */

 %dcl MMNOJOY char ext;
 %if MMNOJOY = '' %then %do;
 /*********************************************************************/
 /*                                                                   */
 /*              Joystick support                                     */
 /*                                                                   */
 /*********************************************************************/

 /* joystick error return values */
 dcl JOYERR_NOERROR   fixed bin(31) value(0);
     /* no error */
 dcl JOYERR_PARMS     fixed bin(31) value(JOYERR_BASE+5);
     /* bad parameters */
 dcl JOYERR_NOCANDO   fixed bin(31) value(JOYERR_BASE+6);
     /* request not completed */
 dcl JOYERR_UNPLUGGED fixed bin(31) value(JOYERR_BASE+7);
     /* joystick is unplugged */

 /* constants used with JOYINFO and JOYINFOEX structures */
 /* and MM_JOY* messages                                 */
 dcl JOY_BUTTON1        fixed bin(15) value('0001'xn);
 dcl JOY_BUTTON2        fixed bin(15) value('0002'xn);
 dcl JOY_BUTTON3        fixed bin(15) value('0004'xn);
 dcl JOY_BUTTON4        fixed bin(15) value('0008'xn);
 dcl JOY_BUTTON1CHG     fixed bin(15) value('0100'xn);
 dcl JOY_BUTTON2CHG     fixed bin(15) value('0200'xn);
 dcl JOY_BUTTON3CHG     fixed bin(15) value('0400'xn);
 dcl JOY_BUTTON4CHG     fixed bin(15) value('0800'xn);

 /* constants used with JOYINFOEX */
 dcl JOY_BUTTON5        fixed bin(31) value('00000010'xn);
 dcl JOY_BUTTON6        fixed bin(31) value('00000020'xn);
 dcl JOY_BUTTON7        fixed bin(31) value('00000040'xn);
 dcl JOY_BUTTON8        fixed bin(31) value('00000080'xn);
 dcl JOY_BUTTON9        fixed bin(31) value('00000100'xn);
 dcl JOY_BUTTON10       fixed bin(31) value('00000200'xn);
 dcl JOY_BUTTON11       fixed bin(31) value('00000400'xn);
 dcl JOY_BUTTON12       fixed bin(31) value('00000800'xn);
 dcl JOY_BUTTON13       fixed bin(31) value('00001000'xn);
 dcl JOY_BUTTON14       fixed bin(31) value('00002000'xn);
 dcl JOY_BUTTON15       fixed bin(31) value('00004000'xn);
 dcl JOY_BUTTON16       fixed bin(31) value('00008000'xn);
 dcl JOY_BUTTON17       fixed bin(31) value('00010000'xn);
 dcl JOY_BUTTON18       fixed bin(31) value('00020000'xn);
 dcl JOY_BUTTON19       fixed bin(31) value('00040000'xn);
 dcl JOY_BUTTON20       fixed bin(31) value('00080000'xn);
 dcl JOY_BUTTON21       fixed bin(31) value('00100000'xn);
 dcl JOY_BUTTON22       fixed bin(31) value('00200000'xn);
 dcl JOY_BUTTON23       fixed bin(31) value('00400000'xn);
 dcl JOY_BUTTON24       fixed bin(31) value('00800000'xn);
 dcl JOY_BUTTON25       fixed bin(31) value('01000000'xn);
 dcl JOY_BUTTON26       fixed bin(31) value('02000000'xn);
 dcl JOY_BUTTON27       fixed bin(31) value('04000000'xn);
 dcl JOY_BUTTON28       fixed bin(31) value('08000000'xn);
 dcl JOY_BUTTON29       fixed bin(31) value('10000000'xn);
 dcl JOY_BUTTON30       fixed bin(31) value('20000000'xn);
 dcl JOY_BUTTON31       fixed bin(31) value('40000000'xn);
 dcl JOY_BUTTON32       fixed bin(31) value('80000000'xn);

 /* constants used with JOYINFOEX structure */
 dcl JOY_POVCENTERED    fixed bin(31) value(-1 );
 dcl JOY_POVFORWARD     fixed bin(31) value(0);
 dcl JOY_POVRIGHT       fixed bin(31) value(9000);
 dcl JOY_POVBACKWARD    fixed bin(31) value(18000);
 dcl JOY_POVLEFT        fixed bin(31) value(27000);

 dcl JOY_RETURNX        fixed bin(31) value('00000001'xn);
 dcl JOY_RETURNY        fixed bin(31) value('00000002'xn);
 dcl JOY_RETURNZ        fixed bin(31) value('00000004'xn);
 dcl JOY_RETURNR        fixed bin(31) value('00000008'xn);
 dcl JOY_RETURNU        fixed bin(31) value('00000010'xn); /* axis 5 */
 dcl JOY_RETURNV        fixed bin(31) value('00000020'xn); /* axis 6 */
 dcl JOY_RETURNPOV      fixed bin(31) value('00000040'xn);
 dcl JOY_RETURNBUTTONS  fixed bin(31) value('00000080'xn);
 dcl JOY_RETURNRAWDATA  fixed bin(31) value('00000100'xn);
 dcl JOY_RETURNPOVCTS   fixed bin(31) value('00000200'xn);
 dcl JOY_RETURNCENTERED fixed bin(31) value('00000400'xn);
 dcl JOY_USEDEADZONE    fixed bin(31) value('00000800'xn);
 dcl JOY_RETURNALL      fixed bin(31) value(ior(JOY_RETURNX,
                                                JOY_RETURNY,
                                                JOY_RETURNZ,
                                                JOY_RETURNR,
                                                JOY_RETURNU,
                                                JOY_RETURNV,
                                                JOY_RETURNPOV,
                                                JOY_RETURNBUTTONS));
 dcl JOY_CAL_READALWAYS fixed bin(31) value('00010000'xn);
 dcl JOY_CAL_READXYONLY fixed bin(31) value('00020000'xn);
 dcl JOY_CAL_READ3      fixed bin(31) value('00040000'xn);
 dcl JOY_CAL_READ4      fixed bin(31) value('00080000'xn);
 dcl JOY_CAL_READXONLY  fixed bin(31) value('00100000'xn);
 dcl JOY_CAL_READYONLY  fixed bin(31) value('00200000'xn);
 dcl JOY_CAL_READ5      fixed bin(31) value('00400000'xn);
 dcl JOY_CAL_READ6      fixed bin(31) value('00800000'xn);
 dcl JOY_CAL_READZONLY  fixed bin(31) value('01000000'xn);
 dcl JOY_CAL_READRONLY  fixed bin(31) value('02000000'xn);
 dcl JOY_CAL_READUONLY  fixed bin(31) value('04000000'xn);
 dcl JOY_CAL_READVONLY  fixed bin(31) value('08000000'xn);

 /* joystick ID constants */
 dcl JOYSTICKID1        fixed bin(31) value(0);
 dcl JOYSTICKID2        fixed bin(31) value(1);

 /* joystick driver capabilites */
 dcl JOYCAPS_HASZ       fixed bin(15) value('0001'xn);
 dcl JOYCAPS_HASR       fixed bin(15) value('0002'xn);
 dcl JOYCAPS_HASU       fixed bin(15) value('0004'xn);
 dcl JOYCAPS_HASV       fixed bin(15) value('0008'xn);
 dcl JOYCAPS_HASPOV     fixed bin(15) value('0010'xn);
 dcl JOYCAPS_POV4DIR    fixed bin(15) value('0020'xn);
 dcl JOYCAPS_POVCTS     fixed bin(15) value('0040'xn);

 /* joystick device capabilities data structure */
 define structure
   1 JOYCAPSA,
     2 wMid        type WORD,/* manufacturer ID */
     2 wPid        type WORD,/* product ID */
     2 szPname     char (MAXPNAMELEN-1) varz,
                             /* product name (NULL terminated string) */
     2 wXmin       type UINT,/* minimum x position value */
     2 wXmax       type UINT,/* maximum x position value */
     2 wYmin       type UINT,/* minimum y position value */
     2 wYmax       type UINT,/* maximum y position value */
     2 wZmin       type UINT,/* minimum z position value */
     2 wZmax       type UINT,/* maximum z position value */
     2 wNumButtons type UINT,/* number of buttons */
     2 wPeriodMin  type UINT,/* minimum message period when captured */
     2 wPeriodMax  type UINT /* maximum message period when captured */
 %if WINVER >= 1024 %then %do;
   , 2 wRmin       type UINT,/* minimum r position value */
     2 wRmax       type UINT,/* maximum r position value */
     2 wUmin       type UINT,/* minimum u (5th axis) position value */
     2 wUmax       type UINT,/* maximum u (5th axis) position value */
     2 wVmin       type UINT,/* minimum v (6th axis) position value */
     2 wVmax       type UINT,/* maximum v (6th axis) position value */
     2 wCaps       type UINT,/* joystick capabilites */
     2 wMaxAxes    type UINT,/* maximum number of axes supported */
     2 wNumAxes    type UINT,/* number of axes in use */
     2 wMaxButtons type UINT,/* maximum number of buttons supported */
     2 szRegKey    char (MAXPNAMELEN-1) varz,/* registry key */
     2 szOEMVxD    char (MAX_JOYSTICKOEMVXDNAME-1) varz
                             /* OEM VxD in use */
 %end;
   ;
 define alias  PJOYCAPSA handle JOYCAPSA;
 define alias NPJOYCAPSA handle JOYCAPSA;
 define alias LPJOYCAPSA handle JOYCAPSA;

 define structure
   1 JOYCAPSW,
     2 wMid        type WORD,/* manufacturer ID */
     2 wPid        type WORD,/* product ID */
     2 szPname     WCHAR (MAXPNAMELEN-1) varz,
                             /* product name (NULL terminated string) */
     2 wXmin       type UINT,/* minimum x position value */
     2 wXmax       type UINT,/* maximum x position value */
     2 wYmin       type UINT,/* minimum y position value */
     2 wYmax       type UINT,/* maximum y position value */
     2 wZmin       type UINT,/* minimum z position value */
     2 wZmax       type UINT,/* maximum z position value */
     2 wNumButtons type UINT,/* number of buttons */
     2 wPeriodMin  type UINT,/* minimum message period when captured */
     2 wPeriodMax  type UINT /* maximum message period when captured */
 %if WINVER >= 1024 %then %do;
   , 2 wRmin       type UINT,/* minimum r position value */
     2 wRmax       type UINT,/* maximum r position value */
     2 wUmin       type UINT,/* minimum u (5th axis) position value */
     2 wUmax       type UINT,/* maximum u (5th axis) position value */
     2 wVmin       type UINT,/* minimum v (6th axis) position value */
     2 wVmax       type UINT,/* maximum v (6th axis) position value */
     2 wCaps       type UINT,/* joystick capabilites */
     2 wMaxAxes    type UINT,/* maximum number of axes supported */
     2 wNumAxes    type UINT,/* number of axes in use */
     2 wMaxButtons type UINT,/* maximum number of buttons supported */
     2 szRegKey    WCHAR (MAXPNAMELEN-1) varz,/* registry key */
     2 szOEMVxD    WCHAR (MAX_JOYSTICKOEMVXDNAME-1) varz
                             /* OEM VxD in use */
 %end;
   ;
 define alias  PJOYCAPSW handle JOYCAPSW;
 define alias NPJOYCAPSW handle JOYCAPSW;
 define alias LPJOYCAPSW handle JOYCAPSW;

 %if UNICODE ^= '' %then %do;
 define alias   JOYCAPS type   JOYCAPSW;
 define alias  PJOYCAPS type  PJOYCAPSW;
 define alias NPJOYCAPS type NPJOYCAPSW;
 define alias LPJOYCAPS type LPJOYCAPSW;
 %end; %else %do;
 define alias   JOYCAPS type   JOYCAPSA;
 define alias  PJOYCAPS type  PJOYCAPSA;
 define alias NPJOYCAPS type NPJOYCAPSA;
 define alias LPJOYCAPS type LPJOYCAPSA;
 %end; /* UNICODE */


 /* joystick information data structure */
 define structure
   1 JOYINFO,
     2 wXpos    type UINT,/* x position */
     2 wYpos    type UINT,/* y position */
     2 wZpos    type UINT,/* z position */
     2 wButtons type UINT;/* button states */
 define alias  PJOYINFO handle JOYINFO;
 define alias NPJOYINFO handle JOYINFO;
 define alias LPJOYINFO handle JOYINFO;


 %if WINVER >= 1024 %then %do;
 define structure
   1 JOYINFOEX,
     2 dwSize         type DWORD,/* size of structure */
     2 dwFlags        type DWORD,/* flags to indicate what to return */
     2 dwXpos         type DWORD,/* x position */
     2 dwYpos         type DWORD,/* y position */
     2 dwZpos         type DWORD,/* z position */
     2 dwRpos         type DWORD,/* rudder/4th axis position */
     2 dwUpos         type DWORD,/* 5th axis position */
     2 dwVpos         type DWORD,/* 6th axis position */
     2 dwButtons      type DWORD,/* button states */
     2 dwButtonNumber type DWORD,/* current button number pressed */
     2 dwPOV          type DWORD,/* point of view state */
     2 dwReserved1    type DWORD,
       /* reserved for communication between winmm & driver */
     2 dwReserved2    type DWORD;/* reserved for future expansion */

 define alias  PJOYINFOEX handle JOYINFOEX;
 define alias NPJOYINFOEX handle JOYINFOEX;
 define alias LPJOYINFOEX handle JOYINFOEX;
 %end; /* WINVER >= 0x0400 */


 /* joystick function prototypes */
 dcl joyGetNumDevs
     entry ()
     returns (type UINT byvalue optional)
     WINAPI WINMMAPI external('joyGetNumDevs');

 dcl joyGetDevCapsA
     entry (type UINT,
            type JOYCAPSA byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('joyGetDevCapsA');
 dcl joyGetDevCapsW
     entry (type UINT,
            type JOYCAPSW byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('joyGetDevCapsW');

 %dcl joyGetDevCaps char ext;
 %if UNICODE ^= '' %then %do;
   %joyGetDevCaps='joyGetDevCapsW';
 %end; %else %do;
   %joyGetDevCaps='joyGetDevCapsA';
 %end; /* !UNICODE */

 dcl joyGetPos
     entry (type UINT,
            type JOYINFO byaddr)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('joyGetPos');


 %if WINVER >= 1024 %then %do;
 dcl joyGetPosEx
     entry (type UINT,
            type JOYINFOEX byaddr)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('joyGetPosEx');
 %end; /* WINVER >= 0x0400 */

 dcl joyGetThreshold
     entry (type UINT,
            type UINT byaddr)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('joyGetThreshold');

 dcl joyReleaseCapture
     entry (type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('joyReleaseCapture');

 dcl joySetCapture
     entry (type HWND,
            type UINT,
            type UINT,
            type BOOL)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('joySetCapture');

 dcl joySetThreshold
     entry (type UINT,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('joySetThreshold');

 %end;  /* ifndef MMNOJOY */

 %dcl MMNOMMIO char ext;
 %if MMNOMMIO = '' %then %do;
 /*********************************************************************/
 /*                                                                   */
 /*          Multimedia File I/O support                              */
 /*                                                                   */
 /*********************************************************************/

 /* MMIO error return values */
 dcl MMIOERR_BASE             fixed bin(31) value(256);
 dcl MMIOERR_FILENOTFOUND     fixed bin(31) value(MMIOERR_BASE + 1);
     /* file not found */
 dcl MMIOERR_OUTOFMEMORY      fixed bin(31) value(MMIOERR_BASE + 2);
     /* out of memory */
 dcl MMIOERR_CANNOTOPEN       fixed bin(31) value(MMIOERR_BASE + 3);
     /* cannot open */
 dcl MMIOERR_CANNOTCLOSE      fixed bin(31) value(MMIOERR_BASE + 4);
     /* cannot close */
 dcl MMIOERR_CANNOTREAD       fixed bin(31) value(MMIOERR_BASE + 5);
     /* cannot read */
 dcl MMIOERR_CANNOTWRITE      fixed bin(31) value(MMIOERR_BASE + 6);
     /* cannot write */
 dcl MMIOERR_CANNOTSEEK       fixed bin(31) value(MMIOERR_BASE + 7);
     /* cannot seek */
 dcl MMIOERR_CANNOTEXPAND     fixed bin(31) value(MMIOERR_BASE + 8);
     /* cannot expand file */
 dcl MMIOERR_CHUNKNOTFOUND    fixed bin(31) value(MMIOERR_BASE + 9);
     /* chunk not found */
 dcl MMIOERR_UNBUFFERED       fixed bin(31) value(MMIOERR_BASE + 10);
     /*  */
 dcl MMIOERR_PATHNOTFOUND     fixed bin(31) value(MMIOERR_BASE + 11);
     /* path incorrect */
 dcl MMIOERR_ACCESSDENIED     fixed bin(31) value(MMIOERR_BASE + 12);
     /* file was protected */
 dcl MMIOERR_SHARINGVIOLATION fixed bin(31) value(MMIOERR_BASE + 13);
     /* file in use */
 dcl MMIOERR_NETWORKERROR     fixed bin(31) value(MMIOERR_BASE + 14);
     /* network not responding */
 dcl MMIOERR_TOOMANYOPENFILES fixed bin(31) value(MMIOERR_BASE + 15);
     /* no more file handles  */
 dcl MMIOERR_INVALIDFILE      fixed bin(31) value(MMIOERR_BASE + 16);
     /* default error file error */

 /* MMIO constants */
 dcl CFSEPCHAR char value ('+');
     /* compound file name separator char. */

 /* MMIO data types */
 define alias FOURCC type DWORD; /* a four character code */
 define alias HPSTR  pointer;    /* a huge version of LPSTR */
 DECLARE_HANDLE (HMMIO);         /* a handle to an open file */

 define alias LPMMIOPROC
        entry (type LPSTR,
               type UINT,
               type LPARAM,
               type LPARAM)
        returns (type LRESULT)
        limited CALLBACK;

 /* general MMIO information data structure */
 define structure
   1 MMIOINFO,
     /* general fields */
     2 dwFlags     type DWORD,/* general status flags */
     2 fccIOProc   type FOURCC,/* pointer to I/O procedure */
     2 pIOProc     type LPMMIOPROC,/* pointer to I/O procedure */
     2 wErrorRet   type UINT, /* place for error to be returned */
     2 htask       type HTASK,/* alternate local task */
     /* fields maintained by MMIO functions during buffered I/O */
     2 cchBuffer   type LONG, /* size of I/O buffer (or 0L) */
     2 pchBuffer   type HPSTR,/* start of I/O buffer (or NULL) */
     2 pchNext     type HPSTR,/* pointer to next byte to read/write */
     2 pchEndRead  type HPSTR,/* pointer to last valid byte to read */
     2 pchEndWrite type HPSTR,/* pointer to last byte to write */
     2 lBufOffset  type LONG, /* disk offset of start of buffer */
     /* fields maintained by I/O procedure */
     2 lDiskOffset type LONG, /* disk offset of next read or write */
     2 adwInfo (3) type DWORD,/* data specific to type of MMIOPROC */
     /* other fields maintained by MMIO */
     2 dwReserved1 type DWORD,/* reserved for MMIO use */
     2 dwReserved2 type DWORD,/* reserved for MMIO use */
     2 hmmio       type HMMIO;/* handle to open file */
 define alias   PMMIOINFO handle MMIOINFO;
 define alias  NPMMIOINFO handle MMIOINFO;
 define alias  LPMMIOINFO handle MMIOINFO;
 define alias LPCMMIOINFO handle MMIOINFO;

 /* RIFF chunk information data structure */
 define structure
   1 MMCKINFO,
     2 ckid         type FOURCC,/* chunk ID */
     2 cksize       type DWORD, /* chunk size */
     2 fccType      type FOURCC,/* form type or list type */
     2 dwDataOffset type DWORD, /* offset of data portion of chunk */
     2 dwFlags      type DWORD; /* flags used by MMIO functions */
 define alias   PMMCKINFO handle MMCKINFO;
 define alias  NPMMCKINFO handle MMCKINFO;
 define alias  LPMMCKINFO handle MMCKINFO;
 define alias LPCMMCKINFO handle MMCKINFO;

 /* bit field masks */
 dcl MMIO_RWMODE      fixed bin(31) value('00000003'xn);
     /* open file for reading/writing/both */
 dcl MMIO_SHAREMODE   fixed bin(31) value('00000070'xn);
     /* file sharing mode number */

 /* constants for dwFlags field of MMIOINFO */
 dcl MMIO_CREATE      fixed bin(31) value('00001000'xn);
     /* create new file (or truncate file) */
 dcl MMIO_PARSE       fixed bin(31) value('00000100'xn);
     /* parse new file returning path */
 dcl MMIO_DELETE      fixed bin(31) value('00000200'xn);
     /* create new file (or truncate file) */
 dcl MMIO_EXIST       fixed bin(31) value('00004000'xn);
     /* checks for existence of file */
 dcl MMIO_ALLOCBUF    fixed bin(31) value('00010000'xn);
     /* mmioOpen() should allocate a buffer */
 dcl MMIO_GETTEMP     fixed bin(31) value('00020000'xn);
     /* mmioOpen() should retrieve temp name */

 dcl MMIO_DIRTY       fixed bin(31) value('10000000'xn);
     /* I/O buffer is dirty */

 /* read/write mode numbers (bit field MMIO_RWMODE) */
 dcl MMIO_READ        fixed bin(31) value('00000000'xn);
     /* open file for reading only */
 dcl MMIO_WRITE       fixed bin(31) value('00000001'xn);
     /* open file for writing only */
 dcl MMIO_READWRITE   fixed bin(31) value('00000002'xn);
     /* open file for reading and writing */

 /* share mode numbers (bit field MMIO_SHAREMODE) */
 dcl MMIO_COMPAT      fixed bin(31) value('00000000'xn);
     /* compatibility mode */
 dcl MMIO_EXCLUSIVE   fixed bin(31) value('00000010'xn);
     /* exclusive-access mode */
 dcl MMIO_DENYWRITE   fixed bin(31) value('00000020'xn);
     /* deny writing to other processes */
 dcl MMIO_DENYREAD    fixed bin(31) value('00000030'xn);
     /* deny reading to other processes */
 dcl MMIO_DENYNONE    fixed bin(31) value('00000040'xn);
     /* deny nothing to other processes */

 /* various MMIO flags */
 dcl MMIO_FHOPEN      fixed bin(31) value('0010'xn);
     /* mmioClose: keep file handle open */
 dcl MMIO_EMPTYBUF    fixed bin(31) value('0010'xn);
     /* mmioFlush: empty the I/O buffer */
 dcl MMIO_TOUPPER     fixed bin(31) value('0010'xn);
     /* mmioStringToFOURCC: to u-case */
 dcl MMIO_INSTALLPROC fixed bin(31) value('00010000'xn);
     /* mmioInstallIOProc: install MMIOProc */
 dcl MMIO_GLOBALPROC  fixed bin(31) value('10000000'xn);
     /* mmioInstallIOProc: install globally */
 dcl MMIO_REMOVEPROC  fixed bin(31) value('00020000'xn);
     /* mmioInstallIOProc: remove MMIOProc */
 dcl MMIO_UNICODEPROC fixed bin(31) value('01000000'xn);
     /* mmioInstallIOProc: Unicode MMIOProc */
 dcl MMIO_FINDPROC    fixed bin(31) value('00040000'xn);
     /* mmioInstallIOProc: find an MMIOProc */
 dcl MMIO_FINDCHUNK   fixed bin(31) value('0010'xn);
     /* mmioDescend: find a chunk by ID */
 dcl MMIO_FINDRIFF    fixed bin(31) value('0020'xn);
     /* mmioDescend: find a LIST chunk */
 dcl MMIO_FINDLIST    fixed bin(31) value('0040'xn);
     /* mmioDescend: find a RIFF chunk */
 dcl MMIO_CREATERIFF  fixed bin(31) value('0020'xn);
     /* mmioCreateChunk: make a LIST chunk */
 dcl MMIO_CREATELIST  fixed bin(31) value('0040'xn);
     /* mmioCreateChunk: make a RIFF chunk */

 /* message numbers for MMIOPROC I/O procedure functions */
 dcl MMIOM_READ       fixed bin(31) value (MMIO_READ);  /* read */
 dcl MMIOM_WRITE      fixed bin(31) value (MMIO_WRITE); /* write */
 dcl MMIOM_SEEK       fixed bin(31) value(2);
     /* seek to a new position in file */
 dcl MMIOM_OPEN       fixed bin(31) value(3); /* open file */
 dcl MMIOM_CLOSE      fixed bin(31) value(4); /* close file */
 dcl MMIOM_WRITEFLUSH fixed bin(31) value(5); /* write and flush */


 %if WINVER >= 778 %then %do;
 dcl MMIOM_RENAME fixed bin(31) value(6); /* rename specified file */
 %end; /* ifdef WINVER >= 0x030a */

 dcl MMIOM_USER fixed bin(31) value('8000'xn);
     /* beginning of user-defined messages */

 /* MMIO macros */
 %mmioFOURCC: proc (ch0, ch1, ch2, ch3) returns (char);
   dcl (ch0, ch1, ch2, ch3) char;
   return (MAKEFOURCC (ch0, ch1, ch2, ch3));
 %end mmioFOURCC;
 %activate mmioFOURCC;

 /* standard four character codes */
 dcl FOURCC_RIFF type FOURCC value (mmioFOURCC('R', 'I', 'F', 'F'));
 dcl FOURCC_LIST type FOURCC value (mmioFOURCC('L', 'I', 'S', 'T'));

 /* four character codes used to identify standard */
 /* built-in I/O procedures                        */
 dcl FOURCC_DOS  type FOURCC value (mmioFOURCC('D', 'O', 'S', ' '));
 dcl FOURCC_MEM  type FOURCC value (mmioFOURCC('M', 'E', 'M', ' '));

 /* flags for mmioSeek() */
 %dcl #SEEK_SET char ext;
 %if #SEEK_SET = '' %then %do;
 %#SEEK_SET = 'Y';
 dcl SEEK_SET fixed bin(31) value(0);
     /* seek to an absolute position */
 dcl SEEK_CUR fixed bin(31) value(1);
     /* seek relative to current position */
 dcl SEEK_END fixed bin(31) value(2);
     /* seek relative to end of file */
 %end; /* ifndef SEEK_SET */

 /* other constants */
 dcl MMIO_DEFAULTBUFFER fixed bin(31) value(8192);
    /* default buffer size */


 /* MMIO function prototypes */

 dcl mmioStringToFOURCCA
     entry (char (*) varz byaddr nonasgn,
            type UINT)
     returns (type FOURCC byvalue optional)
     WINAPI WINMMAPI external('mmioStringToFOURCCA');
 dcl mmioStringToFOURCCW
     entry (WCHAR (*) varz byaddr nonasgn,
            type UINT)
     returns (type FOURCC byvalue optional)
     WINAPI WINMMAPI external('mmioStringToFOURCCW');

 %dcl mmioStringToFOURCC char ext;
 %if UNICODE ^= '' %then %do;
   %mmioStringToFOURCC='mmioStringToFOURCCW';
 %end; %else %do;
   %mmioStringToFOURCC='mmioStringToFOURCCA';
 %end; /* !UNICODE */

 dcl mmioInstallIOProcA
     entry (type FOURCC,
            type LPMMIOPROC optional byvalue,
            type DWORD)
     returns (type LPMMIOPROC byvalue optional)
     WINAPI WINMMAPI external('mmioInstallIOProcA');
 dcl mmioInstallIOProcW
     entry (type FOURCC,
            type LPMMIOPROC optional byvalue,
            type DWORD)
     returns (type LPMMIOPROC byvalue optional)
     WINAPI WINMMAPI external('mmioInstallIOProcW');

 %dcl mmioInstallIOProc char ext;
 %if UNICODE ^= '' %then %do;
   %mmioInstallIOProc='mmioInstallIOProcW';
 %end; %else %do;
   %mmioInstallIOProc='mmioInstallIOProcA';
 %end; /* !UNICODE */

 dcl mmioOpenA
     entry (char (*) varz byaddr optional,
            type MMIOINFO byaddr optional,
            type DWORD)
     returns (type HMMIO byvalue optional)
     WINAPI WINMMAPI external('mmioOpenA');
 dcl mmioOpenW
     entry (WCHAR (*) varz byaddr optional,
            type MMIOINFO byaddr optional,
            type DWORD)
     returns (type HMMIO byvalue optional)
     WINAPI WINMMAPI external('mmioOpenW');

 %dcl mmioOpen char ext;
 %if UNICODE ^= '' %then %do;
   %mmioOpen='mmioOpenW';
 %end; %else %do;
   %mmioOpen='mmioOpenA';
 %end; /* !UNICODE */

 dcl mmioRenameA
     entry (char (*) varz byaddr nonasgn,
            char (*) varz byaddr nonasgn,
            type MMIOINFO byaddr nonasgn optional,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('mmioRenameA');
 dcl mmioRenameW
     entry (WCHAR (*) varz byaddr nonasgn,
            WCHAR (*) varz byaddr nonasgn,
            type MMIOINFO byaddr nonasgn optional,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('mmioRenameW');

 %dcl mmioRename char ext;
 %if UNICODE ^= '' %then %do;
   %mmioRename='mmioRenameW';
 %end; %else %do;
   %mmioRename='mmioRenameA';
 %end; /* !UNICODE */

 dcl mmioClose
     entry (type HMMIO,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('mmioClose');

 dcl mmioRead
     entry (type HMMIO,
            type HPSTR,
            type LONG)
     returns (type LONG byvalue optional)
     WINAPI WINMMAPI external('mmioRead');

 dcl mmioWrite
     entry (type HMMIO,
            pointer, /* to buffer to write */
            type LONG)
     returns (type LONG byvalue optional)
     WINAPI WINMMAPI external('mmioWrite');

 dcl mmioSeek
     entry (type HMMIO,
            type LONG,
            fixed bin(31))
     returns (type LONG byvalue optional)
     WINAPI WINMMAPI external('mmioSeek');

 dcl mmioGetInfo
     entry (type HMMIO,
            type MMIOINFO byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('mmioGetInfo');

 dcl mmioSetInfo
     entry (type HMMIO,
            type MMIOINFO byaddr nonasgn,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('mmioSetInfo');

 dcl mmioSetBuffer
     entry (type HMMIO,
            type LPSTR, /*to buffer, or sysnull */
            type LONG,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('mmioSetBuffer');

 dcl mmioFlush
     entry (type HMMIO,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('mmioFlush');

 dcl mmioAdvance
     entry (type HMMIO,
            type MMIOINFO byaddr optional,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('mmioAdvance');

 dcl mmioSendMessage
     entry (type HMMIO,
            type UINT,
            type LPARAM,
            type LPARAM)
     returns (type LRESULT byvalue optional)
     WINAPI WINMMAPI external('mmioSendMessage');

 dcl mmioDescend
     entry (type HMMIO,
            type MMCKINFO byaddr,
            type MMCKINFO byaddr nonasgn optional,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('mmioDescend');

 dcl mmioAscend
     entry (type HMMIO,
            type MMCKINFO byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('mmioAscend');

 dcl mmioCreateChunk
     entry (type HMMIO,
            type MMCKINFO byaddr,
            type UINT)
     returns (type MMRESULT byvalue optional)
     WINAPI WINMMAPI external('mmioCreateChunk');

 %end; /* ifndef MMNOMMIO */

 %dcl MMNOMCI char ext;
 %if MMNOMCI = '' %then %do;
 /*********************************************************************/
 /*                                                                   */
 /*              MCI support                                          */
 /*                                                                   */
 /*********************************************************************/

 %dcl #MCIERROR_ char ext;
 %if #MCIERROR_ = '' %then %do;
 %#MCIERROR_ = 'Y';
 define alias MCIERROR type DWORD;
   /* error return code, 0 means no error */
 %end;

 %dcl #MCIDEVICEID_ char ext;
 %if #MCIDEVICEID_ = '' %then %do;
 %#MCIDEVICEID_ = 'Y';
 define alias MCIDEVICEID type UINT; /* MCI device ID type */
 %end;

 define alias YIELDPROC
        entry (type MCIDEVICEID,
               type DWORD)
        returns (type UINT byvalue optional)
        limited CALLBACK ;

 /* MCI function prototypes */

 dcl mciSendCommandA
     entry (type MCIDEVICEID,
            type UINT,
            type DWORD,
            type DWORD)
     returns (type MCIERROR byvalue optional)
     WINAPI WINMMAPI external('mciSendCommandA');
 dcl mciSendCommandW
     entry (type MCIDEVICEID,
            type UINT,
            type DWORD,
            type DWORD)
     returns (type MCIERROR byvalue optional)
     WINAPI WINMMAPI external('mciSendCommandW');

 %dcl mciSendCommand char ext;
 %if UNICODE ^= '' %then %do;
   %mciSendCommand='mciSendCommandW';
 %end; %else %do;
   %mciSendCommand='mciSendCommandA';
 %end; /* !UNICODE */

 dcl mciSendStringA
     entry (char (*) varz byaddr nonasgn,
            char (*) varz byaddr optional,
            type UINT,
            type HWND)
     returns (type MCIERROR byvalue optional)
     WINAPI WINMMAPI external('mciSendStringA');
 dcl mciSendStringW
     entry (WCHAR (*) varz byaddr nonasgn,
            WCHAR (*) varz byaddr optional,
            type UINT,
            type HWND)
     returns (type MCIERROR byvalue optional)
     WINAPI WINMMAPI external('mciSendStringW');

 %dcl mciSendString char ext;
 %if UNICODE ^= '' %then %do;
   %mciSendString='mciSendStringW';
 %end; %else %do;
   %mciSendString='mciSendStringA';
 %end; /* !UNICODE */

 dcl mciGetDeviceIDA
     entry (char (*) varz byaddr nonasgn)
     returns (type MCIDEVICEID byvalue optional)
     WINAPI WINMMAPI external('mciGetDeviceIDA');
 dcl mciGetDeviceIDW
     entry (WCHAR (*) varz byaddr nonasgn)
     returns (type MCIDEVICEID byvalue optional)
     WINAPI WINMMAPI external('mciGetDeviceIDW');

 %dcl mciGetDeviceID char ext;
 %if UNICODE ^= '' %then %do;
   %mciGetDeviceID='mciGetDeviceIDW';
 %end; %else %do;
   %mciGetDeviceID='mciGetDeviceIDA';
 %end; /* !UNICODE */

 dcl mciGetDeviceIDFromElementIDA
     entry (type DWORD,
            char (*) varz byaddr nonasgn)
     returns (type MCIDEVICEID byvalue optional)
     WINAPI WINMMAPI external('mciGetDeviceIDFromElementIDA');
 dcl mciGetDeviceIDFromElementIDW
     entry (type DWORD,
            WCHAR (*) varz byaddr nonasgn)
     returns (type MCIDEVICEID byvalue optional)
     WINAPI WINMMAPI external('mciGetDeviceIDFromElementIDW');

 %dcl mciGetDeviceIDFromElementID char ext;
 %if UNICODE ^= '' %then %do;
   %mciGetDeviceIDFromElementID='mciGetDeviceIDFromElementIDW';
 %end; %else %do;
   %mciGetDeviceIDFromElementID='mciGetDeviceIDFromElementIDA';
 %end;  /* !UNICODE */

 dcl mciGetErrorStringA
     entry (type MCIERROR,
            char (*) varz byaddr,
            type UINT)
     returns (type BOOL byvalue optional)
     WINAPI WINMMAPI external('mciGetErrorStringA');
 dcl mciGetErrorStringW
     entry (type MCIERROR,
            WCHAR (*) varz byaddr,
            type UINT)
     returns (type BOOL byvalue optional)
     WINAPI WINMMAPI external('mciGetErrorStringW');

 %dcl mciGetErrorString char ext;
 %if UNICODE ^= '' %then %do;
   %mciGetErrorString='mciGetErrorStringW';
 %end; %else %do;
   %mciGetErrorString='mciGetErrorStringA';
 %end; /* !UNICODE */

 dcl mciSetYieldProc
     entry (type MCIDEVICEID,
            type YIELDPROC optional byvalue,
            type DWORD)
     returns (type BOOL byvalue optional)
     WINAPI WINMMAPI external('mciSetYieldProc');


 %if WINVER >= 778 %then %do;
 dcl mciGetCreatorTask
     entry (type MCIDEVICEID)
     returns (type HTASK byvalue optional)
     WINAPI WINMMAPI external('mciGetCreatorTask');

 dcl mciGetYieldProc
     entry (type MCIDEVICEID,
            type DWORD byaddr optional)
     returns (type YIELDPROC byvalue optional)
     WINAPI WINMMAPI external('mciGetYieldProc');
 %end; /* ifdef WINVER >= 0x030a */

 %if WINVER < 778 %then %do;
 dcl mciExecute
     entry (char (*) varz byaddr nonasgn)
     returns (type BOOL byvalue optional)
     WINAPI WINMMAPI external('mciExecute');
 %end; /* ifdef WINVER < 0x030a */

 /* MCI error return values */
 dcl MCIERR_INVALID_DEVICE_ID     fixed bin(31) value(MCIERR_BASE + 1);
 dcl MCIERR_UNRECOGNIZED_KEYWORD  fixed bin(31) value(MCIERR_BASE + 3);
 dcl MCIERR_UNRECOGNIZED_COMMAND  fixed bin(31) value(MCIERR_BASE + 5);
 dcl MCIERR_HARDWARE              fixed bin(31) value(MCIERR_BASE + 6);
 dcl MCIERR_INVALID_DEVICE_NAME   fixed bin(31) value(MCIERR_BASE + 7);
 dcl MCIERR_OUT_OF_MEMORY         fixed bin(31) value(MCIERR_BASE + 8);
 dcl MCIERR_DEVICE_OPEN           fixed bin(31) value(MCIERR_BASE + 9);
 dcl MCIERR_CANNOT_LOAD_DRIVER    fixed bin(31) value(MCIERR_BASE + 10);
 dcl MCIERR_MISSING_COMMAND_STRING
                                  fixed bin(31) value(MCIERR_BASE + 11);
 dcl MCIERR_PARAM_OVERFLOW        fixed bin(31) value(MCIERR_BASE + 12);
 dcl MCIERR_MISSING_STRING_ARGUMENT
                                  fixed bin(31) value(MCIERR_BASE + 13);
 dcl MCIERR_BAD_INTEGER           fixed bin(31) value(MCIERR_BASE + 14);
 dcl MCIERR_PARSER_INTERNAL       fixed bin(31) value(MCIERR_BASE + 15);
 dcl MCIERR_DRIVER_INTERNAL       fixed bin(31) value(MCIERR_BASE + 16);
 dcl MCIERR_MISSING_PARAMETER     fixed bin(31) value(MCIERR_BASE + 17);
 dcl MCIERR_UNSUPPORTED_FUNCTION  fixed bin(31) value(MCIERR_BASE + 18);
 dcl MCIERR_FILE_NOT_FOUND        fixed bin(31) value(MCIERR_BASE + 19);
 dcl MCIERR_DEVICE_NOT_READY      fixed bin(31) value(MCIERR_BASE + 20);
 dcl MCIERR_INTERNAL              fixed bin(31) value(MCIERR_BASE + 21);
 dcl MCIERR_DRIVER                fixed bin(31) value(MCIERR_BASE + 22);
 dcl MCIERR_CANNOT_USE_ALL        fixed bin(31) value(MCIERR_BASE + 23);
 dcl MCIERR_MULTIPLE              fixed bin(31) value(MCIERR_BASE + 24);
 dcl MCIERR_EXTENSION_NOT_FOUND   fixed bin(31) value(MCIERR_BASE + 25);
 dcl MCIERR_OUTOFRANGE            fixed bin(31) value(MCIERR_BASE + 26);
 dcl MCIERR_FLAGS_NOT_COMPATIBLE  fixed bin(31) value(MCIERR_BASE + 28);
 dcl MCIERR_FILE_NOT_SAVED        fixed bin(31) value(MCIERR_BASE + 30);
 dcl MCIERR_DEVICE_TYPE_REQUIRED  fixed bin(31) value(MCIERR_BASE + 31);
 dcl MCIERR_DEVICE_LOCKED         fixed bin(31) value(MCIERR_BASE + 32);
 dcl MCIERR_DUPLICATE_ALIAS       fixed bin(31) value(MCIERR_BASE + 33);
 dcl MCIERR_BAD_CONSTANT          fixed bin(31) value(MCIERR_BASE + 34);
 dcl MCIERR_MUST_USE_SHAREABLE    fixed bin(31) value(MCIERR_BASE + 35);
 dcl MCIERR_MISSING_DEVICE_NAME   fixed bin(31) value(MCIERR_BASE + 36);
 dcl MCIERR_BAD_TIME_FORMAT       fixed bin(31) value(MCIERR_BASE + 37);
 dcl MCIERR_NO_CLOSING_QUOTE      fixed bin(31) value(MCIERR_BASE + 38);
 dcl MCIERR_DUPLICATE_FLAGS       fixed bin(31) value(MCIERR_BASE + 39);
 dcl MCIERR_INVALID_FILE          fixed bin(31) value(MCIERR_BASE + 40);
 dcl MCIERR_NULL_PARAMETER_BLOCK  fixed bin(31) value(MCIERR_BASE + 41);
 dcl MCIERR_UNNAMED_RESOURCE      fixed bin(31) value(MCIERR_BASE + 42);
 dcl MCIERR_NEW_REQUIRES_ALIAS    fixed bin(31) value(MCIERR_BASE + 43);
 dcl MCIERR_NOTIFY_ON_AUTO_OPEN   fixed bin(31) value(MCIERR_BASE + 44);
 dcl MCIERR_NO_ELEMENT_ALLOWED    fixed bin(31) value(MCIERR_BASE + 45);
 dcl MCIERR_NONAPPLICABLE_FUNCTION
                                  fixed bin(31) value(MCIERR_BASE + 46);
 dcl MCIERR_ILLEGAL_FOR_AUTO_OPEN fixed bin(31) value(MCIERR_BASE + 47);
 dcl MCIERR_FILENAME_REQUIRED     fixed bin(31) value(MCIERR_BASE + 48);
 dcl MCIERR_EXTRA_CHARACTERS      fixed bin(31) value(MCIERR_BASE + 49);
 dcl MCIERR_DEVICE_NOT_INSTALLED  fixed bin(31) value(MCIERR_BASE + 50);
 dcl MCIERR_GET_CD                fixed bin(31) value(MCIERR_BASE + 51);
 dcl MCIERR_SET_CD                fixed bin(31) value(MCIERR_BASE + 52);
 dcl MCIERR_SET_DRIVE             fixed bin(31) value(MCIERR_BASE + 53);
 dcl MCIERR_DEVICE_LENGTH         fixed bin(31) value(MCIERR_BASE + 54);
 dcl MCIERR_DEVICE_ORD_LENGTH     fixed bin(31) value(MCIERR_BASE + 55);
 dcl MCIERR_NO_INTEGER            fixed bin(31) value(MCIERR_BASE + 56);

 dcl MCIERR_WAVE_OUTPUTSINUSE     fixed bin(31) value(MCIERR_BASE + 64);
 dcl MCIERR_WAVE_SETOUTPUTINUSE   fixed bin(31) value(MCIERR_BASE + 65);
 dcl MCIERR_WAVE_INPUTSINUSE      fixed bin(31) value(MCIERR_BASE + 66);
 dcl MCIERR_WAVE_SETINPUTINUSE    fixed bin(31) value(MCIERR_BASE + 67);
 dcl MCIERR_WAVE_OUTPUTUNSPECIFIED
                                  fixed bin(31) value(MCIERR_BASE + 68);
 dcl MCIERR_WAVE_INPUTUNSPECIFIED fixed bin(31) value(MCIERR_BASE + 69);
 dcl MCIERR_WAVE_OUTPUTSUNSUITABLE
                                  fixed bin(31) value(MCIERR_BASE + 70);
 dcl MCIERR_WAVE_SETOUTPUTUNSUITABLE
                                  fixed bin(31) value(MCIERR_BASE + 71);
 dcl MCIERR_WAVE_INPUTSUNSUITABLE fixed bin(31) value(MCIERR_BASE + 72);
 dcl MCIERR_WAVE_SETINPUTUNSUITABLE
                                  fixed bin(31) value(MCIERR_BASE + 73);

 dcl MCIERR_SEQ_DIV_INCOMPATIBLE  fixed bin(31) value(MCIERR_BASE + 80);
 dcl MCIERR_SEQ_PORT_INUSE        fixed bin(31) value(MCIERR_BASE + 81);
 dcl MCIERR_SEQ_PORT_NONEXISTENT  fixed bin(31) value(MCIERR_BASE + 82);
 dcl MCIERR_SEQ_PORT_MAPNODEVICE  fixed bin(31) value(MCIERR_BASE + 83);
 dcl MCIERR_SEQ_PORT_MISCERROR    fixed bin(31) value(MCIERR_BASE + 84);
 dcl MCIERR_SEQ_TIMER             fixed bin(31) value(MCIERR_BASE + 85);
 dcl MCIERR_SEQ_PORTUNSPECIFIED   fixed bin(31) value(MCIERR_BASE + 86);
 dcl MCIERR_SEQ_NOMIDIPRESENT     fixed bin(31) value(MCIERR_BASE + 87);

 dcl MCIERR_NO_WINDOW             fixed bin(31) value(MCIERR_BASE + 90);
 dcl MCIERR_CREATEWINDOW          fixed bin(31) value(MCIERR_BASE + 91);
 dcl MCIERR_FILE_READ             fixed bin(31) value(MCIERR_BASE + 92);
 dcl MCIERR_FILE_WRITE            fixed bin(31) value(MCIERR_BASE + 93);

 dcl MCIERR_NO_IDENTITY           fixed bin(31) value(MCIERR_BASE + 94);

 /* all custom device driver errors must be >= than this value */
 dcl MCIERR_CUSTOM_DRIVER_BASE   fixed bin(31) value(MCIERR_BASE + 256);

 /* Test is appplicable to all commands - mmsystem.cpy */
 dcl MCI_FIRST          fixed bin(15) value(DRV_MCI_FIRST); /* 0x0800 */

 /* MCI command message identifiers */
 dcl MCI_OPEN           fixed bin(15) value('0803'xn);
 dcl MCI_CLOSE          fixed bin(15) value('0804'xn);
 dcl MCI_ESCAPE         fixed bin(15) value('0805'xn);
 dcl MCI_PLAY           fixed bin(15) value('0806'xn);
 dcl MCI_SEEK           fixed bin(15) value('0807'xn);
 dcl MCI_STOP           fixed bin(15) value('0808'xn);
 dcl MCI_PAUSE          fixed bin(15) value('0809'xn);
 dcl MCI_INFO           fixed bin(15) value('080A'xn);
 dcl MCI_GETDEVCAPS     fixed bin(15) value('080B'xn);
 dcl MCI_SPIN           fixed bin(15) value('080C'xn);
 dcl MCI_SET            fixed bin(15) value('080D'xn);
 dcl MCI_STEP           fixed bin(15) value('080E'xn);
 dcl MCI_RECORD         fixed bin(15) value('080F'xn);
 dcl MCI_SYSINFO        fixed bin(15) value('0810'xn);
 dcl MCI_BREAK          fixed bin(15) value('0811'xn);
 dcl MCI_SAVE           fixed bin(15) value('0813'xn);
 dcl MCI_STATUS         fixed bin(15) value('0814'xn);
 dcl MCI_CUE            fixed bin(15) value('0830'xn);
 dcl MCI_REALIZE        fixed bin(15) value('0840'xn);
 dcl MCI_WINDOW         fixed bin(15) value('0841'xn);
 dcl MCI_PUT            fixed bin(15) value('0842'xn);
 dcl MCI_WHERE          fixed bin(15) value('0843'xn);
 dcl MCI_FREEZE         fixed bin(15) value('0844'xn);
 dcl MCI_UNFREEZE       fixed bin(15) value('0845'xn);
 dcl MCI_LOAD           fixed bin(15) value('0850'xn);
 dcl MCI_CUT            fixed bin(15) value('0851'xn);
 dcl MCI_COPY           fixed bin(15) value('0852'xn);
 dcl MCI_PASTE          fixed bin(15) value('0853'xn);
 dcl MCI_UPDATE         fixed bin(15) value('0854'xn);
 dcl MCI_RESUME         fixed bin(15) value('0855'xn);
 dcl MCI_DELETE         fixed bin(15) value('0856'xn);

 /* PL/I: moved here from various other places to ensure unique defn */
 dcl MCI_CAPTURE        fixed bin(15) value('0870'xn);
 dcl MCI_MONITOR        fixed bin(15) value('0871'xn);
 dcl MCI_RESERVE        fixed bin(15) value('0872'xn);
 dcl MCI_SETAUDIO       fixed bin(15) value('0873'xn);
 dcl MCI_SIGNAL         fixed bin(15) value('0875'xn);
 dcl MCI_SETVIDEO       fixed bin(15) value('0876'xn);
 dcl MCI_QUALITY        fixed bin(15) value('0877'xn);
 dcl MCI_LIST           fixed bin(15) value('0878'xn);
 dcl MCI_UNDO           fixed bin(15) value('0879'xn);
 dcl MCI_CONFIGURE      fixed bin(15) value('087a'xn);
 dcl MCI_RESTORE        fixed bin(15) value('087b'xn);

 /* all custom MCI command messages must be >= than this value */
 dcl MCI_USER_MESSAGES  fixed bin(15) value(DRV_MCI_FIRST+'400'xn);
 dcl MCI_LAST           fixed bin(15) value('0FFF'xn);

 /* Return and string constant values */

 dcl MCI_ON                     fixed bin(31) value(1);
 dcl MCI_OFF                    fixed bin(31) value(0);

 /* device ID for "all devices" */
 dcl MCI_ALL_DEVICE_ID          fixed bin(31) value(-1);

 /* constants for predefined MCI device types */
 dcl MCI_DEVTYPE_VCR            fixed bin(31) value(513);
     /* (MCI_STRING_OFFSET + 1) */
 dcl MCI_DEVTYPE_VIDEODISC      fixed bin(31) value(514);
     /* (MCI_STRING_OFFSET + 2) */
 dcl MCI_DEVTYPE_OVERLAY        fixed bin(31) value(515);
     /* (MCI_STRING_OFFSET + 3) */
 dcl MCI_DEVTYPE_CD_AUDIO       fixed bin(31) value(516);
     /* (MCI_STRING_OFFSET + 4) */
 dcl MCI_DEVTYPE_DAT            fixed bin(31) value(517);
    /* (MCI_STRING_OFFSET + 5) */
 dcl MCI_DEVTYPE_SCANNER        fixed bin(31) value(518);
     /* (MCI_STRING_OFFSET + 6) */
 dcl MCI_DEVTYPE_ANIMATION      fixed bin(31) value(519);
     /* (MCI_STRING_OFFSET + 7) */
 dcl MCI_DEVTYPE_DIGITAL_VIDEO  fixed bin(31) value(520);
    /* (MCI_STRING_OFFSET + 8) */
 dcl MCI_DEVTYPE_OTHER          fixed bin(31) value(521);
     /* (MCI_STRING_OFFSET + 9) */
 dcl MCI_DEVTYPE_WAVEFORM_AUDIO fixed bin(31) value(522);
     /* (MCI_STRING_OFFSET + 10) */
 dcl MCI_DEVTYPE_SEQUENCER      fixed bin(31) value(523);
     /* (MCI_STRING_OFFSET + 11) */

 dcl MCI_DEVTYPE_FIRST       fixed bin(31) value(MCI_DEVTYPE_VCR);
 dcl MCI_DEVTYPE_LAST        fixed bin(31) value(MCI_DEVTYPE_SEQUENCER);

 dcl MCI_DEVTYPE_FIRST_USER  fixed bin(15) value('1000'xn);
 /* return values for 'status mode' command */
 dcl MCI_MODE_NOT_READY fixed bin(31) value(MCI_STRING_OFFSET + 12);
 dcl MCI_MODE_STOP      fixed bin(31) value(MCI_STRING_OFFSET + 13);
 dcl MCI_MODE_PLAY      fixed bin(31) value(MCI_STRING_OFFSET + 14);
 dcl MCI_MODE_RECORD    fixed bin(31) value(MCI_STRING_OFFSET + 15);
 dcl MCI_MODE_SEEK      fixed bin(31) value(MCI_STRING_OFFSET + 16);
 dcl MCI_MODE_PAUSE     fixed bin(31) value(MCI_STRING_OFFSET + 17);
 dcl MCI_MODE_OPEN      fixed bin(31) value(MCI_STRING_OFFSET + 18);

 /* constants used in 'set time format' */
 /* and 'status time format' commands   */
 dcl MCI_FORMAT_MILLISECONDS fixed bin(31) value(0);
 dcl MCI_FORMAT_HMS          fixed bin(31) value(1);
 dcl MCI_FORMAT_MSF          fixed bin(31) value(2);
 dcl MCI_FORMAT_FRAMES       fixed bin(31) value(3);
 dcl MCI_FORMAT_SMPTE_24     fixed bin(31) value(4);
 dcl MCI_FORMAT_SMPTE_25     fixed bin(31) value(5);
 dcl MCI_FORMAT_SMPTE_30     fixed bin(31) value(6);
 dcl MCI_FORMAT_SMPTE_30DROP fixed bin(31) value(7);
 dcl MCI_FORMAT_BYTES        fixed bin(31) value(8);
 dcl MCI_FORMAT_SAMPLES      fixed bin(31) value(9);
 dcl MCI_FORMAT_TMSF         fixed bin(31) value(10);

 /* MCI time format conversion macros */

 MCI_MSF_MINUTE: proc (msf) returns (type BYTE byvalue)
                 options (inline reorder) internal;
   dcl msf type DWORD byvalue;
   dcl iand builtin;
   return (iand (msf, 'ff'xn));
 end MCI_MSF_MINUTE;

 MCI_MSF_SECOND: proc (msf) returns (type BYTE byvalue)
                 options (inline reorder) internal;
   dcl msf type DWORD byvalue;
   dcl (iand, isrl) builtin;
   return (iand (isrl (msf, 8), 'ff'xn));
 end MCI_MSF_SECOND;

 MCI_MSF_FRAME: proc (msf) returns (type BYTE byvalue)
                options (inline reorder) internal;
   dcl msf type DWORD byvalue;
   dcl (iand, isrl) builtin;
   return (iand (isrl (msf, 16), 'ff'xn));
 end MCI_MSF_FRAME;

 MCI_MAKE_MSF: proc (m, s, f) returns (type DWORD byvalue)
               options (inline reorder) internal;
   dcl (m, s, f) type BYTE byvalue;
   dcl (ior, isll) builtin;
   return (ior (m, isll (s, 8), isll (f, 16)));
 end MCI_MAKE_MSF;


 MCI_TMSF_TRACK: proc (tmsf) returns (type BYTE byvalue)
                 options (inline reorder) internal;
   dcl tmsf type DWORD byvalue;
   dcl iand builtin;
   return (iand (tmsf, 'ff'xn));
 end MCI_TMSF_TRACK;

 MCI_TMSF_MINUTE: proc (tmsf) returns (type BYTE byvalue)
                  options (inline reorder) internal;
   dcl tmsf type DWORD byvalue;
   dcl (iand, isrl) builtin;
   return (iand (isrl (tmsf, 8), 'ff'xn));
 end MCI_TMSF_MINUTE;

 MCI_TMSF_SECOND: proc (tmsf) returns (type BYTE byvalue)
                  options (inline reorder) internal;
   dcl tmsf type DWORD byvalue;
   dcl (iand, isrl) builtin;
   return (iand (isrl (tmsf, 16), 'ff'xn));
 end MCI_TMSF_SECOND;

 MCI_TMSF_FRAME: proc (tmsf) returns (type BYTE byvalue)
                 options (inline reorder) internal;
   dcl tmsf type DWORD byvalue;
   dcl (iand, isrl) builtin;
   return (iand (isrl (tmsf, 24), 'ff'xn));
 end MCI_TMSF_FRAME;

 MCI_MAKE_TMSF: proc (t, m, s, f) returns (type DWORD byvalue)
                options (inline reorder) internal;
   dcl (t, m, s, f) type BYTE byvalue;
   dcl (ior, isll) builtin;
   return (ior (t, isll (m, 8), isll (f, 16), isll (f, 24)));
 end MCI_MAKE_TMSF;


 MCI_HMS_HOUR: proc (hms) returns (type BYTE byvalue)
               options (inline reorder) internal;
   dcl hms type DWORD byvalue;
   dcl iand builtin;
   return (iand (hms, 'ff'xn));
 end MCI_HMS_HOUR;

 MCI_HMS_MINUTE: proc (hms) returns (type BYTE byvalue)
                 options (inline reorder) internal;
   dcl hms type DWORD byvalue;
   dcl (iand, isrl) builtin;
   return (iand (isrl (hms, 8), 'ff'xn));
 end MCI_HMS_MINUTE;

 MCI_HMS_SECOND: proc (hms) returns (type BYTE byvalue)
                 options (inline reorder) internal;
   dcl hms type DWORD byvalue;
   dcl (iand, isrl) builtin;
   return (iand (isrl (hms, 16), 'ff'xn));
 end MCI_HMS_SECOND;

 MCI_MAKE_HMS: proc (h, m, s) returns (type DWORD byvalue)
               options (inline reorder) internal;
   dcl (h, m, s) type BYTE byvalue;
   dcl (ior, isll) builtin;
   return (ior (h, isll (m, 8), isll (s, 16)));
 end MCI_MAKE_HMS;


 /* flags for wParam of MM_MCINOTIFY message */
 dcl MCI_NOTIFY_SUCCESSFUL          fixed bin(15) value('0001'xn);
 dcl MCI_NOTIFY_SUPERSEDED          fixed bin(15) value('0002'xn);
 dcl MCI_NOTIFY_ABORTED             fixed bin(15) value('0004'xn);
 dcl MCI_NOTIFY_FAILURE             fixed bin(15) value('0008'xn);

 /* common flags for dwFlags parameter of MCI command messages */
 dcl MCI_NOTIFY                     fixed bin(31) value('00000001'xn);
 dcl MCI_WAIT                       fixed bin(31) value('00000002'xn);
 dcl MCI_FROM                       fixed bin(31) value('00000004'xn);
 dcl MCI_TO                         fixed bin(31) value('00000008'xn);
 dcl MCI_TRACK                      fixed bin(31) value('00000010'xn);
 dcl MCI_TEST                       fixed bin(31) value('00000020'xn);

 /* flags for dwFlags parameter of MCI_OPEN command message */
 dcl MCI_OPEN_SHAREABLE             fixed bin(31) value('00000100'xn);
 dcl MCI_OPEN_ELEMENT               fixed bin(31) value('00000200'xn);
 dcl MCI_OPEN_ALIAS                 fixed bin(31) value('00000400'xn);
 dcl MCI_OPEN_ELEMENT_ID            fixed bin(31) value('00000800'xn);
 dcl MCI_OPEN_TYPE_ID               fixed bin(31) value('00001000'xn);
 dcl MCI_OPEN_TYPE                  fixed bin(31) value('00002000'xn);

 /* flags for dwFlags parameter of MCI_SEEK command message */
 dcl MCI_SEEK_TO_START              fixed bin(31) value('00000100'xn);
 dcl MCI_SEEK_TO_END                fixed bin(31) value('00000200'xn);

 /* flags for dwFlags parameter of MCI_STATUS command message */
 dcl MCI_STATUS_ITEM                fixed bin(31) value('00000100'xn);
 dcl MCI_STATUS_START               fixed bin(31) value('00000200'xn);

 /* flags for dwItem field of the MCI_STATUS_PARMS parameter block */
 dcl MCI_STATUS_LENGTH              fixed bin(31) value('00000001'xn);
 dcl MCI_STATUS_POSITION            fixed bin(31) value('00000002'xn);
 dcl MCI_STATUS_NUMBER_OF_TRACKS    fixed bin(31) value('00000003'xn);
 dcl MCI_STATUS_MODE                fixed bin(31) value('00000004'xn);
 dcl MCI_STATUS_MEDIA_PRESENT       fixed bin(31) value('00000005'xn);
 dcl MCI_STATUS_TIME_FORMAT         fixed bin(31) value('00000006'xn);
 dcl MCI_STATUS_READY               fixed bin(31) value('00000007'xn);
 dcl MCI_STATUS_CURRENT_TRACK       fixed bin(31) value('00000008'xn);

 /* flags for dwFlags parameter of MCI_INFO command message */
 dcl MCI_INFO_PRODUCT               fixed bin(31) value('00000100'xn);
 dcl MCI_INFO_FILE                  fixed bin(31) value('00000200'xn);
 dcl MCI_INFO_MEDIA_UPC             fixed bin(31) value('00000400'xn);
 dcl MCI_INFO_MEDIA_IDENTITY        fixed bin(31) value('00000800'xn);
 dcl MCI_INFO_NAME                  fixed bin(31) value('00001000'xn);
 dcl MCI_INFO_COPYRIGHT             fixed bin(31) value('00002000'xn);

 /* flags for dwFlags parameter of MCI_GETDEVCAPS command message */
 dcl MCI_GETDEVCAPS_ITEM            fixed bin(31) value('00000100'xn);

 /* flags for dwItem field of the MCI_GETDEVCAPS_PARMS parameter block*/
 dcl MCI_GETDEVCAPS_CAN_RECORD      fixed bin(31) value('00000001'xn);
 dcl MCI_GETDEVCAPS_HAS_AUDIO       fixed bin(31) value('00000002'xn);
 dcl MCI_GETDEVCAPS_HAS_VIDEO       fixed bin(31) value('00000003'xn);
 dcl MCI_GETDEVCAPS_DEVICE_TYPE     fixed bin(31) value('00000004'xn);
 dcl MCI_GETDEVCAPS_USES_FILES      fixed bin(31) value('00000005'xn);
 dcl MCI_GETDEVCAPS_COMPOUND_DEVICE fixed bin(31) value('00000006'xn);
 dcl MCI_GETDEVCAPS_CAN_EJECT       fixed bin(31) value('00000007'xn);
 dcl MCI_GETDEVCAPS_CAN_PLAY        fixed bin(31) value('00000008'xn);
 dcl MCI_GETDEVCAPS_CAN_SAVE        fixed bin(31) value('00000009'xn);

 /* flags for dwFlags parameter of MCI_SYSINFO command message */
 dcl MCI_SYSINFO_QUANTITY           fixed bin(31) value('00000100'xn);
 dcl MCI_SYSINFO_OPEN               fixed bin(31) value('00000200'xn);
 dcl MCI_SYSINFO_NAME               fixed bin(31) value('00000400'xn);
 dcl MCI_SYSINFO_INSTALLNAME        fixed bin(31) value('00000800'xn);

 /* flags for dwFlags parameter of MCI_SET command message */
 dcl MCI_SET_DOOR_OPEN              fixed bin(31) value('00000100'xn);
 dcl MCI_SET_DOOR_CLOSED            fixed bin(31) value('00000200'xn);
 dcl MCI_SET_TIME_FORMAT            fixed bin(31) value('00000400'xn);
 dcl MCI_SET_AUDIO                  fixed bin(31) value('00000800'xn);
 dcl MCI_SET_VIDEO                  fixed bin(31) value('00001000'xn);
 dcl MCI_SET_ON                     fixed bin(31) value('00002000'xn);
 dcl MCI_SET_OFF                    fixed bin(31) value('00004000'xn);

 /* flags for dwAudio field of MCI_SET_PARMS or MCI_SEQ_SET_PARMS */
 dcl MCI_SET_AUDIO_ALL              fixed bin(31) value('00000000'xn);
 dcl MCI_SET_AUDIO_LEFT             fixed bin(31) value('00000001'xn);
 dcl MCI_SET_AUDIO_RIGHT            fixed bin(31) value('00000002'xn);

 /* flags for dwFlags parameter of MCI_BREAK command message */
 dcl MCI_BREAK_KEY                  fixed bin(31) value('00000100'xn);
 dcl MCI_BREAK_HWND                 fixed bin(31) value('00000200'xn);
 dcl MCI_BREAK_OFF                  fixed bin(31) value('00000400'xn);

 /* flags for dwFlags parameter of MCI_RECORD command message */
 dcl MCI_RECORD_INSERT              fixed bin(31) value('00000100'xn);
 dcl MCI_RECORD_OVERWRITE           fixed bin(31) value('00000200'xn);

 /* flags for dwFlags parameter of MCI_SAVE command message */
 dcl MCI_SAVE_FILE                  fixed bin(31) value('00000100'xn);

 /* flags for dwFlags parameter of MCI_LOAD command message */
 dcl MCI_LOAD_FILE                  fixed bin(31) value('00000100'xn);

 /* generic parameter block for MCI command messages */
 /* with no special parameters                       */
 define structure
   1 MCI_GENERIC_PARMS,
     2 dwCallback type DWORD;
 define alias  PMCI_GENERIC_PARMS handle MCI_GENERIC_PARMS;
 define alias LPMCI_GENERIC_PARMS handle MCI_GENERIC_PARMS;


 /* parameter block for MCI_OPEN command message */

 define structure
   1 MCI_OPEN_PARMSA,
     2 dwCallback       type DWORD,
     2 wDeviceID        type MCIDEVICEID,
     2 lpstrDeviceType  type LPCSTR,
     2 lpstrElementName type LPCSTR,
     2 lpstrAlias       type LPCSTR;
 define alias  PMCI_OPEN_PARMSA handle MCI_OPEN_PARMSA;
 define alias LPMCI_OPEN_PARMSA handle MCI_OPEN_PARMSA;

 define structure
   1 MCI_OPEN_PARMSW,
     2 dwCallback       type DWORD,
     2 wDeviceID        type MCIDEVICEID,
     2 lpstrDeviceType  type LPCWSTR,
     2 lpstrElementName type LPCWSTR,
     2 lpstrAlias       type LPCWSTR;
 define alias  PMCI_OPEN_PARMSW handle MCI_OPEN_PARMSW;
 define alias LPMCI_OPEN_PARMSW handle MCI_OPEN_PARMSW;

 %if UNICODE ^= '' %then %do;
 define alias   MCI_OPEN_PARMS type   MCI_OPEN_PARMSW;
 define alias  PMCI_OPEN_PARMS type  PMCI_OPEN_PARMSW;
 define alias LPMCI_OPEN_PARMS type LPMCI_OPEN_PARMSW;
 %end; %else %do;
 define alias   MCI_OPEN_PARMS type   MCI_OPEN_PARMSA;
 define alias  PMCI_OPEN_PARMS type  PMCI_OPEN_PARMSA;
 define alias LPMCI_OPEN_PARMS type LPMCI_OPEN_PARMSA;
 %end; /* UNICODE */

 /* parameter block for MCI_PLAY command message */
 define structure
   1 MCI_PLAY_PARMS,
     2 dwCallback type DWORD,
     2 dwFrom     type DWORD,
     2 dwTo       type DWORD;
 define alias  PMCI_PLAY_PARMS handle MCI_PLAY_PARMS;
 define alias LPMCI_PLAY_PARMS handle MCI_PLAY_PARMS;

 /* parameter block for MCI_SEEK command message */
 define structure
   1 MCI_SEEK_PARMS,
     2 dwCallback type DWORD,
     2 dwTo       type DWORD;
 define alias  PMCI_SEEK_PARMS handle MCI_SEEK_PARMS;
 define alias LPMCI_SEEK_PARMS handle MCI_SEEK_PARMS;

 /* parameter block for MCI_STATUS command message */
 define structure
   1 MCI_STATUS_PARMS,
     2 dwCallback type DWORD,
     2 dwReturn   type DWORD,
     2 dwItem     type DWORD,
     2 dwTrack    type DWORD;
 define alias  PMCI_STATUS_PARMS handle MCI_STATUS_PARMS;
 define alias LPMCI_STATUS_PARMS handle MCI_STATUS_PARMS;

 /* parameter block for MCI_INFO command message */
 define structure
   1 MCI_INFO_PARMSA,
     2 dwCallback  type DWORD,
     2 lpstrReturn type LPSTR,
     2 dwRetSize   type DWORD;
 define alias LPMCI_INFO_PARMSA handle MCI_INFO_PARMSA;

 define structure
   1 MCI_INFO_PARMSW,
     2 dwCallback  type DWORD,
     2 lpstrReturn type LPWSTR,
     2 dwRetSize   type DWORD;
 define alias LPMCI_INFO_PARMSW handle MCI_INFO_PARMSW;

 %if UNICODE ^= '' %then %do;
 define alias   MCI_INFO_PARMS type   MCI_INFO_PARMSW;
 define alias LPMCI_INFO_PARMS type LPMCI_INFO_PARMSW;
 %end; %else %do;
 define alias   MCI_INFO_PARMS type   MCI_INFO_PARMSA;
 define alias LPMCI_INFO_PARMS type LPMCI_INFO_PARMSA;
 %end; /* UNICODE */


 /* parameter block for MCI_GETDEVCAPS command message */
 define structure
   1 MCI_GETDEVCAPS_PARMS,
     2 dwCallback type DWORD,
     2 dwReturn   type DWORD,
     2 dwItem     type DWORD;
 define alias  PMCI_GETDEVCAPS_PARMS handle MCI_GETDEVCAPS_PARMS;
 define alias LPMCI_GETDEVCAPS_PARMS handle MCI_GETDEVCAPS_PARMS;

 /* parameter block for MCI_SYSINFO command message */
 define structure
   1 MCI_SYSINFO_PARMSA,
     2 dwCallback  type DWORD,
     2 lpstrReturn type LPSTR,
     2 dwRetSize   type DWORD,
     2 dwNumber    type DWORD,
     2 wDeviceType type UINT;
 define alias  PMCI_SYSINFO_PARMSA handle MCI_SYSINFO_PARMSA;
 define alias LPMCI_SYSINFO_PARMSA handle MCI_SYSINFO_PARMSA;

 define structure
   1 MCI_SYSINFO_PARMSW,
     2 dwCallback  type DWORD,
     2 lpstrReturn type LPWSTR,
     2 dwRetSize   type DWORD,
     2 dwNumber    type DWORD,
     2 wDeviceType type UINT;
 define alias  PMCI_SYSINFO_PARMSW handle MCI_SYSINFO_PARMSW;
 define alias LPMCI_SYSINFO_PARMSW handle MCI_SYSINFO_PARMSW;

 %if UNICODE ^= '' %then %do;
 define alias   MCI_SYSINFO_PARMS type   MCI_SYSINFO_PARMSW;
 define alias  PMCI_SYSINFO_PARMS type  PMCI_SYSINFO_PARMSW;
 define alias LPMCI_SYSINFO_PARMS type LPMCI_SYSINFO_PARMSW;
 %end; %else %do;
 define alias   MCI_SYSINFO_PARMS type   MCI_SYSINFO_PARMSA;
 define alias  PMCI_SYSINFO_PARMS type  PMCI_SYSINFO_PARMSA;
 define alias LPMCI_SYSINFO_PARMS type LPMCI_SYSINFO_PARMSA;
 %end; /* UNICODE */

 /* parameter block for MCI_SET command message */
 define structure
   1 MCI_SET_PARMS,
     2 dwCallback   type DWORD,
     2 dwTimeFormat type DWORD,
     2 dwAudio      type DWORD;
 define alias  PMCI_SET_PARMS handle MCI_SET_PARMS;
 define alias LPMCI_SET_PARMS handle MCI_SET_PARMS;

 /* parameter block for MCI_BREAK command message */
 define structure
   1 MCI_BREAK_PARMS,
     2 dwCallback type DWORD,
     2 nVirtKey   fixed bin(31),
     2 hwndBreak  type HWND;
 define alias  PMCI_BREAK_PARMS handle MCI_BREAK_PARMS;
 define alias LPMCI_BREAK_PARMS handle MCI_BREAK_PARMS;

 /* parameter block for MCI_SAVE command message */
 define structure
   1 MCI_SAVE_PARMSA,
     2 dwCallback type DWORD,
     2 lpfilename type LPCSTR;
 define alias  PMCI_SAVE_PARMSA handle MCI_SAVE_PARMSA;
 define alias LPMCI_SAVE_PARMSA handle MCI_SAVE_PARMSA;

 define structure
   1 MCI_SAVE_PARMSW,
     2 dwCallback type DWORD,
     2 lpfilename type LPCWSTR;
 define alias  PMCI_SAVE_PARMSW handle MCI_SAVE_PARMSW;
 define alias LPMCI_SAVE_PARMSW handle MCI_SAVE_PARMSW;

 %if UNICODE ^= '' %then %do;
 define alias   MCI_SAVE_PARMS type   MCI_SAVE_PARMSW;
 define alias  PMCI_SAVE_PARMS type  PMCI_SAVE_PARMSW;
 define alias LPMCI_SAVE_PARMS type LPMCI_SAVE_PARMSW;
 %end; %else %do;
 define alias   MCI_SAVE_PARMS type   MCI_SAVE_PARMSA;
 define alias  PMCI_SAVE_PARMS type  PMCI_SAVE_PARMSA;
 define alias LPMCI_SAVE_PARMS type LPMCI_SAVE_PARMSA;
 %end; /* UNICODE */

 /* parameter block for MCI_LOAD command message */
 define structure
   1 MCI_LOAD_PARMSA,
     2 dwCallback type DWORD,
     2 lpfilename type LPCSTR;
 define alias  PMCI_LOAD_PARMSA handle MCI_LOAD_PARMSA;
 define alias LPMCI_LOAD_PARMSA handle MCI_LOAD_PARMSA;

 define structure
   1 MCI_LOAD_PARMSW,
     2 dwCallback type DWORD,
     2 lpfilename type LPCWSTR;
 define alias  PMCI_LOAD_PARMSW handle MCI_LOAD_PARMSW;
 define alias LPMCI_LOAD_PARMSW handle MCI_LOAD_PARMSW;

 %if UNICODE ^= '' %then %do;
 define alias MCI_LOAD_PARMS type MCI_LOAD_PARMSW;
 define alias PMCI_LOAD_PARMS type PMCI_LOAD_PARMSW;
 define alias LPMCI_LOAD_PARMS type LPMCI_LOAD_PARMSW;
 %end; %else %do;
 define alias MCI_LOAD_PARMS type MCI_LOAD_PARMSA;
 define alias PMCI_LOAD_PARMS type PMCI_LOAD_PARMSA;
 define alias LPMCI_LOAD_PARMS type LPMCI_LOAD_PARMSA;
 %end; /* UNICODE */

 /* parameter block for MCI_RECORD command message */
 define structure
   1 MCI_RECORD_PARMS,
     2 dwCallback type DWORD,
     2 dwFrom     type DWORD,
     2 dwTo       type DWORD;
 define alias LPMCI_RECORD_PARMS handle MCI_RECORD_PARMS;

 /* MCI extensions for videodisc devices */

 /* flag for dwReturn field of MCI_STATUS_PARMS */
 /* MCI_STATUS command, (dwItem == MCI_STATUS_MODE) */
 dcl MCI_VD_MODE_PARK           fixed bin(31) value(MCI_VD_OFFSET + 1);

 /* flag for dwReturn field of MCI_STATUS_PARMS */
 /* MCI_STATUS command, (dwItem == MCI_VD_STATUS_MEDIA_TYPE) */
 dcl MCI_VD_MEDIA_CLV           fixed bin(31) value(MCI_VD_OFFSET + 2);
 dcl MCI_VD_MEDIA_CAV           fixed bin(31) value(MCI_VD_OFFSET + 3);
 dcl MCI_VD_MEDIA_OTHER         fixed bin(31) value(MCI_VD_OFFSET + 4);

 dcl MCI_VD_FORMAT_TRACK        fixed bin(15) value('4001'xn);

 /* flags for dwFlags parameter of MCI_PLAY command message */
 dcl MCI_VD_PLAY_REVERSE           fixed bin(31) value('00010000'xn);
 dcl MCI_VD_PLAY_FAST              fixed bin(31) value('00020000'xn);
 dcl MCI_VD_PLAY_SPEED             fixed bin(31) value('00040000'xn);
 dcl MCI_VD_PLAY_SCAN              fixed bin(31) value('00080000'xn);
 dcl MCI_VD_PLAY_SLOW              fixed bin(31) value('00100000'xn);

 /* flag for dwFlags parameter of MCI_SEEK command message */
 dcl MCI_VD_SEEK_REVERSE             fixed bin(31) value('00010000'xn);

 /* flags for dwItem field of MCI_STATUS_PARMS parameter block */
 dcl MCI_VD_STATUS_SPEED             fixed bin(31) value('00004002'xn);
 dcl MCI_VD_STATUS_FORWARD           fixed bin(31) value('00004003'xn);
 dcl MCI_VD_STATUS_MEDIA_TYPE        fixed bin(31) value('00004004'xn);
 dcl MCI_VD_STATUS_SIDE              fixed bin(31) value('00004005'xn);
 dcl MCI_VD_STATUS_DISC_SIZE         fixed bin(31) value('00004006'xn);

 /* flags for dwFlags parameter of MCI_GETDEVCAPS command message */
 dcl MCI_VD_GETDEVCAPS_CLV           fixed bin(31) value('00010000'xn);
 dcl MCI_VD_GETDEVCAPS_CAV           fixed bin(31) value('00020000'xn);

 dcl MCI_VD_SPIN_UP                  fixed bin(31) value('00010000'xn);
 dcl MCI_VD_SPIN_DOWN                fixed bin(31) value('00020000'xn);

 /* flags for dwItem field of MCI_GETDEVCAPS_PARMS parameter block */
 dcl MCI_VD_GETDEVCAPS_CAN_REVERSE   fixed bin(31) value('00004002'xn);
 dcl MCI_VD_GETDEVCAPS_FAST_RATE     fixed bin(31) value('00004003'xn);
 dcl MCI_VD_GETDEVCAPS_SLOW_RATE     fixed bin(31) value('00004004'xn);
 dcl MCI_VD_GETDEVCAPS_NORMAL_RATE   fixed bin(31) value('00004005'xn);

 /* flags for the dwFlags parameter of MCI_STEP command message */
 dcl MCI_VD_STEP_FRAMES              fixed bin(31) value('00010000'xn);
 dcl MCI_VD_STEP_REVERSE             fixed bin(31) value('00020000'xn);

 /* flag for the MCI_ESCAPE command message */
 dcl MCI_VD_ESCAPE_STRING            fixed bin(31) value('00000100'xn);

 /* parameter block for MCI_PLAY command message */
 define structure
   1 MCI_VD_PLAY_PARMS,
     2 dwCallback type DWORD,
     2 dwFrom     type DWORD,
     2 dwTo       type DWORD,
     2 dwSpeed    type DWORD;
 define alias  PMCI_VD_PLAY_PARMS handle MCI_VD_PLAY_PARMS;
 define alias LPMCI_VD_PLAY_PARMS handle MCI_VD_PLAY_PARMS;

 /* parameter block for MCI_STEP command message */
 define structure
   1 MCI_VD_STEP_PARMS,
     2 dwCallback type DWORD,
     2 dwFrames   type DWORD;
 define alias  PMCI_VD_STEP_PARMS handle MCI_VD_STEP_PARMS;
 define alias LPMCI_VD_STEP_PARMS handle MCI_VD_STEP_PARMS;

 /* parameter block for MCI_ESCAPE command message */
 define structure
   1 MCI_VD_ESCAPE_PARMSA,
     2 dwCallback   type DWORD,
     2 lpstrCommand type LPCSTR;
 define alias  PMCI_VD_ESCAPE_PARMSA handle MCI_VD_ESCAPE_PARMSA;
 define alias LPMCI_VD_ESCAPE_PARMSA handle MCI_VD_ESCAPE_PARMSA;

 define structure
   1 MCI_VD_ESCAPE_PARMSW,
     2 dwCallback   type DWORD,
     2 lpstrCommand type LPCWSTR;
 define alias  PMCI_VD_ESCAPE_PARMSW handle MCI_VD_ESCAPE_PARMSW;
 define alias LPMCI_VD_ESCAPE_PARMSW handle MCI_VD_ESCAPE_PARMSW;

 %if UNICODE ^= '' %then %do;
 define alias   MCI_VD_ESCAPE_PARMS type   MCI_VD_ESCAPE_PARMSW;
 define alias  PMCI_VD_ESCAPE_PARMS type  PMCI_VD_ESCAPE_PARMSW;
 define alias LPMCI_VD_ESCAPE_PARMS type LPMCI_VD_ESCAPE_PARMSW;
 %end; %else %do;
 define alias   MCI_VD_ESCAPE_PARMS type   MCI_VD_ESCAPE_PARMSA;
 define alias  PMCI_VD_ESCAPE_PARMS type  PMCI_VD_ESCAPE_PARMSA;
 define alias LPMCI_VD_ESCAPE_PARMS type LPMCI_VD_ESCAPE_PARMSA;
 %end; /* UNICODE */

 /* MCI extensions for CD audio devices */

 /* flags for the dwItem field of the MCI_STATUS_PARMS parameter block*/
 dcl MCI_CDA_STATUS_TYPE_TRACK fixed bin(31) value('00004001'xn);

 /* flags for the dwReturn field of MCI_STATUS_PARMS parameter block */
 /* MCI_STATUS command, (dwItem == MCI_CDA_STATUS_TYPE_TRACK) */
 dcl MCI_CDA_TRACK_AUDIO       fixed bin(31) value(MCI_CD_OFFSET + 0);
 dcl MCI_CDA_TRACK_OTHER       fixed bin(31) value(MCI_CD_OFFSET + 1);

 /* MCI extensions for waveform audio devices */

 dcl MCI_WAVE_PCM              fixed bin(31) value(MCI_WAVE_OFFSET + 0);
 dcl MCI_WAVE_MAPPER           fixed bin(31) value(MCI_WAVE_OFFSET + 1);

 /* flags for the dwFlags parameter of MCI_OPEN command message */
 dcl MCI_WAVE_OPEN_BUFFER      fixed bin(31) value('00010000'xn);

 /* flags for the dwFlags parameter of MCI_SET command message */
 dcl MCI_WAVE_SET_FORMATTAG          fixed bin(31) value('00010000'xn);
 dcl MCI_WAVE_SET_CHANNELS           fixed bin(31) value('00020000'xn);
 dcl MCI_WAVE_SET_SAMPLESPERSEC      fixed bin(31) value('00040000'xn);
 dcl MCI_WAVE_SET_AVGBYTESPERSEC     fixed bin(31) value('00080000'xn);
 dcl MCI_WAVE_SET_BLOCKALIGN         fixed bin(31) value('00100000'xn);
 dcl MCI_WAVE_SET_BITSPERSAMPLE      fixed bin(31) value('00200000'xn);

 /* flags for the dwFlags parameter of MCI_STATUS, MCI_SET command */
 /* messages */
 dcl MCI_WAVE_INPUT                  fixed bin(31) value('00400000'xn);
 dcl MCI_WAVE_OUTPUT                 fixed bin(31) value('00800000'xn);

 /* flags for the dwItem field of MCI_STATUS_PARMS parameter block */
 dcl MCI_WAVE_STATUS_FORMATTAG       fixed bin(31) value('00004001'xn);
 dcl MCI_WAVE_STATUS_CHANNELS        fixed bin(31) value('00004002'xn);
 dcl MCI_WAVE_STATUS_SAMPLESPERSEC   fixed bin(31) value('00004003'xn);
 dcl MCI_WAVE_STATUS_AVGBYTESPERSEC  fixed bin(31) value('00004004'xn);
 dcl MCI_WAVE_STATUS_BLOCKALIGN      fixed bin(31) value('00004005'xn);
 dcl MCI_WAVE_STATUS_BITSPERSAMPLE   fixed bin(31) value('00004006'xn);
 dcl MCI_WAVE_STATUS_LEVEL           fixed bin(31) value('00004007'xn);

 /* flags for the dwFlags parameter of MCI_SET command message */
 dcl MCI_WAVE_SET_ANYINPUT           fixed bin(31) value('04000000'xn);
 dcl MCI_WAVE_SET_ANYOUTPUT          fixed bin(31) value('08000000'xn);

 /* flags for the dwFlags parameter of MCI_GETDEVCAPS command message */
 dcl MCI_WAVE_GETDEVCAPS_INPUTS      fixed bin(31) value('00004001'xn);
 dcl MCI_WAVE_GETDEVCAPS_OUTPUTS     fixed bin(31) value('00004002'xn);

 /* parameter block for MCI_OPEN command message */
 define structure
   1 MCI_WAVE_OPEN_PARMSA,
     2 dwCallback       type DWORD,
     2 wDeviceID        type MCIDEVICEID,
     2 lpstrDeviceType  type LPCSTR,
     2 lpstrElementName type LPCSTR,
     2 lpstrAlias       type LPCSTR,
     2 dwBufferSeconds  type DWORD;
 define alias  PMCI_WAVE_OPEN_PARMSA handle MCI_WAVE_OPEN_PARMSA;
 define alias LPMCI_WAVE_OPEN_PARMSA handle MCI_WAVE_OPEN_PARMSA;

 define structure
   1 MCI_WAVE_OPEN_PARMSW,
     2 dwCallback       type DWORD,
     2 wDeviceID        type MCIDEVICEID,
     2 lpstrDeviceType  type LPCWSTR,
     2 lpstrElementName type LPCWSTR,
     2 lpstrAlias       type LPCWSTR,
     2 dwBufferSeconds  type DWORD;
 define alias  PMCI_WAVE_OPEN_PARMSW handle MCI_WAVE_OPEN_PARMSW;
 define alias LPMCI_WAVE_OPEN_PARMSW handle MCI_WAVE_OPEN_PARMSW;

 %if UNICODE ^= '' %then %do;
 define alias   MCI_WAVE_OPEN_PARMS type   MCI_WAVE_OPEN_PARMSW;
 define alias  PMCI_WAVE_OPEN_PARMS type  PMCI_WAVE_OPEN_PARMSW;
 define alias LPMCI_WAVE_OPEN_PARMS type LPMCI_WAVE_OPEN_PARMSW;
 %end; %else %do;
 define alias   MCI_WAVE_OPEN_PARMS type   MCI_WAVE_OPEN_PARMSA;
 define alias  PMCI_WAVE_OPEN_PARMS type  PMCI_WAVE_OPEN_PARMSA;
 define alias LPMCI_WAVE_OPEN_PARMS type LPMCI_WAVE_OPEN_PARMSA;
 %end; /* UNICODE */

 /* parameter block for MCI_DELETE command message */
 define structure
   1 MCI_WAVE_DELETE_PARMS,
     2 dwCallback type DWORD,
     2 dwFrom     type DWORD,
     2 dwTo       type DWORD;
 define alias  PMCI_WAVE_DELETE_PARMS handle MCI_WAVE_DELETE_PARMS;
 define alias LPMCI_WAVE_DELETE_PARMS handle MCI_WAVE_DELETE_PARMS;

 /* parameter block for MCI_SET command message */
 define structure
   1 MCI_WAVE_SET_PARMS,
     2 dwCallback      type DWORD,
     2 dwTimeFormat    type DWORD,
     2 dwAudio         type DWORD,
     2 wInput          type UINT,
     2 wOutput         type UINT,
     2 wFormatTag      type WORD,
     2 wReserved2      type WORD,
     2 nChannels       type WORD,
     2 wReserved3      type WORD,
     2 nSamplesPerSec  type DWORD,
     2 nAvgBytesPerSec type DWORD,
     2 nBlockAlign     type WORD,
     2 wReserved4      type WORD,
     2 wBitsPerSample  type WORD,
     2 wReserved5      type WORD;
 define alias  PMCI_WAVE_SET_PARMS handle MCI_WAVE_SET_PARMS;
 define alias LPMCI_WAVE_SET_PARMS handle MCI_WAVE_SET_PARMS;

 /* MCI extensions for MIDI sequencer devices */

 /* flags for the dwReturn field of MCI_STATUS_PARMS parameter block */
 /* MCI_STATUS command, (dwItem == MCI_SEQ_STATUS_DIVTYPE) */
 dcl MCI_SEQ_DIV_PPQN         fixed bin(31) value(0 + MCI_SEQ_OFFSET);
 dcl MCI_SEQ_DIV_SMPTE_24     fixed bin(31) value(1 + MCI_SEQ_OFFSET);
 dcl MCI_SEQ_DIV_SMPTE_25     fixed bin(31) value(2 + MCI_SEQ_OFFSET);
 dcl MCI_SEQ_DIV_SMPTE_30DROP fixed bin(31) value(3 + MCI_SEQ_OFFSET);
 dcl MCI_SEQ_DIV_SMPTE_30     fixed bin(31) value(4 + MCI_SEQ_OFFSET);

 /* flags for the dwMaster field of MCI_SEQ_SET_PARMS parameter block */
 /* MCI_SET command, (dwFlags == MCI_SEQ_SET_MASTER) */
 dcl MCI_SEQ_FORMAT_SONGPTR          fixed bin(15) value('4001'xn);
 dcl MCI_SEQ_FILE                    fixed bin(15) value('4002'xn);
 dcl MCI_SEQ_MIDI                    fixed bin(15) value('4003'xn);
 dcl MCI_SEQ_SMPTE                   fixed bin(15) value('4004'xn);
 dcl MCI_SEQ_NONE                    fixed bin(31) value(65533);
 dcl MCI_SEQ_MAPPER                  fixed bin(31) value(65535);

 /* flags for the dwItem field of MCI_STATUS_PARMS parameter block */
 dcl MCI_SEQ_STATUS_TEMPO            fixed bin(31) value('00004002'xn);
 dcl MCI_SEQ_STATUS_PORT             fixed bin(31) value('00004003'xn);
 dcl MCI_SEQ_STATUS_SLAVE            fixed bin(31) value('00004007'xn);
 dcl MCI_SEQ_STATUS_MASTER           fixed bin(31) value('00004008'xn);
 dcl MCI_SEQ_STATUS_OFFSET           fixed bin(31) value('00004009'xn);
 dcl MCI_SEQ_STATUS_DIVTYPE          fixed bin(31) value('0000400A'xn);
 dcl MCI_SEQ_STATUS_NAME             fixed bin(31) value('0000400B'xn);
 dcl MCI_SEQ_STATUS_COPYRIGHT        fixed bin(31) value('0000400C'xn);

 /* flags for the dwFlags parameter of MCI_SET command message */
 dcl MCI_SEQ_SET_TEMPO               fixed bin(31) value('00010000'xn);
 dcl MCI_SEQ_SET_PORT                fixed bin(31) value('00020000'xn);
 dcl MCI_SEQ_SET_SLAVE               fixed bin(31) value('00040000'xn);
 dcl MCI_SEQ_SET_MASTER              fixed bin(31) value('00080000'xn);
 dcl MCI_SEQ_SET_OFFSET              fixed bin(31) value('01000000'xn);

 /* parameter block for MCI_SET command message */
 define structure
   1 MCI_SEQ_SET_PARMS,
     2 dwCallback   type DWORD,
     2 dwTimeFormat type DWORD,
     2 dwAudio      type DWORD,
     2 dwTempo      type DWORD,
     2 dwPort       type DWORD,
     2 dwSlave      type DWORD,
     2 dwMaster     type DWORD,
     2 dwOffset     type DWORD;
 define alias  PMCI_SEQ_SET_PARMS handle MCI_SEQ_SET_PARMS;
 define alias LPMCI_SEQ_SET_PARMS handle MCI_SEQ_SET_PARMS;

 /* MCI extensions for animation devices */

 /* flags for dwFlags parameter of MCI_OPEN command message */
 dcl MCI_ANIM_OPEN_WS                fixed bin(31) value('00010000'xn);
 dcl MCI_ANIM_OPEN_PARENT            fixed bin(31) value('00020000'xn);
 dcl MCI_ANIM_OPEN_NOSTATIC          fixed bin(31) value('00040000'xn);

 /* flags for dwFlags parameter of MCI_PLAY command message */
 dcl MCI_ANIM_PLAY_SPEED             fixed bin(31) value('00010000'xn);
 dcl MCI_ANIM_PLAY_REVERSE           fixed bin(31) value('00020000'xn);
 dcl MCI_ANIM_PLAY_FAST              fixed bin(31) value('00040000'xn);
 dcl MCI_ANIM_PLAY_SLOW              fixed bin(31) value('00080000'xn);
 dcl MCI_ANIM_PLAY_SCAN              fixed bin(31) value('00100000'xn);

 /* flags for dwFlags parameter of MCI_STEP command message */
 dcl MCI_ANIM_STEP_REVERSE           fixed bin(31) value('00010000'xn);
 dcl MCI_ANIM_STEP_FRAMES            fixed bin(31) value('00020000'xn);

 /* flags for dwItem field of MCI_STATUS_PARMS parameter block */
 dcl MCI_ANIM_STATUS_SPEED           fixed bin(31) value('00004001'xn);
 dcl MCI_ANIM_STATUS_FORWARD         fixed bin(31) value('00004002'xn);
 dcl MCI_ANIM_STATUS_HWND            fixed bin(31) value('00004003'xn);
 dcl MCI_ANIM_STATUS_HPAL            fixed bin(31) value('00004004'xn);
 dcl MCI_ANIM_STATUS_STRETCH         fixed bin(31) value('00004005'xn);

 /* flags for the dwFlags parameter of MCI_INFO command message */
 dcl MCI_ANIM_INFO_TEXT              fixed bin(31) value('00010000'xn);

 /* flags for dwItem field of MCI_GETDEVCAPS_PARMS parameter block */
 dcl MCI_ANIM_GETDEVCAPS_CAN_REVERSE fixed bin(31) value('00004001'xn);
 dcl MCI_ANIM_GETDEVCAPS_FAST_RATE   fixed bin(31) value('00004002'xn);
 dcl MCI_ANIM_GETDEVCAPS_SLOW_RATE   fixed bin(31) value('00004003'xn);
 dcl MCI_ANIM_GETDEVCAPS_NORMAL_RATE fixed bin(31) value('00004004'xn);
 dcl MCI_ANIM_GETDEVCAPS_PALETTES    fixed bin(31) value('00004006'xn);
 dcl MCI_ANIM_GETDEVCAPS_CAN_STRETCH fixed bin(31) value('00004007'xn);
 dcl MCI_ANIM_GETDEVCAPS_MAX_WINDOWS fixed bin(31) value('00004008'xn);

 /* flags for the MCI_REALIZE command message */
 dcl MCI_ANIM_REALIZE_NORM           fixed bin(31) value('00010000'xn);
 dcl MCI_ANIM_REALIZE_BKGD           fixed bin(31) value('00020000'xn);

 /* flags for dwFlags parameter of MCI_WINDOW command message */
 dcl MCI_ANIM_WINDOW_HWND            fixed bin(31) value('00010000'xn);
 dcl MCI_ANIM_WINDOW_STATE           fixed bin(31) value('00040000'xn);
 dcl MCI_ANIM_WINDOW_TEXT            fixed bin(31) value('00080000'xn);
 dcl MCI_ANIM_WINDOW_ENABLE_STRETCH  fixed bin(31) value('00100000'xn);
 dcl MCI_ANIM_WINDOW_DISABLE_STRETCH fixed bin(31) value('00200000'xn);

 /* flags for hWnd field of MCI_ANIM_WINDOW_PARMS parameter block */
 /* MCI_WINDOW command message, (dwFlags == MCI_ANIM_WINDOW_HWND) */
 dcl MCI_ANIM_WINDOW_DEFAULT         fixed bin(31) value('00000000'xn);

 /* flags for dwFlags parameter of MCI_PUT command message */
 dcl MCI_ANIM_RECT                   fixed bin(31) value('00010000'xn);
 dcl MCI_ANIM_PUT_SOURCE             fixed bin(31) value('00020000'xn);
 dcl MCI_ANIM_PUT_DESTINATION        fixed bin(31) value('00040000'xn);

 /* flags for dwFlags parameter of MCI_WHERE command message */
 dcl MCI_ANIM_WHERE_SOURCE           fixed bin(31) value('00020000'xn);
 dcl MCI_ANIM_WHERE_DESTINATION      fixed bin(31) value('00040000'xn);

 /* flags for dwFlags parameter of MCI_UPDATE command message */
 dcl MCI_ANIM_UPDATE_HDC             fixed bin(31) value('00020000'xn);

 /* parameter block for MCI_OPEN command message */
 define structure
   1 MCI_ANIM_OPEN_PARMSA,
     2 dwCallback       type DWORD,
     2 wDeviceID        type MCIDEVICEID,
     2 lpstrDeviceType  type LPCSTR,
     2 lpstrElementName type LPCSTR,
     2 lpstrAlias       type LPCSTR,
     2 dwStyle          type DWORD,
     2 hWndParent       type HWND;
 define alias  PMCI_ANIM_OPEN_PARMSA handle MCI_ANIM_OPEN_PARMSA;
 define alias LPMCI_ANIM_OPEN_PARMSA handle MCI_ANIM_OPEN_PARMSA;

 define structure
   1 MCI_ANIM_OPEN_PARMSW,
     2 dwCallback       type DWORD,
     2 wDeviceID        type MCIDEVICEID,
     2 lpstrDeviceType  type LPCWSTR,
     2 lpstrElementName type LPCWSTR,
     2 lpstrAlias       type LPCWSTR,
     2 dwStyle          type DWORD,
     2 hWndParent       type HWND;
 define alias  PMCI_ANIM_OPEN_PARMSW handle MCI_ANIM_OPEN_PARMSW;
 define alias LPMCI_ANIM_OPEN_PARMSW handle MCI_ANIM_OPEN_PARMSW;

 %if UNICODE ^= '' %then %do;
 define alias   MCI_ANIM_OPEN_PARMS type   MCI_ANIM_OPEN_PARMSW;
 define alias  PMCI_ANIM_OPEN_PARMS type  PMCI_ANIM_OPEN_PARMSW;
 define alias LPMCI_ANIM_OPEN_PARMS type LPMCI_ANIM_OPEN_PARMSW;
 %end; %else %do;
 define alias   MCI_ANIM_OPEN_PARMS type   MCI_ANIM_OPEN_PARMSA;
 define alias  PMCI_ANIM_OPEN_PARMS type  PMCI_ANIM_OPEN_PARMSA;
 define alias LPMCI_ANIM_OPEN_PARMS type LPMCI_ANIM_OPEN_PARMSA;
 %end; /* UNICODE */

 /* parameter block for MCI_PLAY command message */
 define structure
   1 MCI_ANIM_PLAY_PARMS,
     2 dwCallback type DWORD,
     2 dwFrom     type DWORD,
     2 dwTo       type DWORD,
     2 dwSpeed    type DWORD;
 define alias  PMCI_ANIM_PLAY_PARMS handle MCI_ANIM_PLAY_PARMS;
 define alias LPMCI_ANIM_PLAY_PARMS handle MCI_ANIM_PLAY_PARMS;

 /* parameter block for MCI_STEP command message */
 define structure
   1 MCI_ANIM_STEP_PARMS,
     2 dwCallback type DWORD,
     2 dwFrames   type DWORD;
 define alias  PMCI_ANIM_STEP_PARMS handle MCI_ANIM_STEP_PARMS;
 define alias LPMCI_ANIM_STEP_PARMS handle MCI_ANIM_STEP_PARMS;

 /* parameter block for MCI_WINDOW command message */
 define structure
   1 MCI_ANIM_WINDOW_PARMSA,
     2 dwCallback type DWORD,
     2 hWnd       type HWND,
     2 nCmdShow   type UINT,
     2 lpstrText  type LPCSTR;
 define alias  PMCI_ANIM_WINDOW_PARMSA handle MCI_ANIM_WINDOW_PARMSA;
 define alias LPMCI_ANIM_WINDOW_PARMSA handle MCI_ANIM_WINDOW_PARMSA;

 define structure
   1 MCI_ANIM_WINDOW_PARMSW,
     2 dwCallback type DWORD,
     2 hWnd       type HWND,
     2 nCmdShow   type UINT,
     2 lpstrText  type LPCWSTR;
 define alias  PMCI_ANIM_WINDOW_PARMSW handle MCI_ANIM_WINDOW_PARMSW;
 define alias LPMCI_ANIM_WINDOW_PARMSW handle MCI_ANIM_WINDOW_PARMSW;

 %if UNICODE ^= '' %then %do;
 define alias   MCI_ANIM_WINDOW_PARMS type   MCI_ANIM_WINDOW_PARMSW;
 define alias  PMCI_ANIM_WINDOW_PARMS type  PMCI_ANIM_WINDOW_PARMSW;
 define alias LPMCI_ANIM_WINDOW_PARMS type LPMCI_ANIM_WINDOW_PARMSW;
 %end; %else %do;
 define alias   MCI_ANIM_WINDOW_PARMS type   MCI_ANIM_WINDOW_PARMSA;
 define alias  PMCI_ANIM_WINDOW_PARMS type  PMCI_ANIM_WINDOW_PARMSA;
 define alias LPMCI_ANIM_WINDOW_PARMS type LPMCI_ANIM_WINDOW_PARMSA;
 %end; /* UNICODE */

 /* parameter block for MCI_PUT, MCI_UPDATE, MCI_WHERE command */
 /* messages                                                   */
 define structure
   1 MCI_ANIM_RECT_PARMS,
     2 dwCallback type DWORD,
 %dcl MCI_USE_OFFEXT char ext;
 %if MCI_USE_OFFEXT ^= '' %then %do;
     2 ptOffset   type POINT,
     2 ptExtent   type POINT;
 %end; %else %do;
     2 rc         type RECT;
 %end; /* ifdef MCI_USE_OFFEXT */
 define alias  PMCI_ANIM_RECT_PARMS handle MCI_ANIM_RECT_PARMS;
 define alias LPMCI_ANIM_RECT_PARMS handle MCI_ANIM_RECT_PARMS;

 /* parameter block for MCI_UPDATE PARMS */
 define structure
   1 MCI_ANIM_UPDATE_PARMS,
     2 dwCallback type DWORD,
     2 rc         type RECT,
     2 hDC        type HDC;
 define alias  PMCI_ANIM_UPDATE_PARMS handle MCI_ANIM_UPDATE_PARMS;
 define alias LPMCI_ANIM_UPDATE_PARMS handle MCI_ANIM_UPDATE_PARMS;

 /* MCI extensions for video overlay devices */

 /* flags for dwFlags parameter of MCI_OPEN command message */
 dcl MCI_OVLY_OPEN_WS                fixed bin(31) value('00010000'xn);
 dcl MCI_OVLY_OPEN_PARENT            fixed bin(31) value('00020000'xn);

 /* flags for dwFlags parameter of MCI_STATUS command message */
 dcl MCI_OVLY_STATUS_HWND            fixed bin(31) value('00004001'xn);
 dcl MCI_OVLY_STATUS_STRETCH         fixed bin(31) value('00004002'xn);

 /* flags for dwFlags parameter of MCI_INFO command message */
 dcl MCI_OVLY_INFO_TEXT              fixed bin(31) value('00010000'xn);

 /* flags for dwItem field of MCI_GETDEVCAPS_PARMS parameter block */
 dcl MCI_OVLY_GETDEVCAPS_CAN_STRETCH fixed bin(31) value('00004001'xn);
 dcl MCI_OVLY_GETDEVCAPS_CAN_FREEZE  fixed bin(31) value('00004002'xn);
 dcl MCI_OVLY_GETDEVCAPS_MAX_WINDOWS fixed bin(31) value('00004003'xn);

 /* flags for dwFlags parameter of MCI_WINDOW command message */
 dcl MCI_OVLY_WINDOW_HWND            fixed bin(31) value('00010000'xn);
 dcl MCI_OVLY_WINDOW_STATE           fixed bin(31) value('00040000'xn);
 dcl MCI_OVLY_WINDOW_TEXT            fixed bin(31) value('00080000'xn);
 dcl MCI_OVLY_WINDOW_ENABLE_STRETCH  fixed bin(31) value('00100000'xn);
 dcl MCI_OVLY_WINDOW_DISABLE_STRETCH fixed bin(31) value('00200000'xn);

 /* flags for hWnd parameter of MCI_OVLY_WINDOW_PARMS parameter block */
 dcl MCI_OVLY_WINDOW_DEFAULT         fixed bin(31) value('00000000'xn);

 /* flags for dwFlags parameter of MCI_PUT command message */
 dcl MCI_OVLY_RECT                   fixed bin(31) value('00010000'xn);
 dcl MCI_OVLY_PUT_SOURCE             fixed bin(31) value('00020000'xn);
 dcl MCI_OVLY_PUT_DESTINATION        fixed bin(31) value('00040000'xn);
 dcl MCI_OVLY_PUT_FRAME              fixed bin(31) value('00080000'xn);
 dcl MCI_OVLY_PUT_VIDEO              fixed bin(31) value('00100000'xn);

 /* flags for dwFlags parameter of MCI_WHERE command message */
 dcl MCI_OVLY_WHERE_SOURCE           fixed bin(31) value('00020000'xn);
 dcl MCI_OVLY_WHERE_DESTINATION      fixed bin(31) value('00040000'xn);
 dcl MCI_OVLY_WHERE_FRAME            fixed bin(31) value('00080000'xn);
 dcl MCI_OVLY_WHERE_VIDEO            fixed bin(31) value('00100000'xn);

 /* parameter block for MCI_OPEN command message */
 define structure
   1 MCI_OVLY_OPEN_PARMSA,
     2 dwCallback       type DWORD,
     2 wDeviceID        type MCIDEVICEID,
     2 lpstrDeviceType  type LPCSTR,
     2 lpstrElementName type LPCSTR,
     2 lpstrAlias       type LPCSTR,
     2 dwStyle          type DWORD,
     2 hWndParent       type HWND;
 define alias  PMCI_OVLY_OPEN_PARMSA handle MCI_OVLY_OPEN_PARMSA;
 define alias LPMCI_OVLY_OPEN_PARMSA handle MCI_OVLY_OPEN_PARMSA;

 define structure
   1 MCI_OVLY_OPEN_PARMSW,
     2 dwCallback       type DWORD,
     2 wDeviceID        type MCIDEVICEID,
     2 lpstrDeviceType  type LPCWSTR,
     2 lpstrElementName type LPCWSTR,
     2 lpstrAlias       type LPCWSTR,
     2 dwStyle          type DWORD,
     2 hWndParent       type HWND;
 define alias  PMCI_OVLY_OPEN_PARMSW handle MCI_OVLY_OPEN_PARMSW;
 define alias LPMCI_OVLY_OPEN_PARMSW handle MCI_OVLY_OPEN_PARMSW;

 %if UNICODE ^= '' %then %do;
 define alias   MCI_OVLY_OPEN_PARMS type   MCI_OVLY_OPEN_PARMSW;
 define alias  PMCI_OVLY_OPEN_PARMS type  PMCI_OVLY_OPEN_PARMSW;
 define alias LPMCI_OVLY_OPEN_PARMS type LPMCI_OVLY_OPEN_PARMSW;
 %end; %else %do;
 define alias   MCI_OVLY_OPEN_PARMS type   MCI_OVLY_OPEN_PARMSA;
 define alias  PMCI_OVLY_OPEN_PARMS type  PMCI_OVLY_OPEN_PARMSA;
 define alias LPMCI_OVLY_OPEN_PARMS type LPMCI_OVLY_OPEN_PARMSA;
 %end; /* UNICODE */

 /* parameter block for MCI_WINDOW command message */
 define structure
   1 MCI_OVLY_WINDOW_PARMSA,
     2 dwCallback type DWORD,
     2 hWnd       type HWND,
     2 nCmdShow   type UINT,
     2 lpstrText  type LPCSTR;
 define alias  PMCI_OVLY_WINDOW_PARMSA handle MCI_OVLY_WINDOW_PARMSA;
 define alias LPMCI_OVLY_WINDOW_PARMSA handle MCI_OVLY_WINDOW_PARMSA;
 define structure
   1 MCI_OVLY_WINDOW_PARMSW,
     2 dwCallback type DWORD,
     2 hWnd       type HWND,
     2 nCmdShow   type UINT,
     2 lpstrText  type LPCWSTR;
 define alias  PMCI_OVLY_WINDOW_PARMSW handle MCI_OVLY_WINDOW_PARMSW;
 define alias LPMCI_OVLY_WINDOW_PARMSW handle MCI_OVLY_WINDOW_PARMSW;

 %if UNICODE ^= '' %then %do;
 define alias   MCI_OVLY_WINDOW_PARMS type   MCI_OVLY_WINDOW_PARMSW;
 define alias  PMCI_OVLY_WINDOW_PARMS type  PMCI_OVLY_WINDOW_PARMSW;
 define alias LPMCI_OVLY_WINDOW_PARMS type LPMCI_OVLY_WINDOW_PARMSW;
 %end; %else %do;
 define alias   MCI_OVLY_WINDOW_PARMS type   MCI_OVLY_WINDOW_PARMSA;
 define alias  PMCI_OVLY_WINDOW_PARMS type  PMCI_OVLY_WINDOW_PARMSA;
 define alias LPMCI_OVLY_WINDOW_PARMS type LPMCI_OVLY_WINDOW_PARMSA;
 %end; /* UNICODE */

 /* parameter block for MCI_PUT, MCI_UPDATE, and MCI_WHERE */
 /* command messages                                       */
 define structure
   1 MCI_OVLY_RECT_PARMS,
     2 dwCallback type DWORD,
 %dcl MCI_USE_OFFEXT char ext;
 %if MCI_USE_OFFEXT ^= '' %then %do;
     2 ptOffset   type POINT,
     2 ptExtent   type POINT;
 %end; %else %do;
     2 rc         type RECT;
 %end;/* ifdef MCI_USE_OFFEXT */
 define alias  PMCI_OVLY_RECT_PARMS handle MCI_OVLY_RECT_PARMS;
 define alias LPMCI_OVLY_RECT_PARMS handle MCI_OVLY_RECT_PARMS;

 /* parameter block for MCI_SAVE command message */
 define structure
   1 MCI_OVLY_SAVE_PARMSA,
     2 dwCallback type DWORD,
     2 lpfilename type LPCSTR,
     2 rc         type RECT;
 define alias  PMCI_OVLY_SAVE_PARMSA handle MCI_OVLY_SAVE_PARMSA;
 define alias LPMCI_OVLY_SAVE_PARMSA handle MCI_OVLY_SAVE_PARMSA;

 define structure
   1 MCI_OVLY_SAVE_PARMSW,
     2 dwCallback type DWORD,
     2 lpfilename type LPCWSTR,
     2 rc         type RECT;
 define alias PMCI_OVLY_SAVE_PARMSW  handle MCI_OVLY_SAVE_PARMSW;
 define alias LPMCI_OVLY_SAVE_PARMSW handle MCI_OVLY_SAVE_PARMSW;

 %if UNICODE ^= '' %then %do;
 define alias   MCI_OVLY_SAVE_PARMS type   MCI_OVLY_SAVE_PARMSW;
 define alias  PMCI_OVLY_SAVE_PARMS type  PMCI_OVLY_SAVE_PARMSW;
 define alias LPMCI_OVLY_SAVE_PARMS type LPMCI_OVLY_SAVE_PARMSW;
 %end; %else %do;
 define alias   MCI_OVLY_SAVE_PARMS type   MCI_OVLY_SAVE_PARMSA;
 define alias  PMCI_OVLY_SAVE_PARMS type  PMCI_OVLY_SAVE_PARMSA;
 define alias LPMCI_OVLY_SAVE_PARMS type LPMCI_OVLY_SAVE_PARMSA;
 %end; /* UNICODE */

 /* parameter block for MCI_LOAD command message */
 define structure
   1 MCI_OVLY_LOAD_PARMSA,
     2 dwCallback type DWORD,
     2 lpfilename type LPCSTR,
     2 rc         type RECT;
 define alias  PMCI_OVLY_LOAD_PARMSA handle MCI_OVLY_LOAD_PARMSA;
 define alias LPMCI_OVLY_LOAD_PARMSA handle MCI_OVLY_LOAD_PARMSA;

 define structure
   1 MCI_OVLY_LOAD_PARMSW,
     2 dwCallback type DWORD,
     2 lpfilename type LPCWSTR,
     2 rc         type RECT;
 define alias  PMCI_OVLY_LOAD_PARMSW handle MCI_OVLY_LOAD_PARMSW;
 define alias LPMCI_OVLY_LOAD_PARMSW handle MCI_OVLY_LOAD_PARMSW;

 %if UNICODE ^= '' %then %do;
 define alias   MCI_OVLY_LOAD_PARMS type   MCI_OVLY_LOAD_PARMSW;
 define alias  PMCI_OVLY_LOAD_PARMS type  PMCI_OVLY_LOAD_PARMSW;
 define alias LPMCI_OVLY_LOAD_PARMS type LPMCI_OVLY_LOAD_PARMSW;
 %end; %else %do;
 define alias   MCI_OVLY_LOAD_PARMS type   MCI_OVLY_LOAD_PARMSA;
 define alias  PMCI_OVLY_LOAD_PARMS type  PMCI_OVLY_LOAD_PARMSA;
 define alias LPMCI_OVLY_LOAD_PARMS type LPMCI_OVLY_LOAD_PARMSA;
 %end; /* UNICODE */

 %end; /* ifndef MMNOMCI */

 /*********************************************************************/
 /*                                                                   */
 /*          DISPLAY Driver extensions                                */
 /*                                                                   */
 /*********************************************************************/

 %dcl #NEWTRANSPARENT char ext;
 %if #NEWTRANSPARENT = '' %then %do;
 %#NEWTRANSPARENT = 'Y';
 dcl NEWTRANSPARENT  fixed bin(31) value(3); /* use with SetBkMode() */

 dcl QUERYROPSUPPORT fixed bin(31) value(40);
     /* use to determine ROP support */
 %end; /* ifndef NEWTRANSPARENT */

 /*********************************************************************/
 /*                                                                   */
 /*          DIB Driver extensions                                    */
 /*                                                                   */
 /*********************************************************************/

 dcl SELECTDIB fixed bin(31) value(41); /* DIB.DRV select dib escape */

 %DIBINDEX: proc (n) returns (char);
   dcl n char;
   return ('MAKELONG (' || n || ', ''10ff''xn)');
 %end DIBINDEX;
 %activate DIBINDEX;

 /*********************************************************************/
 /*                                                                   */
 /*          ScreenSaver support - is in winuser                      */
 /*                                                                   */
 /*********************************************************************/

 %end; /* _INC_MMSYSTEM */
